
from basics import *
from pgplot import *
from scipy.special import *
from Multipack import *
from masks import *
from Cheetah.Template import Template 
import Image
from ephem import *

ChipCorners = [(700,1200),(1024,64),(1024,64),(1348,1200),(1024,64),(1348,1200),(700,1200),(1024,64)]

class Dataset:
   def __init__(self,fitsfile,cut=5,foc_scale=1.0,com_scale=1.0,ast_scale=1.0,np=0,native=0,angles=None,rmax=12.0):
       self.fitsfile = fitsfile
       self.native = native
       self.foc_scale=foc_scale
       self.com_scale=com_scale
       self.ast_scale=ast_scale
       self.name = os.path.basename(fitsfile).split(".")[0][:7]
       self.Read()
       self.Cull(cut=2*cut,np=np)
       self.Cull(cut=cut,np=np)
       self.FITSData()
       self.ReadDistortion(native=native,angles=angles)

       self.ToSky(["x1","y1"],["u1","v1"])
       self.ToSky(["x2","y2"],["u2","v2"])
       self.ToSky(["corner_x","corner_y"],["Cu","Cv"])
       self.table["t1"] = self.table["t1"] - self.pa
       self.table["t2"] = self.table["t2"] - self.pa

       self.table["rp2"], self.table["tp2"] = ToPolar( [self.table["u2"], self.table["v2"]])
       self.table["rp1"], self.table["tp1"] = ToPolar( [self.table["u1"], self.table["v1"]])
       self.table["zeros"] = zeros(self.table["tp1"].shape,Float32)
       self.table["ucoma"] = (self.table["u1"] - self.table["u2"]) * self.com_scale
       self.table["vcoma"] = (self.table["v1"] - self.table["v2"]) * self.com_scale
       self.RadialCut(rmax)

       if 0: self.table["a2"] = self.table["b2"] = self.table["r2"] = self.table["p"]/50000.0

   def Read(self):
       data = self.fitsfile.replace(".fits",".dnt")
       self.table = readtable(data,header=0,icols=[0])
       self.table["a2"], self.table["b2"] = self.foc_scale*self.table["a2"], self.foc_scale*self.table["b2"]
       self.table["r2"] = sqrt( self.table["a2"] * self.table["b2"])
       b2,a2 = extrema( [self.table["b2"], self.table["a2"]])
       b1,a1 = extrema( [self.table["b1"], self.table["a1"]])
       self.table["t2"] = self.table["t2"] + 90.0*greater(self.table["b2"],self.table["a2"])
       self.table["t1"] = self.table["t1"] + 90.0*greater(self.table["b1"],self.table["a1"])
       self.table["t2"] = self.table["t2"] + 360 * less(self.table["t2"],0)
       self.table["t1"] = self.table["t1"] + 360 * less(self.table["t1"],0)
       self.table["b2"], self.table["a2"] = b2,a2
       self.table["b1"], self.table["a1"] = b1,a1
   def Ellipse(self,a="a2",b="b2",dmag=1,emag=1,suf="",smag=10):
       p = arange(-1,1.05,0.1,Float32)
       e = self.table[b] / self.table[a]
       e = sqrt(1-(1-e**2)*emag)
       yl = dmag * p[NewAxis,::]*(e*self.table[a])[::,NewAxis]
       xl = dmag * sqrt(1-power(p,2))[NewAxis,::]*self.table[a][::,NewAxis]
       yr = yl
       xr = -xl
       self.table["ct2"+suf], self.table["st2"+suf] = [fn(self.table["t2"+suf]*pi/180)[::,NewAxis] for fn in [cos,sin]]
       self.table["xr"+suf],self.table["yr"+suf] = self.table["x2"][::,NewAxis] + xr*self.table["ct2"+suf] + yr*self.table["st2"+suf], self.table["y2"][::,NewAxis] -  xr*self.table["st2"+suf] + yr*self.table["ct2"+suf]
       self.table["xl"+suf],self.table["yl"+suf] = self.table["x2"][::,NewAxis] + xl*self.table["ct2"+suf] + yl*self.table["st2"+suf], self.table["y2"][::,NewAxis] -  xl*self.table["st2"+suf] + yl*self.table["ct2"+suf]
       self.ToSky(["xr"+suf,"yr"+suf],["ur"+suf,"vr"+suf])
       self.ToSky(["xl"+suf,"yl"+suf],["ul"+suf,"vl"+suf])
       if suf:
          self.table["ur"+suf] = (self.table["ur"] - self.table["ur"+suf])
          self.table["ul"+suf] = (self.table["ul"] - self.table["ul"+suf])
          self.table["vr"+suf] = (self.table["vr"] - self.table["vr"+suf])
          self.table["vl"+suf] = (self.table["vl"] - self.table["vl"+suf])
          self.table["ur"+suf] = smag*self.table["ur"+suf] + self.table["u2"][::,NewAxis]
          self.table["ul"+suf] = smag*self.table["ul"+suf] + self.table["u2"][::,NewAxis]
          self.table["vr"+suf] = smag*self.table["vr"+suf] + self.table["v2"][::,NewAxis]
          self.table["vl"+suf] = smag*self.table["vl"+suf] + self.table["v2"][::,NewAxis]
   def Orient(self,amag=1):
       t,a,b,r = "t2","a2","b2","r2"
       o = (self.table[a] - self.table[b])
       ut = (self.table[t] + self.pa) * pi/180
       self.table["c"+t], self.table["s"+t] = [fn(ut) for fn in [cos,sin]]
       x1, y1 = o * self.table["c"+t],  o * self.table["s"+t]
       self.table["ax1"],self.table["ay1"] = self.table["x2"] + x1, self.table["y2"] +  y1
       self.ToSky(["ax1","ay1"],["au1","av1"])
       self.table["uast"] = (self.table["au1"] - self.table["u2"]) * self.ast_scale
       self.table["vast"] = (self.table["av1"] - self.table["v2"]) * self.ast_scale
       aast = hypot( self.table["uast"], self.table["vast"])
       tast = arctan2( self.table["vast"], self.table["uast"])
#       tast = remainder(arctan2( self.table["vast"], self.table["uast"])-self.pa*pi/180,pi)
       ut2 = 2*tast + self.pa*pi/180
       self.table["c2"+t], self.table["s2"+t] = [fn(ut2) for fn in [cos,sin]]
       x2, y2 = aast * self.table["c2"+t], aast * self.table["s2"+t]
       self.table["u2ast"] = x2
       self.table["v2ast"] = y2

#       x2, y2 = o * self.table["c2"+t], o * self.table["s2"+t]
#       self.table["ax2"],self.table["ay2"] = self.table["x2"] + x2, self.table["y2"] +  y2
#       self.ToSky(["ax2","ay2"],["au2","av2"])
#       self.table["u2ast"] = (self.table["au2"] - self.table["u2"]) * self.ast_scale
#       self.table["v2ast"] = (self.table["av2"] - self.table["v2"]) * self.ast_scale
   def RadialCut(self,rmax):
       d = sqrt(self.table["u1"]**2 + self.table["v1"]**2)
       use = less(d,rmax)
       for k in self.table:
          if self.table[k].shape == use.shape:
             self.table[k] = compress(use,self.table[k])
   def Cull(self,cut=5,np=0):
       p = self.table["p"]
       dx = self.table["x2"] - self.table["x1"]
       dy = self.table["y2"] - self.table["y1"]
       e2  = self.table["b2"] / self.table["a2"]
       r2  = sqrt(self.table["b2"]**2 + self.table["a2"]**2)
       xs0,xs1 = bwt(dx)
       ys0,ys1 = bwt(dy)
       es0,es1 = bwt(e2)
       rs0,rs1 = bwt(r2)
       print "%s %6.2f %5.2f %6.2f %5.2f %6.2f %5.2f %.4f %.4f" % (self.fitsfile,xs0,xs1,ys0,ys1,rs0,rs1,es0,es1),
       usex = 1.0*less(abs(dx-xs0),cut*xs1)
       usey = 1.0*less(abs(dy-ys0),cut*ys1)
       user = 1.0*less(abs(r2-rs0),cut*rs1)
       usee = 1.0*less(abs(e2-es0),cut*es1)
       use = usex*usey*user*usee
       if np < len(use):
         pmin = sort(p)[-np]
         usep = 1.0*greater_equal(self.table["p"],pmin)
         use = use * usep
       print "%3d %3d" % (sum(use),len(use))
       for k in self.table:
          self.table[k] = compress(use,self.table[k])
   def FITSData(self):
       f = FITS(self.fitsfile)
       self.nx,self.ny = f.naxes()
       self.night = f["night"]
       self.utdate = f["ut-date"]
       self.uttime = f["ut-time"]
       self.lctime = f["lc-time"]
       self.epoch = f["epoch"]
       self.ra = f["ra"]
       self.dec = f["dec"]
       self.chip = int(f["chip"])
       self.choffx = degrad(float(f["choffx"])/3600.0)
       self.choffy = degrad(float(f["choffy"])/3600.0)
       self.binning = f["binning"]
       self.rotator = f["rotangle"]
       self.orient = f["dewarori"]
       self.filter = f["filter"]
       self.scale = f["scale"]
       f.close()
       self.s1,self.s2 = map(int,self.binning.lower().split("x"))
       self.table["corner_x"] = array( [ChipCorners[self.chip-1][0]],Float)
       self.table["corner_y"] = array( [ChipCorners[self.chip-1][1]],Float)
   def ReadDistortion(self,native=0,angles=None):
       SFILE = join([os.environ["PYTHONHOME"],"Database","pal5_f_c%d.soln" % (self.chip)],"/")
       self.S = readsoln(SFILE)
       self.S["c"] = self.S["c"]

       if angles:
          self.altaz = angles.altaz
          self.skypa = angles.skypa
          self.pa = angles.pa
       else:
          self.altaz = ComputeAltAz(self,angle=1)
          self.skypa = -(self.rotator - 223.53)
          if self.orient == "Nod&Shuffle": self.skypa = self.skypa - 90
          if native: self.pa = 0.0
          else: self.pa = self.skypa + self.altaz

       xtran = self.S["c"][::2] * (self.scale/0.2)
       ytran = self.S["c"][1::2] * (self.scale/0.2)
       cpa, spa = cos(self.pa*pi/180), sin(self.pa*pi/180)
       xtran, ytran = -xtran*cpa + ytran*spa, -ytran*cpa - xtran*spa
       xtran[1:], ytran[1:] = xtran[1:]/self.s1, ytran[1:]/self.s2
       put(self.S["c"],arange(0,self.S["c"].shape[0],2,Int32),xtran)
       put(self.S["c"],arange(1,self.S["c"].shape[0],2,Int32),ytran)

       self.rra = hrrad(scanSexagesimal(self.ra))
       self.rdec = degrad(scanSexagesimal(self.dec))

   def ToSky(self,keysin,keysout):
       self.table[keysout[0]], self.table[keysout[1]] = \
          mbasis(self.S["c"],self.table[keysin[0]]-self.S["center"][0],self.table[keysin[1]]-self.S["center"][1])
       for key in keysout: self.table[key] = self.table[key] * 60*180/pi

def FieldBox(FOV,native,ch=2.2):
  if native:
     newbox(FOV,FOV,ch=ch)
  else:
     newbox(-FOV,FOV,ch=ch)

def LabelField(title,dataset,FOV,native,ch=2.0,ci=1,lw=2):
  title = title % (dataset[0].name)
  if native:
     x,y  = "X (arcmin)", "Y (arcmin)"
  else:
     x,y = "Azimuth (arcmin)", "Altitude (arcmin)"
  xlabel(x,ch=ch,ci=ci,lw=lw)
  ylabel(y,ch=ch,ci=ci,lw=lw)
  subtitle(title,ch=ch,ci=ci,lw=lw)
  LabelChips(dataset,FOV)

def LabelChips(showset,FOV,ch=1.7,ci=7,lw=2):
   [drawptl(clip(d.table["Cu"]-1,FOV[0]+1,FOV[1]-2),clip(d.table["Cv"]-1,FOV[0]+1,FOV[1]-2),[d.chip],ch=ch,ci=ci,lw=lw) for d in showset]

def ComputeAltAz(dataset,angle=0):
   space=Circumstance()
   space.latitude=degrad(-29.01423)
   space.longitude=degrad(-70.69242)
   space.elevation=(EarthRadius+2400.0)/EarthRadius
   space.temperature=10.0
   space.pressure=615.0
   space.epoch=EOD
   year,month,day = dataset.utdate.split("-")
   day = int(day) + scanSexagesimal(dataset.uttime)/24.0
   mjd = fromGregorian(int(month),day,int(year))
   space.mjd = mjd

   target=Obj()
   target.any.type=FIXED
   target.f.name = "donut field"
   target.f.dec = degrad(scanSexagesimal(dataset.dec))
   target.f.ra = hrrad(scanSexagesimal(dataset.ra))
   target.f.epoch = dataset.epoch
   computeLocation( space, target)
   lst = computeSiderealTime(space)
   ha = radhr(target.any.ra)-lst

   north=Obj()
   north.any.type=FIXED
   north.f.name = "donut field"
   north.f.ra = target.f.ra
   north.f.dec = target.f.dec + pi/180/3600
   north.f.epoch = dataset.epoch
   computeLocation( space, north)

   east=Obj()
   east.any.type=FIXED
   east.f.name = "donut field"
   east.f.ra = target.f.ra + pi/180/3600
   east.f.dec = target.f.dec
   east.f.epoch = dataset.epoch
   computeLocation( space, east)

   alt = array([obj.any.altitude for obj in [target,north]])
   az = array([obj.any.azimuth for obj in [target,north]])
   rs = array([obj.f.ra for obj in [target,north]])
   ds = array([obj.f.dec for obj in [target,north]])
   dalt = alt-alt[0]
   daz = (az-az[0])*cos(alt[0])
   dds = ds-ds[0]
   drs = (rs-rs[0])*cos(ds[0])
   bases = abasis(0,drs,dds,rot=1)
   co = svdfit(bases,concatenate([daz,dalt]))
   S = raddeg(arctan2(co[3],co[2]))
   return S

def ToPolar((x,y)): return  hypot(x,y), arctan2(y,x)

def FromPolar((r,t)): return [r*sc(t) for sc in [cos,sin]]

def polynom(x,nl):
  pbase=zeros((nl+1,x.shape[0]),Float64)
  pbase[0] = ones(x.shape,Float64)
  for i in range(nl): pbase[i+1] = power(x,i+1)
  return pbase

def bessels(x,nl):
  bbase=zeros((nl+1,x.shape[0]),Float64)
  for i in range(nl+1): bbase[i] = jn(i,i)
  return bbase

def getharm(x,nharm):
  hbase=zeros((2*nharm+1,x.shape[0]),Float64)
  hbase[0] = ones(x.shape,Float64)
  for i in range(1,nharm+1):
     hbase[2*i-1] = cos(i*x)
     hbase[2*i] = sin(i*x)
  return hbase

def getwordharm(nharm):
  hbase = ["1",]
  for i in range(1,nharm+1):
     hbase.append("cos(%dx)" % (i))
     hbase.append("sin(%dx)" % (i))
  return hbase


def GetBases(u1,v1,ro,to,polar=0,pr=0,u0=None,v0=None):
   if polar:
      r,t = ToPolar([u1,v1])
      r = r/10.0
      rbs = [power(r,ro1) for ro1 in ro]
      hbs = getharm(t,to)
      pbases = transpose([rb*hb for hb in hbs for rb in rbs])
      return pbases
   else:
      if sum(v1):
         u1=u1/10.0
         v1=v1/10.0
         if pr: print ["u1^%d v1^%d" % (ro[i],ro[j]) for i in range(len(ro)) for j in range(len(ro)-i)]
         ubases = [power(u1,ro1) for ro1 in ro]
         vbases = [power(v1,ro1) for ro1 in ro]
         bases = transpose([ubases[i]*vbases[j] for i in range(len(ubases)) for j in range(len(vbases)-i)])
      else:
         bases = transpose([power(u1/10.0,ro1) for ro1 in ro])
      return bases

def GetWordBases(u1,v1,ro,to,polar=0,pr=0,u0=None,v0=None):
   if polar:
      rbs = ["(R/10')" + ((ro1>1 and "<sup>%d</sup>" % (ro1)) or "") for ro1 in ro]
      if not ro[0]: rbs[0] = ""
      hbs = getwordharm(to)
      hbs[0] = ""
      pbases = ["%s %s" % (rb,hb) for hb in hbs for rb in rbs]
      return pbases
   else:
      if sum(v1):
         ubases = ["[X/10']"+ ((ro1>1 and "<sup>%d</sup>" % (ro1)) or "") for ro1 in ro]
         ubases[0] = ""
         vbases = ["[Y/10']"+ ((ro1>1 and "<sup>%d</sup>" % (ro1)) or "") for ro1 in ro]
         vbases[0] = ""
         bases = ["%s %s" % (ubases[i],vbases[j]) for i in range(len(ubases)) for j in range(len(vbases)-i)]
      else:
         print ro
         rbs = ["(R/10')" + ((ro1>1 and "<sup>%d</sup>" % (ro1)) or "") for ro1 in ro]
         print rbs
      return bases


def UseCenter(center,coords,fns,fitting=1,ro=[0],to=0,polar=0,sep=0,radial=0,verb=0):
   ucen,vcen = center
   if len(coords) == 2:
      (u0,v0) = coords
      u0,v0 = u0-ucen,v0-vcen
      u1,v1 = u0-ucen,v0-vcen
   else:
      u1 = coords[0]
      v1 = zeros(u1.shape,Float32)
   if polar:
      u1,v1 = ToPolar([u1,v1])
      if len(fns) == 2:
         ufns = asarray(ToPolar(fns))
         if radial == 0:
            ufns[1] = (ufns[1] - v1)
         else:
            ufns = ufns[:1]
#         if not radial: ufns[1] = ufns[1] + pi/2
      else:
         ufns = fns
   else:
      ufns = fns
   if sep:
      bases = GetBases(u1,v1,ro,0,polar)
      sols = [svdfit(bases,ufns[0])]
      fits = [dot(bases,sols[0])]
      if to==-1:
         sols += [[0.0]]
         fits += [zeros(u1.shape,Float32)]
      elif to==-2:
         sols += [[pi/2]]
         fits += [pi/2*ones(u1.shape,Float32)]
      elif ufns[1:]:
         bases = GetBases(u1,v1,[0],to,polar)
         sols += [svdfit(bases,ufns[1])]
         fits += [dot(bases,sols[1])]
   else:
      bases = GetBases(u1,v1,ro,to,polar,pr=0,u0=u0,v0=v0)
      sols = [svdfit(bases,fn) for fn in ufns]
      fits = [dot(bases,sol) for sol in sols]
   if polar:
#      if not radial: fits[1] = fits[1] - pi/2
      if len(fns) == 2:
        if radial == 0:
           fits[1] = fits[1] + v1
        else:
           fits += [v1]
        fits = FromPolar(fits)
   if polar == 2:
      fu1,fv1 = fits
      ou1,ov1 = fns
      fh = hypot(fv1,fu1)
      oh = hypot(ov1,ou1)
      ft = arctan2(fv1,fu1)
      ot = arctan2(ov1,ou1)
      fu2,fv2 = fh*cos(ft*2),fh*sin(ft*2)
      ou2,ov2 = oh*cos(ot*2),oh*sin(ot*2)
      ru2 = fu2-ou2
      rv2 = fv2-ov2
      rh = hypot(rv2,ru2)
      rt = arctan2(rv2,ru2)
      ru,rv = rh*cos(rt/2),rh*sin(rt/2)
      resid = concatenate([ru,rv])
   else:
      resid = concatenate(fits)-concatenate(fns)
   if fitting:
      if verb: print ucen,vcen,1.49*median(abs(resid)),sols[0]
      return resid
   else:
#      print sols[0]
      scatter = sqrt(mean(resid**2))
#      print rms(resid),
#      print 1.49*median(abs(resid))
      return sols,scatter

def fitted(d,keys,sols,center,(ro,to),polar=1,sep=0,radial=0,u="u2",v="v2"):
   ucen,vcen = center
   u0,v0 = d.table[u]-ucen, d.table[v]-vcen
   u1,v1 = d.table[u]-ucen, d.table[v]-vcen
   if polar:
      u1,v1 = ToPolar([u1,v1])
   if sep:
      bases = GetBases(u1,v1,ro,0,polar)
      fits = [dot(bases,sols[0])]
      if to==-1:
         fits += [zeros(u1.shape,Float32)]
      elif to==-2:
         fits += [pi/2*ones(u1.shape,Float32)]
      elif sols[1:]:
         bases = GetBases(u1,v1,[0],to,polar)
         fits += [dot(bases,sols[1])]
   else:
      bases = GetBases(u1,v1,ro,to,polar,pr=0,u0=u0,v0=v0)
      fits = [dot(bases,sol) for sol in sols]
   if polar:
      if polar > 0:
#         if not radial: fits[1] = fits[1] - pi/2
         if radial == 0: fits[1] = fits[1] + v1
         else: fits += [v1]
         fits = FromPolar(fits)
   for j,f in enumerate(fits): d.table[keys[j]] = f

def worddot(bases,sol):
   words = ["%.3f %s" % (sol[j],bases[j]) for j in range(len(sol))]
   line = words[0]
   for j in range(1,len(sol)):
     if sol[j] >=0: s = "+"
     else: s = "-"
     line = line + " %s %.3f %s" % (s,abs(sol[j]),bases[j])
   return line

def wordfitted(d,keys,sols,center,(ro,to),polar=1,sep=0,radial=0,u="u2",v="v2"):
   ucen,vcen = center
   u0,v0 = d.table[u]-ucen, d.table[v]-vcen
   u1,v1 = d.table[u]-ucen, d.table[v]-vcen
   if polar:
      u1,v1 = ToPolar([u1,v1])
   if sep:
      bases = GetWordBases(u1,v1,ro,0,polar)
      fits = [worddot(bases,sols[0])]
      if to==-1:
         fits += [zeros(u1.shape,Float32)]
      elif to==-2:
         fits += [pi/2*ones(u1.shape,Float32)]
      elif sols[1:]:
         bases = GetWordBases(u1,v1,[0],to,polar)
         fits += [worddot(bases,sols[1])]
   else:
      bases = GetWordBases(u1,v1,ro,to,polar,pr=0,u0=u0,v0=v0)
      fits = [worddot(bases,sol) for sol in sols]
   return fits

def FocusRescale(dataset,r0,rmag,pre="s"):
   for d in dataset:
       if type(r0) == FloatType:
         frat = rmag*(d.table["r2"]-r0)/d.table["r2"]
       elif type(r0) == StringType:
         frat = rmag*(d.table["r2"]-d.table[r0])/d.table["r2"]
       d.table[pre+"ur"] = (frat)[::,NewAxis]*(d.table["ur"]-d.table["u2"][::,NewAxis]) + d.table["u2"][::,NewAxis]
       d.table[pre+"vr"] = (frat)[::,NewAxis]*(d.table["vr"]-d.table["v2"][::,NewAxis]) + d.table["v2"][::,NewAxis]
       d.table[pre+"ul"] = (frat)[::,NewAxis]*(d.table["ul"]-d.table["u2"][::,NewAxis]) + d.table["u2"][::,NewAxis]
       d.table[pre+"vl"] = (frat)[::,NewAxis]*(d.table["vl"]-d.table["v2"][::,NewAxis]) + d.table["v2"][::,NewAxis]

def FocusFit(dataset,r0,smag,pre="r",offset=0,simplef=0,p_=""):
   for d in dataset:
       if simplef:
          f = d.table["fr2"+p_]
       else:
          f = d.table["fsr2"+p_]
       if pre == "f":
          frat = smag*(f-r0)/d.table["r2"+p_]
       else:
          frat = smag*(d.table["r2"+p_]-f-r0)/d.table["r2"+p_]
       d.table[pre+"ur"+p_] = (frat)[::,NewAxis]*(d.table["ur"]-d.table["u2"][::,NewAxis]) + d.table["u2"][::,NewAxis]
       d.table[pre+"vr"+p_] = (frat)[::,NewAxis]*(d.table["vr"]-d.table["v2"][::,NewAxis]) + d.table["v2"][::,NewAxis]
       d.table[pre+"ul"+p_] = (frat)[::,NewAxis]*(d.table["ul"]-d.table["u2"][::,NewAxis]) + d.table["u2"][::,NewAxis]
       d.table[pre+"vl"+p_] = (frat)[::,NewAxis]*(d.table["vl"]-d.table["v2"][::,NewAxis]) + d.table["v2"][::,NewAxis]

def MakeCircle(x0,y0,a,e):
    p = arange(-1,1.05,0.1,Float32)
    yl = p*(e*a)
    xl = sqrt(1-power(p,2))*a
    yr = yl
    xr = -xl
    circle = {"ul":xl+x0,"vl":yl+y0,"ur":xr+x0,"vr":yr+y0}
    return circle

def InsertImage(jpg,kind,indent=5*" ",caption=0,imfac=3):
    cwd = "./"
    image = Image.open(jpg)
    width,height=add(divide(image.size,imfac),[2,2])
    bjpg = os.path.basename(jpg)
    if caption:
       imagebody = indent + "<th>%s=%s</th>" % (kind,jpg.replace(".jpg",""))
    else:
       imagebody = indent + "<td><a href='%s'><image height='%d' width='%d' src='%s%s'></a></td>\n" % (bjpg,height,width,cwd,bjpg)
    return imagebody

def InsertResults(name,result,indent=5*" "):
    head = indent + "<p><table border=1>\n"
    tail= indent + "</table></p>\n"
    body = indent + "<tr><td>%s</td><td>%f</td><td>%f</td></tr>\n" % (name, result[0], result[1])
    table = head + body + tail
    return table 


def WriteWebPage(finfo,html,ref,obs,presults,sresults,imfac=2.2):
    H = []
    if ref:
       jpg1 = ref.replace(".fits",".jpg").replace("?","").replace("*","")
       image = Image.open(jpg1)
       width1,height1=add(divide(image.size,imfac),[2,2])
       H.append( "<tr><th>Aberration</th><th>RMS<sub>0</sub>(\")</th><th>RMS(\")</th><th>Xcen(')</th><th>Ycen(')</th><th>Functional Form(\")</th></tr>\n")
    if obs:
       jpg2 = obs.replace(".fits",".jpg").replace("?","").replace("*","")
       image = Image.open(jpg2)
       width2,height2=add(divide(image.size,imfac),[2,2])
       H.append( "<tr><th>Aberration</th><th>RMS<sub>0</sub>(\")</th><th>RMS(\")</th><th>Functional Form(\")</th><th>Angle(deg)</th></tr>\n")
    indent = 5*" "
    templateDef = """
    <html>
    <head><title>$title</title></head>
    <body>
      $info
      $whatever
    </body>
    </html>\n"""
    t = Template(templateDef,searchList=[])
    if ref and obs: word = "%s and %s" % (ref,obs)
    elif ref: word = ref
    elif obs: word = obs
    t.title = "Analysis of focus, coma, and astigmatism in %s" % (word)
    t.info = "<h3><p><b>Analysis of focus, coma, and astigmatism in %s </b><br>\n" % (word)
    t.info += indent + "<h4>[<i>Markers have full width of 1\"</i>]<br><hr></p>\n"

    fits = []

    uresults = [presults]
    if sresults: uresults += [ sresults]

    for R,results in enumerate(uresults):
        fits1 = "\n"+indent + "<p><table border=1 width='%d'>\n" % (width1)
        fits1 += indent + H[R]
        keys = ["focus","focus2","coma","ast","ast2"]
        terms = ["Focus (Tilt)", "Focus (R)", "Coma", "Astigmatism","Astigmatism"]
        for key in keys:
          if key in results:
             ncomp = len(results[key][4])
             polar = results[key][5]
             for jresult in range(ncomp):
                 form = results[key][4][jresult]
                 if "focus" in key: comp = ""
                 elif ncomp == 1:
                    comp = "(R)"
                 elif polar:
                    if ncomp == 2 and jresult == 0: comp = "(R)"
                    elif ncomp == 2 and jresult == 1: comp = "(Th)"
                 else:
                    if "ast" in key:
                       if ncomp == 2 and jresult == 0: comp = "[cos (2theta)]"
                       elif ncomp == 2 and jresult == 1: comp = "[sin (2theta)]"
                    else:
                       if ncomp == 2 and jresult == 0: comp = "(X)"
                       elif ncomp == 2 and jresult == 1: comp = "(Y)"
                 which = terms[keys.index(key)]
                 if comp: which = "%s %s" % (which,comp)
                 xcen,ycen = results[key][0], results[key][1],
                 if xcen == ycen == 0.0: xcen, ycen = "---", "---"
                 else: xcen,ycen = "%6.2f" % (xcen), "%6.2f" % (ycen)
                 if "Angle" in H[R]:
                   if key == "focus": angle = "%6.2f" % (results[key][6])
                   else: angle = "---"
                   fits1 += indent + "<tr><td>%s</td><td>%6.3f</td><td>%6.3f</td><td>%s</td><td>%6s</td></tr>\n" % \
                    (which, results[key][2], results[key][3], form, angle)
                 else:
                   fits1 += indent + "<tr><td>%s</td><td>%6.3f</td><td>%6.3f</td><td>%s</td><td>%s</td><td>%s</td></tr>\n" % \
                    (which, results[key][2], results[key][3], xcen, ycen, form)
        fits1 += indent + "</table></p>\n"
        fits.append(fits1)

    t.whatever = indent + "<p><table border=1>"
    t.whatever += "<tr>"
    if ref: t.whatever += InsertImage(jpg1,"reference",caption=1,imfac=imfac)
    if obs: t.whatever += InsertImage(jpg2,"observation",caption=1,imfac=imfac)
    t.whatever += "</tr>"
    t.whatever += "<tr>"
    for fits1 in fits: t.whatever += "<td>%s</td>\n" % (fits1)
    t.whatever += "</tr>"
    t.whatever += "<tr>"
    if ref: t.whatever += InsertImage(jpg1,"reference",imfac=imfac)
    if obs: t.whatever += InsertImage(jpg2,"observation",imfac=imfac)
    t.whatever += "</tr>"
    t.whatever += indent + "</table></p>"
    print "Writing %s" % (html)
    htout = open(html,"w")
    htout.write(t.respond())
    htout.close()


