
import time
from glob import glob
from sys import argv,exit,stdout
from VTKHelperFunctions import *
from MLab import *
from basics import *
from FITS import *
from FFT import *
from Multipack import *
from LinearAlgebra import *
from LineIdentify import *

def solvells(b,y): return svdfit(b,y)

def wcalib(slitno,spectrum,disco,order=3,fwhm=5.0,w1=3000.0,w2=11000.0,arc=0,skyamp=0,nim=1,\
        tele="N/A",lamps="0,0,0,0",gran="600gmm",dich=1,redsid=1,\
        ll="skylines.dat",d0=1.13,wfft=0,pgapop=0,pgamit=0,bth=8.0,wzero=0,wtol=1.0,red=0,verb=0):
    if arc:
       ltype="arc"; cut=3.0
       if arc==2: ltype="amp"; cut = 3.5
    if skyamp: ltype="amp";cut=3.5
    elif not arc: ltype="sky"; cut=2.0
    disp = d0
    thresh = bth
    amp = 10
    if w1==w2: w1=3000.0;w2=11000.0
    tempfits = "temp_%d.fits" % (os.getpid())
    temp2fits = "temp2_%d.fits" % (os.getpid())
    disco,scat,wspec = wavecalib(spectrum,order,disco,w1,w2,fwhm,thresh,disp,\
        dich,cut,amp,ll,ltype,wzero,wfft,pgapop,pgamit,verb,\
        tempfits=tempfits,temp2fits=temp2fits)
    err = 0
    return err,disco,scat,wspec

def fdump(fitsfile,data):
   na=list(data.shape)
   na.reverse()
   nx=len(na)
   print "creating",fitsfile,nx,na
   GGG=FITS(fitsfile,create=1,readonly=0)
   GGG.newhead("T",-32,nx,na)
   GGG.putdata(data)
   GGG.close()

def rmsol(image,keyroot):
   f=FITS(image,readonly=0)
   if f[keyroot+"n"] != "N/A":
     oldnpy,oldnupy,oldnt,oldxord,oldyord=eval(f[keyroot+"n"])
     f.delkey(keyroot+"n")
     f.delkey(keyroot+"r")
     for j in range(oldnt): f.delkey(keyroot+str(j))
   f.close()

def getslitinfo(image):
   f=FITS(image,readonly=1)
   if f["nslits"] != "N/A":
        nslits=int(f["nslits"])
        ca=[]; cb=[]
        for j in range(nslits):
         ca.append( int(f["csect"+str(j+1)+"a"])-1)
         cb.append( int(f["csect"+str(j+1)+"b"]))
   else:
     ca = [0]
     cb = [f.naxes()[1]]
   f.close()
   return ca, cb

def readsol(image,keyroot):
     f=FITS(image)
     efn = eval(f[keyroot+"n"])
     zord = -1
     if len(efn) == 5: npy,nupy,nt,xord,yord=efn
     elif len(efn) == 6: npy,nupy,nt,xord,yord,zord=efn
     if f[keyroot+"r"] != "N/A": rms=f[keyroot+"r"]
     else: rms=0.0
     sol=[]
     for j in range(nt): sol+=eval(f[keyroot+str(j)])
     f.close()
     coef = zeros((len(sol),),Float64)
     if sol[0] != "N/A":
        for j in range(len(sol)): coef[j] = sol[j]
     if zord == -1: return npy,nupy,[coef,rms],xord,yord
     else: return npy,nupy,[coef,rms],xord,yord,zord

def writesol(npy,nupy,sol,f,keyroot,xord=4,yord=4,verb=0,zord=0):
   sol,rms = sol
   nr = len(sol) % 2; ns = len(sol) / 2
   hs=[]
   for j in range(ns): hs.append(str(sol[j*2:(j+1)*2].tolist()))
   if nr: hs.append(str(sol[ns*2:].tolist()))
   lockfile = "%s.lock" % ( f.name())
   while os.path.exists(lockfile):
      st = 1.0+random(1)[0]
      print "Waiting %.4fs for %s to be unlocked." % (st,f.name())
      time.sleep(st)
   lockout = open(lockfile,"w")
   if zord: f.modkey(keyroot+"n",str([npy,nupy,nr+ns,xord,yord,zord]))
   else: f.modkey(keyroot+"n",str([npy,nupy,nr+ns,xord,yord]))
   f.modkey(keyroot+"r",rms)
   if verb: print keyroot+"n",f[keyroot+"n"]
   if verb: print keyroot+"r",f[keyroot+"r"]
   for j in range(ns+nr):
       f.modkey(keyroot+str(j),hs[j])
       if verb: print keyroot+str(j),hs[j]
   f.flush()
   lockout.close()
   try: os.remove(lockfile)
   except: pass

def specfilter(s,nbg):
   if nbg > 0:
      s = s.astype(Float32)
      f = VTKMedian(s,2*(nbg/2)+1,1,numret=1)
      f = VTKShrinkN(f,nbg,1,usemed=1,numret=1)
      f = VTKErode(f,nbg,1,numret=1)
      f = VTKGrowN(f,nbg,1,interp=1,origsize=s.shape,numret=1)
      f = s - f
#      f = s - funcfilt(getpct,s,nbg,0,arg=pct)
#      o = divz(1,s).astype(Float32)
#      o2 = power(o,2).astype(Float32)
#      f = s - divz(VTKLevel(o,nbg,k=5),VTKLevel(o2,nbg,k=5))
      return f
   else: return s

def crosscor(xin,yin,shift=1):
   n=len(xin)
   n2 = 2**(int(log2(n-1))+2)
   n2f = n2/2+1
   x=zeros((n2),xin.typecode()); y=zeros((n2),yin.typecode())
   x[n:] = (xin[-1]+x[n:]).astype(x.typecode())
   y[n:] = (yin[-1]+y[n:]).astype(y.typecode())
   x[:n] = xin; y[:n] = yin
   x = x - getpct(xin,10)
   y = y - getpct(yin,10)
   i = arange(n2)
##   v = ones((n2,),Float64)
   v = zeros((n2,),Float64)
#   v[:n] = (exp(-0.5*(i[:n]-n/2.0)**2/(n/2.0)**2)-exp(-0.5))/(1-exp(-0.5))
#   v = power(v,4)
   v[:n] = hanning(n)
   fc = fft(v*x)*conjugate(fft(v*y))
#   fc = real_fft(x)*conjugate(real_fft(y))
#   w = 2*pi*arange(n2f)/n2
#   if sig > 0:
#      expon=-(0.5*sig**2)*power(w,2)
#      fc = fc * exp(2*expon*less(abs(expon),100))
   c = inverse_fft(fc).real
   if shift: c=take(c,range(-n2/2,n2/2))
#   if x != y:
#     qdump("a.fits",x)
#     qdump("b.fits",y)
#     qdump("c.fits",c);exit()
   return c

def peak(e,g=None,pkwin=10):
   n2=len(e)
   xe = arange(n2)
   if g == None: gnew=argmax(e[n2/3:2*n2/3])+(n2/3)
   else:
       gnew=int(g)
   el = max([pkwin,2])
   eh = el+1
   g0 = max((gnew-el,0))
   g1 = min((gnew+eh,len(e)))
   x = xe[g0:g1]
   y = e[g0:g1]
   tck = splrep(x,y,s=0,k=3,nest=2*len(x))
   ntck = splrep(x,divz((2*max(y)-y),max(y)),s=0,k=3,nest=2*len(x))
   p = leastsq(lsplev,gnew,args=(ntck,0),factor=1.0,full_output=1)[0]
   iw = 1.5
   fr = 0.85
   vol = splint(p-iw,p+iw,tck)
   hei = fr*vol/(2*iw)
#   print p,splev(p,tck),hei,vol
#   return p
   vtck = splrep(xe,e-hei,s=0,k=3,nest=3*len(xe))
#   vtck = splrep(xe,e-fr*splev(p,tck),s=0,k=3)
   z = sproot(vtck,mest=len(xe))
   z = take(z,argsort(abs(z-p)))
   if len(z) > 1: return mean(z[:2])
   else: return p
#   else: print "aaaak!";return p

def lsplev(x,tck,der=0,val=0):
   y=splev(x,tck,der)-val
   return [y]

def getshift(s,u,g=None,pw=10):
   if g == -99:
      return -99,0.0
   else:
      cc = crosscor(s,u)
#      peaks = getlines(arange(cc.shape[0]),cc,cc,cc,argsort(cc),5,max(cc)/10,verb=0)
      pe = peak( cc, g, pw)
      if g:
         if abs(pe-g) > pw: pk=0.0
         else: pk=cc[int(pe)]
      else:
         pk=cc[int(pe)]
      return pe,max((0,pk))

def preprow(j,d,h=4,renorm=0,arc=0,nbg=0):
  j0 = max([0,j-h/2])
  j1 = min([d.shape[0],j+h/2+1])
  if arc:
    s = sum(d[j0:j1])
  else:
    s = median(d[j0:j1])
  s = s.astype(Float32)
  if renorm:
     norm = median(d[j])
     if norm: s = s/norm
  if nbg: s = specfilter(s,nbg)
  else: s = s-median(s)
  return s

def getpwr(r,u): return max( power(crosscor(r,u),2))

def getcof(r0):
    n=len(r0); i=arange(n)
#    v = (exp(-0.5*(i-n/2.0)**2/(n/2.0)**2)-exp(-0.5))/(1-exp(-0.5))
#    v = power(v,4)
    v = hanning(n)
    r = v * r0
    jz = nonzero(r)
    if len(jz):
       j0,j1 = jz[0]+1,jz[-1]-1
#       p = power(abs(r[j0+1:j1+1] - r[j0-1:j1-1]),2)
       p = power(r[j0:j1],2)
       x = i[j0:j1]
       numer, denom = add.reduce(x*p),add.reduce(p)
##       p=abs(diff(r[j0:j1-1])+diff(r[j0+1:j1]))
##       p=abs(r[j0+1:j1-1] - getpct(r[j0:1:j1-1],10))**2
##       x=i[j0+1:j1-1] + 1.0
##       numer, denom = add.reduce(x*p),add.reduce(p)
       if denom == 0.0: com = 0.0
       else: com = numer/denom
    else: com = 0.0
    return com

def gettran(d,nsub,xord,yord,pkwin=10,ofits=None,refrow=0.5,ymasks=[],\
        bord1=0,bord2=0,ny=0,verb=0,dy=0,r1=0,ydist=0,arc=0,nbg=0,\
        superit=1,wtcut=0,normwt=0,niter=1,wscale=1,nsub2=2):
    if ydist: d=transpose(d)
#    d = VTKGrowN(d,nsub2,1,numret=1,interp=1,xshift=-nsub2/2)
#    d = repeat(d,nsub2*ones(d.shape[1]),1)
    npos = nsub2*nsub-(nsub2-1)
    n=[d.shape[0], d.shape[1]]
    i1 = arange(npos)*n[1]/(nsub2*nsub)
    i2 = i1 + n[1]/nsub
    if dy == 0:
       if ny == 0 or ny >= n[0]: ny=n[0]
       dy = n[0]/ny
       j = arange(ny)*dy + dy/2
    else:
       j = arange(0,n[0],dy)
       ny = len(j)
    yref = ny/2
    jup = j[yref:]
    jdown = j[:yref][::-1]
    super = 0
    if verb: print "Doing X sections around"
    if verb: print i1
    if verb: print i2
    if verb: print "Doing Y sections around"
    if verb: print j
    u=preprow(j[int(refrow*ny)],d,h=4+5*ydist,renorm=0,arc=arc,nbg=nbg)
#    qdump("refdat.fits",u)
    u2=preprow(j[yref],d,h=dy,renorm=0,arc=arc,nbg=nbg)
    m = asarray([getcof(u[i1[jj]:i2[jj]]) for jj in range(len(i1))])+i1
    if verb: print "Centers="; print m
#    pq = asarray( map(lambda ii1,ii2: getshift(u2[ii1:ii2],u[ii1:ii2]) ,i1,i2))
    pq = asarray([getshift(u2[i1[jj]:i2[jj]],u[i1[jj]:i2[jj]]) for jj in range(len(i1))])
    p,q = transpose(pq)
    pref = 1*p
    predict = p
    predbad = -99
    for super in range(superit):
       if verb: print "Using peakwin=",pkwin

       jp2 = []; jq2 = []
       pred=p
       for jj in jdown:
         if jj < bord1: fine = 0
         else: fine = 1
         for yms in ymasks:
#           print jj+r1-dy/2, jj+r1+dy/2, yms[0],yms[1]
           if jj+r1-dy/2 <= yms[1] and yms[0] <= jj+r1+dy/2: fine = 0; break
         if fine:
            s=preprow(jj,d,h=dy,renorm=0,arc=arc,nbg=nbg)
            jpp2,jqq2=transpose([getshift(s[i1[pp]:i2[pp]],u[i1[pp]:i2[pp]],pred[pp],pkwin) for pp in range(len(pred))])
            pred=jpp2 * not_equal(jqq2,0) + pred*equal(jqq2,0)
         else:
            print "masked."
            jqq2=multiply(pred,0)
            jpp2=multiply(pred,0)
         jp2.append(jpp2); jq2.append(jqq2)
       jp2.reverse(); jq2.reverse()

       jp1 = []; jq1 = []
       pred=p
       for jj in jup:
         if jj > n[0]-bord2: fine = 0
         else: fine =1
         for yms in ymasks:
#           print jj+r1-dy/2, jj+r1+dy/2, yms[0],yms[1]
           if jj+r1-dy/2 <= yms[1] and yms[0] <= jj+r1+dy/2: fine = 0; break
         if fine:
            s=preprow(jj,d,h=dy,arc=arc,nbg=nbg)
            jpp1,jqq1=transpose([getshift(s[i1[pp]:i2[pp]],u[i1[pp]:i2[pp]],pred[pp],pkwin) for pp in range(len(pred))])
            pred=jpp1*not_equal(jqq1,0) + pred*equal(jqq1,0)
         else:
            print "masked."
            jqq1=multiply(pred,0)
            jpp1=multiply(pred,0)
         jp1.append(jpp1); jq1.append(jqq1)

       wp = concatenate([jq2,jq1])
       pp = concatenate([jp2,jp1])-pref
       if ofits: fdump(ofits[0],pp)
       if super == 0: ww = ones(pp.shape)
       if normwt: wp = clip(wp,0.0,normwt)
       if wtcut: wp = wp*greater_equal(wp,wtcut)
       zz=ravel(pp)
       x=array(ny*[m])
       y=array(map(lambda y: npos*[y],j))
#       y=concatenate(npos*[j])
       ww=ravel(ww)
       rwp=ravel(wp)
       xx = 2.0*(x.flat-n[1]/2.0)/n[1]
       yy = 2.0*(y.flat-n[0]/2.0)/n[0]

       uxord=xord; uyord=yord
       basis=prepbasis(xx,yy,uxord,uyord)
       pkwin = max([2,int(pkwin/1.5+0.5)])

       for iter in range(niter):
         bbb, yyy = transpose(rwp*basis/ww), rwp*zz/n[1]/ww
         solution = solvells( bbb, yyy)
         if verb: print "Coefficients="
         if verb: print solution
         shiftmap = add.reduce( solution*transpose(basis), -1)*n[1]
         shiftmap2 = reshape( shiftmap,pp.shape)
         predict = reshape( shiftmap,pp.shape) + p
         resid = (shiftmap-zz)
         rms = bwt(resid)[1]
         print "BWT residual=",rms
         rms = sqrt(sum(power(resid,2)*divz(rwp,ww))/sum(divz(rwp,ww)))
         print "RMS residual=",rms
         ww = 1+(wscale**2)*(shiftmap-zz)**2/(rms)**2
         ww = ww/mean(compress(rwp.flat,ww.flat))
         basis=prepbasis(xx,yy,uxord,uyord)
       if ofits: fdump(ofits[1],shiftmap2);fdump(ofits[2],wp)
    return [solution,rms]

def getharm(x,nharm):
  z = (x+1)/2
  hbase = [ones(x.shape,Float32)]
  sbase = [ sin(2*pi*i*z) for i in range(1,nharm+1)]
  cbase = [ cos(2*pi*i*z) for i in range(1,nharm+1)]
  for i in range(1,nharm+1):
    hbase.append(sbase[i-1])
    hbase.append(cbase[i-1])
  return asarray(hbase)

def prepbasis(x,y,ox,oy,bf=legendre):
    bax = bf(x,ox); bay = bf(y,oy)
    oxp = ox+1; oyp = oy + 1
    b = asarray([lx*bay for lx in bax])
    xs = list(x.shape)
#    xs.reverse()
    newshape = (oxp*oyp,) + tuple(xs)
    b = reshape(b,newshape)
    return asarray(b)

def fitmap(xord,yord,xx,yy,fxy,n):
    xx = 2.0*(xx-n[1]/2.0)/n[1]
    yy = 2.0*(yy-n[0]/2.0)/n[0]
    fxy = fxy / n[1]
    basis=prepbasis(xx,yy,xord,yord)
    solution = svdfit(transpose(basis),fxy)
    return solution

def getmap(xord,yord, solution,xx,yy,n):
    xx = 2.0*(xx-n[1]/2.0)/n[1]
    yy = 2.0*(yy-n[0]/2.0)/n[0]
    basis=prepbasis(xx,yy,xord,yord)
#    print solution.shape,basis.shape
    shiftmap=add.reduce( solution*transpose(basis), -1)
    return transpose(shiftmap*n[1])

def getmapping( xord, yord, solution, d, ydist=0, verb=1):
    dxa = zeros(d.shape,Float32)
    if ydist: dxa = transpose(dxa)
    xi = arange(dxa.shape[1])
    yi = arange(dxa.shape[0])
    n = dxa.shape
    jrange = range(n[0])
    tim0 = time.time()
    for j in jrange:
       yi = j*ones(xi.shape)
       shm = transpose(getmap(xord,yord,solution,xi,yi,n))
       dxa[j] = shm.astype(Float32)
       if verb:
         if not j % (len(jrange)/10) and j:
           tim1 = time.time()
           print "J=",j,100.0*j/len(jrange),(tim1-tim0)/j*(len(jrange)-j)
    if ydist: dxa = transpose(dxa)
    return dxa

def wavemapping( dxa, wxfm,ydist=0):
    wbtype,wsol,w1,w2,dw,logb=wxfm
    nw = int((w2-w1)/dw+0.5); w2=w1+nw*dw
    dw = (w2-w1)/nw
    d2 = zeros((dxa.shape[0],nw),Float64)
    xu = arange(0,dxa.shape[1],1,Float32)
    if wbtype:
       naxis1 = dxa.shape[1]
       xx = (2*(xu+1.0) - (1+naxis1))/(naxis1-1)
       xcheb = chebyshev(xx,len(wsol)-1)
       wx = add.reduce(wsol*transpose(xcheb),1)
    else:
       xc = (xu[0] + xu[-1])/2.0
       xw = (xu[-1] - xu[0])/2.0
       xx = (xu-xc)/xw
       xleg = legendre(xx,len(wsol)-1)
       wx = add.reduce(wsol*transpose(xleg),1)
    wo0, wo1 = wx[0], wx[-1]
#    print "Wavelength coverage=",wo0,wo1
    try:
       derw = (wx[2:]-wx[:-2])/2.0
       derw = concatenate([[derw[0]],derw,[derw[-1]]])
       gw = nonzero(greater(derw,0))+1
       g0 = gw[0]; g1 = gw[-1]
       for j in range(g0,-1,-1): wx[j] = wx[j+1] - derw[gw[g0]]
       for j in range(g1,len(wx)): wx[j] = wx[j-1] + derw[gw[g1]]
       wtck = splrep(wx,xu,s=0,k=3)
    except:
       print "Couldn't splrep!"
       return d2
    if logb:
      lw1 = log10(w1); lw2 = log10(w2); dlw = (lw2-lw1)/nw
      wa = power(10,dlw*(arange(nw)-0.0) + lw1)
    else:
      wa = dw*(arange(nw)-0.0) + w1
    xi = splev(wa,wtck)
    wt = zeros((dxa.shape[0],nw),Float64)
    for row in range(dxa.shape[0]): wt[row] = splev(xi, splrep(xu,dxa[row],k=3,s=0))
    gw = between(wa,wo0,wo1,inc=1).astype(Float32)
    gw = gw * ones(wt.shape)
    if not ydist: wt = xi + wt - arange(nw)
    return wt,gw

def derivc(x):
   d = 0.5 * (x[2:] - x[:-2])
   d = concatenate([[d[0]],d,[d[-1]]])
   return d

def rebin( d, dxa, dya, data=1, cubic=0, verb=1, constant=0, wrap=0, mirror=0):
    if verb: print "Transforming."
    if data:
       y,x = indices(dxa.shape)
       g = VTKGradMag(y+dya,1)*VTKGradMag(x+dxa,1)
    t = VTKImageTransform(d,dxa.astype(Float32),dya.astype(Float32),numret=1,interp=1,cubic=cubic,constant=constant,wrap=wrap,mirror=mirror)
    if data: t=t*g
    return t

def reversebin( d, dxa, dya, origsize=None, cubic=0, data=1, word="coordinates",verb=1,constant=0.0,wrap=0, mirror=0):
    if verb:
       if origsize: print "Reverse Transforming to size:",origsize
       else: print "Reverse Transforming %s." % (word)
    if data:
       y,x = indices(dxa.shape)
       g = VTKGradMag(y+dya,1)*VTKGradMag(x+dxa,1)
       d = divz(d,g)
    t = VTKImageTransform(d,dxa.astype(Float32),dya.astype(Float32),numret=1,reverse=1,origsize=origsize,interp=1,cubic=cubic,constant=constant,wrap=wrap,mirror=mirror)
    return t

def spatial1d(pky,oy,fwhm,uwid,txi,tyi,dqi,pwi,utyp):
     print "getting 1d profile."
     tim0 = time.time()
     pxx,pyy,pzz,pww = take([txi,tyi,dqi,pwi],argsort(txi),1)
     pzz0 = VTKMedian(pzz,4*int(fwhm)+1,1,numret=1)
     pzz1 = VTKGauss(pzz0*pww,int(fwhm),numret=1)
     pzz2 = VTKGauss(pww,int(fwhm),numret=1)
     pzz3 = divz(pzz1,pzz2)
     qdump("pzz3.fits",pzz3)
     pzz = divz(pzz,pzz0)
     pxx,pyy,pzz,pww = take([pxx,pyy,pzz,pww],argsort(pyy),1)
     pzz = VTKMedian(pzz,4*int(fwhm)+1,1,numret=1)
     pzz = pzz/mean(pzz)
     keep = greater(pzz,0)*greater(pww,max(pww)/10)
     spyy,spzz,spww = compress(keep,[pyy,pzz,pww],1)
     spyy,spzz,spww = adjspac( [pyy,pzz,pww],1e-4)
     mpyy = mean(spyy)
     tpy = sort(concatenate([extrema(spyy.flat),[mpyy-2,mpyy-1,mpyy,mpyy+1,mpyy+2]]))
     tpy = concatenate([pky*[tpy[0]],tpy,pky*[tpy[-1]]])
#     qdump("pzz.fits",pzz)
     print "fitting 1d b-spline";stdout.flush()
     pspl = splrep(spyy,spzz,w=spww,xb=oy-fwhm,xe=oy+fwhm,k=pky,t=tpy,
       nest=len(tpy),task=-1,s=None,full_output=1,quite=0)
     nspl = splint(oy-uwid,oy+uwid,pspl[0])/(2*uwid)
     prf = splev(pyy,pspl[0])
#     prf = prf*greater(prf,0)
     prf = prf / nspl
     qdump("prf.fits",prf)
     prf = take(prf,argsort(pxx))
#     qdump("prf.fits",take(prf,argsort(tyi)))
     tim1 = time.time()
     print "time for 1d profile = %.3f" % (tim1-tim0)
     return prf

def spatial2d(pky,pkx,oy,fwhm,uwid,txi,tyi,dqi,dri,pwi,prf,naxis1,adu,rn):
     print "getting 2d profile."
     tim0 = time.time()
     txi, tyi, dqi, dri, pwi, prf = take([txi, tyi, dqi, dri, pwi, prf],argsort(txi),1)
     txip, tyip, dqip, drip, pwip, prfp = compress(greater(pwi,0)*greater(dqi,0)*greater(prf,0), [txi, tyi, dqi, dri, pwi, prf],1)
     ke = greater(dqip,0)
     mfitted = VTKGauss(ke*dqip*prfp,4*int(fwhm)+1,1)/VTKGauss(ke*prfp,4*int(fwhm)+1,1)
#     qdump("dqip0.fits",dqip)
#     qdump("mfitted0.fits",mfitted)
     scats = 1.49*VTKMedian(abs(dqip-mfitted),2*int(fwhm)+1,1,numret=1)
     ddqip = diff(dqip)
     keep = nonzero(less(abs(ddqip),3*scats[1:]))+1
     dqi0 = dqip
     txip, tyip, dqi0, dqip, drip, pwip, prfp = take([txip, tyip, dqi0, dqip, drip, pwip, prfp],keep,1)
     mtyi = mean(tyi)
     ke = greater(dqip,0)
#     ke = ke*between(tyip,oy-uwid,oy+uwid,inc=1)
     ke = ke*exp(-0.5*power((tyip-mtyi)/(uwid),2))
     mfitted = VTKGauss(ke*dqip,2*uwid,1)/VTKGauss(ke,2*uwid,1)
#     qdump("dqip.fits",dqip)
#     qdump("mfitted.fits",mfitted)
     dqip = dqip/mfitted
     ddqip = diff(dqip)
     keep = nonzero(less(abs(dqip),uwid))
     txip, tyip, dqi0, dqip, drip, pwip, prfp = take([txip, tyip, dqi0, dqip, drip, pwip, prfp],keep,1)
     dnip = dqip
     qdump("dnip.fits",dnip)
#     tpy = sort(concatenate([extrema(tyi.flat),\
#           [oy-uwid-1,oy-uwid+1,oy-1,oy,oy+1,oy+uwid-1,oy+uwid+1]]))
     tpy = sort(concatenate([extrema(tyi.flat),\
             arange(mtyi,mtyi+uwid+0.5,+1.0,Float32)[1:], \
             arange(mtyi,mtyi-uwid-0.5,-1.0,Float32)]))
#     tpx = arange(naxis1)
     tpx = sort([naxis1/16.0,naxis1-naxis1/16.0])
     ktpy = concatenate([pky*[tpy[0]],tpy,pky*[tpy[-1]]])
     ktpx = concatenate([pkx*[tpx[0]],tpx,pkx*[tpx[-1]]])
     print extrema(dnip)
     print tpy
     print "fitting 2d b-spline";stdout.flush()
     pspl = bisplrep(txip,tyip,dnip,w=1+0*pwip,kx=pkx,ky=pky,s=None,task=-1,\
            tx=ktpx,nxest=len(ktpx),xb=min(txi.flat),xe=max(txi.flat),\
            ty=ktpy,nyest=len(ktpy),yb=min(tyi.flat),ye=max(tyi.flat),\
            full_output=1,quite=0)
     ptck = pspl[0]
     prff = bisplev2(txip,tyip,ptck)
     prff = where(between(prff,-Inf,Inf),prff,0.0)
     qdump("prff.fits",prff)
     bx = arange(naxis1)
     prf2 = transpose(bisplev(bx,arange(mtyi-fwhm,mtyi+fwhm,1),ptck))
     pint = array([dblint(bx1-0.5,bx1+0.5,mtyi-uwid,mtyi+uwid,ptck) for bx1 in bx[16:-16:16]])
     pint = pint/(2*uwid)
     pbx,pint = compress( greater(pint,0),[bx[16:-16:16],pint],1)
     ntck = splrep(pbx,pint,s=0,k=3,xb=0,xe=naxis1)
     np = splev(bx,ntck)
     np = np*greater(np,0)
     qdump("pint.fits",pint)
     qdump("np.fits",np)
     prf2 = divz(prf2,np)
     qdump("prf2.fits",prf2)
     prf2 = bisplev2(txi,tyi,ptck)
     prf2 = where(between(prf2,-Inf,Inf),prf2,0.0)
     np = splev(txi,ntck)
     np = np*greater(np,0)
     prf2 = divz(prf2,np)
     tim1 = time.time()
#     qdump("prf.fits",prf2)
     print "time for 2d profile = %.3f" % (tim1-tim0)
#     exit()
     return prf2,np

def adjspac(A,s=1e-6,extras=None,verb=1):
   x = A[0]
   o = argsort(x)
   A = take(A,o,1)
   if extras: extras = [take(extra,o) for extra in extras]
   x = A[0]
   dx = diff(x)
   while sometrue(equal(dx,0)):
      if verb: print "adjusting spacings...", sum(equal(dx,0))
      p = concatenate([[0.0],equal(dx,0)])
      x = x + s * p * (random(p.shape)-0.5)
      A[0] = x
      o = argsort(x)
      A = take(A,o,1)
      if extras: extras = [take(extra,o) for extra in extras]
      x = A[0]
      dx = diff(x)
   if extras: return A,extras
   else: return A

def bspec(tli,tyi,dqi,pwi,kx,tw,w1,w2):
   xb,xe = w1,w2
   tw = compress(between(tw,xb,xe,inc=0), tw)
   tli,dqi,pwi = compress(between(tli,xb,xe,inc=1), [tli,dqi,pwi], 1)
   splout = splrep(tli, dqi, w=pwi, xb=xb, xe=xe, k=kx, t=tw,\
           nest=len(tw), task=-1, s=None, full_output=1, quite=0)
   return splout

def bspec2(tli,tyi,dqi,pwi,kx,tw,w1,w2,ky,ty,oy,uwid):
   xb,xe = w1,w2
   yb,ye = oy-uwid,oy+uwid
   tw = compress(between(tw,xb,xe,inc=0), tw)
   ty = compress(between(ty,yb,ye,inc=0), ty)
   tw = concatenate([kx*[tw[0]],tw,kx*[tw[-1]]])
   ty = concatenate([ky*[ty[0]],ty,ky*[ty[-1]]])
   print tw[:20]
   print tw[-20:]
   print ty
   splout = bisplrep(tli, tyi, dqi, w=pwi,\
           xb=xb, xe=xe, kx=kx, tx=tw, yb=yb, ye=ye, ky=ky, ty=ty,\
           task=-1, s=None, full_output=1, quite=1)
   return splout

def quickstat(x):
   m = median(x)
   s = 1.49*median(abs(x-m))
   return m,s

