
from basics import *
from sets import Set
from Expector import *
from LDP import *
from FITS import *
from Multipack import *
from Mestimators import *
import os

pkeys = ["subsec1", "subsec1m", "subsec1r", "subsec1b", "subsec1n",
        "subsec1X", "subsec1y", "subsec1w", "subsec1gh", "fitted1"]
dkeys = ["ds","ms","rs","bs","ns",
         "xs","ys","ws","ps","fs"]
ldkeys = dkeys + ["dws","ts"]


def OneIndex(dfile,verb=0):
    d = FITS(dfile,readonly=1)
    objects = []
    while not len(objects) or objects[-1] != "N/A":
       objects.append(d["OBJ%04d" % (len(objects)+1)].strip())
    d.close()
    objects = objects[:-1]
    if verb: print "Read %d object names from %s" % (len(objects),dfile)
    return objects

def ArrayToDict(X,t):
    a = {}
    for jkey,dkey in enumerate(dkeys): a[dkey] = X[jkey]
    a["dws"] = Derivative(a["xs"],a["ws"])
    a["ts"] = t*ones(a["xs"].shape,Float64)
    return a

def Derivative(x,w):
    wa = sort(list(Set(w.flat)))
    xa = array([median(compress(equal(w.flat,w1),x.flat)) for w1 in wa])
    xa,wa = take([xa,wa],argsort(xa),1)
    xtck = splrep(xa,log(wa),s=0,k=3)
    dlwdx = zeros(x.flat.shape,Float64)
    order = argsort(x.flat)
    put(dlwdx,order,splev(take(x.flat,order),xtck,1))
    dws = reshape(dlwdx*w.flat,x.shape)
    return abs(dws)

def ReadOneFile(dfile,objno):
    print "Reading from",dfile,
    qs = None
    if "odc" in dfile:
       qfile = dfile.replace("odc","oqc")
       if os.path.exists(qfile):
          d = FITS(qfile,readonly=1)
          qw = d.data()[3][objno-1]
          qq = d.data()[5][objno-1]
          d.close()
          qs = (qw,qq)
    d = FITS(dfile,readonly=1)
    etime = d["exptime"]
    if d["nshuffle"]:
      pa = -(d["rotangle"]-133.55) * pi/180
      nr = d["nod-ra"]/0.2
      nd = d["nod-dec"]/0.2
      nsh = nd*cos(pa) + nr*sin(pa)
      print nr,nd,pa,nsh,
    else: nsh = 0
    if nsh: nf = 2
    else: nf = 1
    hap = d["hap"]
    corners = []
    hdu = nf*(objno-1)+2
    print hdu,
    d.goto(hdu)
    slitobj = d["slitobj"]
    print slitobj,
    rchi2 = d["rchi2"]
    print "%.4f" % (rchi2),
    pm1 = []
    while d["pm1"+str(len(pm1)+1)] != "N/A": pm1.append(d["pm1"+str(len(pm1)+1)])
    pm1 = eval("["+"".join(pm1)+"]")[:1]
    pm2 = []
    while d["pm2"+str(len(pm2)+1)] != "N/A": pm2.append(d["pm2"+str(len(pm2)+1)])
    pm2 = eval("["+"".join(pm2)+"]")[:1]
    pmh = []
    while d["pmh"+str(len(pmh)+1)] != "N/A": pmh.append(d["pmh"+str(len(pmh)+1)])
    pmh = eval("["+"".join(pmh)+"]")
    corners.append(eval(d["corners"]))
    adata = d.data()
    if nsh:
       d.goto(hdu+1)
       slitobj = d["slitobj"]
       print slitobj
       corners.append(eval(d["corners"]))
       bdata = d.data()
    d.close()
#    qdump("/tmp/a.fits",adata[0])
#    qdump("/tmp/b.fits",bdata[0])
#    exit()
    o = {"t":etime,"h":hap,"nf":nf,"nsh":nsh,"corners":corners,"rchi2":rchi2,"qs":qs}
    o["pm1"] = pm1
    o["pm2"] = pm2
    o["pmh"] = pmh
    o["a"] = adata
    if nsh:
       o["b"] = bdata
    return o

def ReadOneObject(object,robjects,objno):
    print object
    robject = robjects[object]
    print robject
#    objno = [1+dobjects[dfile].index(object) for dfile in robject]
    print objno
    data = {}
    for r,o in zip(robject,objno):
       data[r] = ReadOneFile(r,o)
    for r in data:
       d = data[r]
       d["a"] = ArrayToDict(d["a"],d["t"])
       if d["nsh"]: d["b"] = ArrayToDict(d["b"],d["t"])
       if d["qs"]:
           ws = d["a"]["ws"]
           qw,qq = d["qs"]
           qd = {}
           [qd.setdefault(qw1,qq1) for qw1,qq1 in zip(qw,qq)]
           ws0 = ws[0]
           qs0 = array([(ws1 in qd and qd[ws1] or 0.0) for ws1 in ws0])
           qs = asarray(len(ws)*[qs0])
           for k in ["ds","ms","rs","bs","fs"]:
#              qdump("ds0.fits",d["a"]["ds"])
              d["a"][k] = divz(d["a"][k],qs)
#              qdump("ds.fits",d["a"]["ds"])
#              qdump("qs.fits",qs)
#              exit()
              if d["nsh"]: d["b"][k] = divz(d["b"][k],qs)
    return data

WBinsDict = {}

def GetWBins(nbin=1.0,minbin=0.0,w1=0.0,w2=0.0):
    if (nbin,minbin,w1,w2) not in WBinsDict:
       mw = arange(3800,11.0e3,5.0,Float32)
       if w1: mw = concatenate([mw,[w1]])
       if w2: mw = concatenate([mw,[w2]])
       mw = sort(list(Set(mw)))
       wxbasis = legendre((mw-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
       mx = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ
#       for (a,b) in zip(mw,mx): print a,b
       if w1: mx0 = mx[searchsorted(mw,w1)]
       else: mx0 = mx[0]
       mx = mx - mx0
       lmw = log(mw)
       mxtck = splrep(mx,lmw,s=0,k=1)
       mdwdx = splev(mx,mxtck,1)*mw
#       mdwdx = clip(mdwdx,minbin,1e3)
       if w1: bin0 = w1
       else: bin0 = mw[0]+mdwdx[0]/2.0
       if w2: wmax = w2
       else: wmax = LDPDISP["lambda"][-1]+nbin*mdwdx[-1]/2.
       def getx(www,xxx=-1):
           wxbasis = legendre((www-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
           p = (LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ - mx0)
#           raw_input(str((p,xxx)))
           if xxx == -1: return p
           else: return p - xxx
       bins = [bin0]
       xins = [getx(bin0)]
#       print nbin
       while bins[-1] < wmax:
#          print xins
#          print bins,wmax
          new = fsolve(getx,bins[-1],args=(xins[-1]+nbin,))
#          raw_input(str(new))
          xins.append(xins[-1]+nbin)
          bins.append(new)
       wbins = array(bins)
       if w2: wbins = compress(less_equal(wbins,w2),wbins)
       xbins = getx(wbins)
       dbins = nbin*splev(xbins,mxtck,1)*wbins
#       dbins = nbin * clip(dbins,minbin,1e3)
       WBinsDict[(nbin,minbin)] = wbins,dbins
    return WBinsDict[(nbin,minbin)]


def oldGetWBins(nbin=1.0,minbin=0.0,w1=0.0,w2=0.0):
    if (nbin,minbin,w1,w2) not in WBinsDict:
       mw = LDPDISP["lambda"]
       if w1: mw = concatenate([mw,[w1]])
       if w2: mw = concatenate([mw,[w2]])
       mw = sort(list(Set(mw)))
       wxbasis = legendre((mw-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
       mx = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ
       mx,mw = take([mx,mw],argsort(mx),1)
       if w1:
          mx0 = mx[searchsorted(mw,w1)]
       else:
          mx0 = mx[0]
       mx = mx - mx0
       lmw = log(mw)
       mxtck = splrep(mx,lmw,s=0,k=3)
       mdwdx = splev(mx,mxtck,1)*mw
       mdwdx = clip(mdwdx,minbin,1e3)
       if w1: bin0 = w1
       else: bin0 = mw[0]+mdwdx[0]/2.0
       if w2: wmax = w2+nbin*mdwdx[searchsorted(mw,w2)]/2.
       else: wmax = LDPDISP["lambda"][-1]+nbin*mdwdx[-1]/2
       bins = [bin0]
       while bins[-1] < wmax:
          bw0 = 1e38
          bw = bins[-1]
          co = 0
          while abs(bw-bw0)/bw > 1e-6:
             bw0 = bw
             co += 1
             if len(bins) > 1:
                wxbasis = legendre((array([bins[-1],bw])-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
                bx = mean(LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ - mx0)
             else:
                wxbasis = legendre((bw-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
                bx = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ - mx0
             dwdxb = splev(bx,mxtck,1)*bw
             dwdxb = clip(dwdxb,minbin,1e3)
             bw = bins[-1]+nbin*dwdxb/2.
             print co,bw,bw0,dwdxb
             if bw > 9000: raw_input(str(bins))
          bins.append( bins[-1]+nbin*dwdxb)
       wbins = array(bins)
       if w2: wbins = compress(less_equal(wbins,w2),wbins)
       wxbasis = legendre((wbins-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
       xbins = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ - mx0
       dbins = splev(xbins,mxtck,1)*wbins
       dbins = nbin * clip(dbins,minbin,1e3)
       WBinsDict[(nbin,minbin)] = wbins,dbins
    return WBinsDict[(nbin,minbin)]

def SingleBin(w,dw,d,h,nsh=1,t=0,M=L2,pw=1,scut=5.0):
    if scut > 100:
       adw = [1.0]
    else:
       adw = [2.0,1.0]
    for jdw,ndw in enumerate(adw):
       keep = less(abs(d["ws"]-w),ndw*dw/2.)
       if not sum(keep.flat): return asarray(6*[0.0])
       e0 = {}
       for dkey in ldkeys: e0[dkey] = compress(keep.flat,d[dkey].flat)
       yc = e0["ys"].flat[argmax(e0["ps"])]
       keep = less_equal(abs(e0["ys"]-yc),h+0.5)
       e = {}
       for dkey in ldkeys:
          e[dkey] = compress(keep.flat,e0[dkey].flat)
       dt = e["ts"]
       dd = abs(e["dws"])
       y = e["ys"] - yc
       p = e["ps"]
       f = e["fs"]
       s = e["ds"]
       n = e["ns"]
       v = n**2
       r0 = (s-f)/n
       er = M(r0/scut,w=1)
       k = 1/(dt*dd*p)
       s = k*s
       v = k*v
       q = divz(1.0,v)
       spec1 = divz(sum(er*q*s),sum(er*q))
       nois1 = divz(1,getsqrt(divz(sum(er*q*(divz(1,v))),sum(er*q))))
       if jdw == 0:
          Me = less(abs(r0),scut)
       else:
#          nfac = bwt((s-spec2/k)/n)[1]
          nfac = 1.0
#          print nfac
#          Me = less(abs(r0),scut)
          Me = Fair((s-spec2/k)/n/nfac,w=1)
#          Me = Huber2((s-spec2/k)/n/nfac,w=1)
#          Me = L1((s-spec2/k)/n/nfac,w=1)
#          Me = L2((s-spec2/k)/n/nfac,w=1)
#          Me = M(r0/scut,w=1)
       qM = power(p,pw)*q
       spec2 = divz(sum(qM*s),sum(qM))
       nois2 = divz(1,sqrt(divz(sum(qM/v),sum(qM))))
       scat = divz(sum(q*(s-spec2)**2),sum(q))
       scat = sqrt(scat)
    if 5000 < w < 8000 and nois2 == 0: print "here",w,dw,e["ys"].shape
    apkeep = less_equal(abs(y),h)
    A = 2.0*h
    R = dw*A*t
    if nsh != 0: R = 2*R
    spec2 = spec2 * R
    nois2 = nois2 * sqrt(R)
    spec1 = spec1 * R
    nois1 = nois1 * sqrt(R)
    eds,ens,er0 = compress(apkeep,[e["ds"],e["ns"],r0],1)
    wt = divz(1,ens**2*(1+er0**2))
    spec0 = divz(sum(wt*eds),sum(wt))*A
    nois0 = sqrt(divz(sum(wt*ens**2),sum(wt))*A)
    pkg = asarray([spec0,nois0,spec1,nois1,spec2,nois2])
    return pkg

def FitOneObject(data,nbin=1,output="Spectra",name=None,inter=0,scut=5.0,w1=0,w2=0):
    w1a = w1
    w2a = w2
    ro = data.keys()
    ro.sort()
#    ro = [r for r in ro if data[r]["rchi2"] < 2]
    nsh = data[ro[0]]["nsh"]
    unsh = [0,1][nsh!=0]
    h = mean([data[r]["h"] for r in ro])

#    if inter: pdev = "/xs"
#    else: pdev = "%s/%s.ps/vcps"

    npit = 4
    Mp = lambda r,w: 1.0/(1+r**2)
#    Mp = lambda r,w: 1.0
    for pit,pbin,pMp in zip(range(npit),[2.0,1.5,1.0,1.0],[Mp,Mp,Mp,Mp]):
       for r in ro:
          for jab,ab in enumerate("ab"[:unsh+1]):
              ds = data[r][ab]["ds"]
              fs = data[r][ab]["fs"]
              ns = data[r][ab]["ns"]
              bs = data[r][ab]["bs"]
              mbs = median(bs)
              rs = divz(ds-fs,ns)**2
              brs = divz(bs-mbs,ns)**2
              wrs = logical_or(greater(rs,scut**2),greater(brs,0.1))
#              qdump("/tmp/dbs.fits",divz(bs-mbs,ns))
#              [qdump("/tmp/%s.fits" % (key),data[r][ab][key]) for key in ldkeys]
#              exit()
              ls = 1.0*less(VTKConvolve(1.0*wrs,k=3,numret=1),0.25)
              data[r][ab]["ls"] = ls

       tt = 0
       t = []
       if inter > 1:
          nx=(1+unsh)
          ny=len(ro)
          if ny > 4:
             nx = nx + 2
             ny = 6
          opendev("/xs",nx=nx,ny=ny,size=min([nx*5,12]),land=0,asp=0.5*ny/nx,ask=1)
          for r in ro:
             for jab,ab in enumerate("ab"[:unsh+1]):
                corners = data[r]["corners"][jab]
                ds = data[r][ab]["ds"]
                fs = data[r][ab]["fs"]
                ps = data[r][ab]["ps"]
                ns = data[r][ab]["ns"]
                ws = data[r][ab]["ws"]
                xs = data[r][ab]["xs"]
                ys = data[r][ab]["ys"]
                ls = data[r][ab]["ls"]
                rs = divz(ds-fs,ns)**2
                gz = ps
                wrz = 1/(1+rs)

                for js,us in enumerate([ds,ds-fs]):
                   if js == 0: cpgsvp(0.1,0.95,0.525,0.90)
                   else: cpgsvp(0.1,0.95,0.15,0.525)
                   newbox(add(extrema(xs.flat),[-0.5,2]),add(extrema(ys.flat),[-0.5,-2.0]),ch=2.5,manvp=1,xlab=(js==1),ylab=2,era=(js==0))
                   if js == 0:
                     subtitle("%s (%s)" % (r,ab),ch=2.5)

                   xfm = array([xs.flat[0]-0.5,1.0,0.0,ys.flat[0]-0.5,0.0,1.0])
                   mds = divz(sum(wrz.flat*gz.flat*ps.flat*ds.flat),sum(wrz.flat*gz.flat*ps.flat))
                   mds = median(ns.flat)
                   mds = 1.49*median(abs(us.flat))

                   if mds: a1,a2 = -5*mds,5*mds
                   else: a1,a2 = -100,100
                   cpggray(us,us.shape[1],us.shape[0],1,us.shape[1],1,ds.shape[0],a1,a2,xfm)

                   cpgsci(5)
                   cpgsls(4)
                   cont = array([data[r]["pm1"]])
                   cpgcont(ys,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                   cpgsci(1)
                   cpgsls(1)

                   cpgsci(7)
                   cpgsls(2)
                   cont = array([max(ps.flat)/2.0])
                   cpgcont(ps,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                   cpgsci(1)
                   cpgsls(1)

                   if js:
                      cpgsci(3)
                      cpgsls(1)
                      cont = array([w1])
                      cpgcont(ws,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                      cont = array([w2])
                      cpgcont(ws,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                      cpgsci(1)
                      cpgsls(1)

                   else:
                      cpgsci(4)
                      cpgsls(1)
                      cont = array([scut**2])
                      cpgcont(rs,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                      cpgsci(1)
                      cpgsls(1)

                      cpgsci(2)
                      cpgsls(1)
                      cont = array([1.0])
                      cpgcont(ls,ps.shape[1],ps.shape[0],1,ps.shape[1],1,ps.shape[0],cont,-len(cont),xfm)
                      cpgsci(1)
                      cpgsls(1)


          curs = ["O",(0,0)]
          while curs[0] != "g":
             if curs[0] in "qQI": os._exit(0)
             else: curs = getcurs(curs[1][0],curs[1][1])
          closedev()

       d = {}
       for r in ro:
          for ab in "ab"[:unsh+1]:
             t.append(data[r]["t"])
          d[r] = {}
          for dkey in data[r]["a"]:
             d[r][dkey] = []
             for ab in "ab"[:unsh+1]:
                d[r][dkey] = concatenate([d[r][dkey],data[r][ab][dkey].flat])
          keep = greater(d[r]["ps"],1e-3)*greater(d[r]["ns"],0)*greater(d[r]["ds"],-3*d[r]["ns"])*d[r]["ls"]
          keep = keep*not_equal(d[r]["ds"],0)
          for dkey in ldkeys: d[r][dkey] = compress(keep,d[r][dkey])
          order = argsort(d[r]["ws"])
          for dkey in ldkeys: d[r][dkey] = take(d[r][dkey],order)
    
       tt = sum(t)
       A = 2.0*h
       print t

       w1b,w2b = extrema([extrema(data[r][ab]["ws"].flat) for r in ro for ab in "ab"[:unsh+1]])

       wbins,dbins = GetWBins(pbin*nbin,minbin=1.0,w1=w1,w2=w2)
       print pMp

       individ = {}
       norms = {}
       scales = {}
       weights = {}
       if pit: pw = 2
       else: pw = 2
       for r in ro:
          individ[r] = transpose([SingleBin(wb,db,d[r],h,nsh=unsh,t=data[r]["t"],M=pMp,pw=pw,scut=scut) for wb,db in zip(wbins,dbins)])
#          print individ[r][4]
#          print individ[r][5]
          for x in range(3):
             individ[r][2*x] = individ[r][2*x]/((unsh+1)*data[r]["t"])
             individ[r][2*x+1] = individ[r][2*x+1]/((unsh+1)*data[r]["t"])
          norms[r] = sum(individ[r][4]*between(wbins,5500.0,8000.0))

       print [norms[r] for r in ro]
       unorms = [norms[r] for r in ro]
       gnorms = compress(greater(unorms,0),unorms)
       if gnorms: mnorm = median(gnorms)
       else: mnorm = 0.0
       for r in ro:
          norms[r] = (norms[r] > mnorm/4 and norms[r]) or 0.0
       nnorm = [norms[r] for r in ro]
       mnorm = mean(compress(nnorm,nnorm))

       print [norms[r] for r in ro]
       sumweights = 0.0
       nro = 0
       tt = 0.0
       for r in ro:
          if mnorm:
             norms[r] = divz(mnorm,norms[r])
             if norms[r]:
                tt = tt + data[r]["t"]*(unsh+1)
                nro = nro + 1
          else:
             norms[r] = 1.0
          scales[r] = norms[r]*data[r]["t"]*(unsh+1)
          weights[r] = divz(1,sqrt(scales[r])*individ[r][5]**2)
#          print weights[r]
          sumweights = sumweights + weights[r]
       print [norms[r] for r in ro]
       sumnorms = sum([norms[r] for r in ro])
       sum2norms = sum([norms[r]**2 for r in ro])
       sumweights = divz(sumweights, nro)
       for r in ro:
          weights[r] = divz(weights[r],sumweights)
       print [median(weights[r]) for r in ro]
       print nro

       print A,tt

       for x in range(3):
          exec("sbins%d = divz(sum([(weights[r]*scales[r]*individ[r][%d]) for r in ro]),tt)" % (x,2*x))
          exec("ebins%d = divz(sqrt(sum([weights[r]*(scales[r]*individ[r][%d])**2 for r in ro])),tt)" % (x,2*x+1))

       if pbin == 1.0 or 1:
         ds = []
         bs = []
         vs = []
         ns = []
         ps = []
         ws = []
         tx = None
         for r in ro:
            if mnorm and norms[r]:
               print r
               ws2 = []
               ds2 = []
               bs2 = []
               vs2 = []
               ns2 = []
               ps2 = []
               for ab in "ab"[:unsh+1]:
                  if 0 and "yc" in data[r][ab]:
                     keep = less(abs(data[r][ab]["yc"].flat),h+0.5)
                  else:
                     ws1 = data[r][ab]["ws"].flat
                     ys1 = data[r][ab]["ys"].flat
                     wxbasis = legendre((ws1-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
                     dcx0 = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ
                     m1 = data[r]["pm1"]
                     if len(shape(m1)) == 0: m1 = [m1]
                     m1b = polynom(dcx0,len(m1)-1)
                     dy1 = dot(m1,m1b)
                     yc1 = ys1 - dy1
                     keep = less(abs(yc1.flat),h+0.5)
                  keep = keep * (data[r][ab]["ls"].flat)
                  keep = keep * greater(data[r][ab]["ns"].flat,0.0)
                  keep = keep * greater(data[r][ab]["ws"].flat,0)
#                  keep = keep * less(data[r][ab]["ws"].flat,11e3)
                  keep = keep * greater(data[r][ab]["ps"].flat,1e-3)
                  keep = keep * greater(data[r][ab]["ds"].flat,-3* data[r][ab]["ns"].flat)
                  keep = keep * not_equal(data[r][ab]["ds"].flat,0)
#                  keep = keep * less(abs(data[r][ab]["ds"].flat-data[r][ab]["fs"].flat),scut*data[r][ab]["ns"].flat)
                  ds1 = compress(keep,data[r][ab]["ds"].flat)
                  bs1 = compress(keep,data[r][ab]["bs"].flat)
                  ns1 = compress(keep,data[r][ab]["ns"].flat)
                  ws1 = compress(keep,data[r][ab]["ws"].flat)
                  dws1 = compress(keep,data[r][ab]["dws"].flat)
                  fs1 = compress(keep,data[r][ab]["fs"].flat)
                  ps1 = compress(keep,data[r][ab]["ps"].flat)
                  ms1 = pMp((ds1-fs1)/(scut*ns1),w=1)
                  ds1 = ds1*norms[r]/ps1/dws1/data[r]["t"]
                  bs1 = bs1*norms[r]/dws1/data[r]["t"]
                  ns1 = ns1*sqrt(norms[r]/ps1/dws1/data[r]["t"])
                  vs1 = ns1**2
                  ns1 = divz(ms1,ns1)*sqrt(data[r]["t"])
                  ws1,ds1,bs1,vs1,ns1,ps1 = take([ws1,ds1,bs1,vs1,ns1,ps1],argsort(ws1),1)
                  if not tx: tx = sort(list(Set(ws1)))
                  ws2 = concatenate([ws2,ws1])
                  ds2 = concatenate([ds2,ds1])
                  bs2 = concatenate([bs2,bs1])
                  vs2 = concatenate([vs2,vs1])
                  ns2 = concatenate([ns2,ns1])
                  ps2 = concatenate([ps2,ps1])
               ws2,ds2,bs2,vs2,ns2,ps2 = take([ws2,ds2,bs2,vs2,ns2,ps2],argsort(ws2),1)
               ws = concatenate([ws,ws2])
               ds = concatenate([ds,ds2])
               bs = concatenate([bs,bs2])
               vs = concatenate([vs,vs2])
               ns = concatenate([ns,ns2])
               ps = concatenate([ps,ps2])
         if len(ws):
            ws,ds,bs,vs,ns,ps = take([ws,ds,bs,vs,ns,ps],argsort(ws),1)
            ws,ds,bs,vs,ns,ps = adjspac([ws,ds,bs,vs,ns,ps])
            xb,xe = extrema(ws)
            print xb,xe
            tx = wbins[:-1]+dbins[:-1]/2.0
#            tx = wbins
#            tx = compress(between(tx,xb,xe),tx)[::2]
            stck = splrep(ws,ds,w=(ps)*ns,s=None,task=-1,k=1,t=tx,xb=xb,xe=xe)
            etck = splrep(ws,vs,w=(ps)*ns,s=None,task=-1,k=1,t=tx,xb=xb,xe=xe)
            btck = splrep(ws,bs,w=(ps)*ns,s=None,task=-1,k=1,t=tx,xb=xb,xe=xe)
            sbinss = A*dbins*splev(wbins,stck)*between(wbins,xb,xe)
            bbinss = A*dbins*splev(wbins,btck)*between(wbins,xb,xe)
            ebinss = sqrt((sumnorms-1)/(sumnorms+1.))*getsqrt(dbins*splev(wbins,etck)*between(wbins,xb,xe))/A/sumnorms
            sbins2 = sbinss
            ebins2 = ebinss
            bbins2 = bbinss
       else:
          stck = splrep(wbins,sbins2/dbins,s=0,k=3)

       for r in ro:
          if mnorm and norms[r]:
             d[r]["fs0"] = d[r]["fs"]
             ps = d[r]["ps"]
             rfs = splev(d[r]["ws"],stck)
             rfs = (data[r]["t"]) * d[r]["dws"] * rfs
             if norms[r]: rfs = rfs/norms[r]
             d[r]["fs"] = rfs * ps
             for jab,ab in enumerate("ab"[:unsh+1]):
                ws = data[r][ab]["ws"].flat
                ss = zeros(ws.shape,Float64)
                order = argsort(ws.flat)
                put(ss,order,splev(take(ws,order),stck))
                ss = reshape(ss,data[r][ab]["ds"].shape)
                ss = ss * data[r][ab]["dws"] * data[r]["t"]
                data[r][ab]["rfs"] = ss
                if norms[r]: data[r][ab]["rfs"] = data[r][ab]["rfs"] / norms[r]
                data[r][ab]["fs"] = data[r][ab]["rfs"] * data[r][ab]["ps"]
             if pit < npit-1:
                print "Let's try a new set of moments for",r
                ws = concatenate([data[r][ab]["ws"].flat for ab in "ab"[:unsh+1]])
                xs = concatenate([data[r][ab]["xs"].flat for ab in "ab"[:unsh+1]])
                ys = concatenate([data[r][ab]["ys"].flat for ab in "ab"[:unsh+1]])
                ds = concatenate([data[r][ab]["ds"].flat for ab in "ab"[:unsh+1]])
                bs = concatenate([data[r][ab]["bs"].flat for ab in "ab"[:unsh+1]])
                ns = concatenate([data[r][ab]["ns"].flat for ab in "ab"[:unsh+1]])
                ls = concatenate([data[r][ab]["ls"].flat for ab in "ab"[:unsh+1]])
                rs = concatenate([data[r][ab]["rfs"].flat for ab in "ab"[:unsh+1]])
                fs = concatenate([data[r][ab]["fs"].flat for ab in "ab"[:unsh+1]])
                ps = concatenate([data[r][ab]["ps"].flat for ab in "ab"[:unsh+1]])
                keep = greater(ds,-3*ns)*greater(ns,0)*greater(ps,1e-3)*ls*not_equal(ds,0)
                ws,xs,ys,ds,bs,ns,fs,ps,rs = [compress(keep,a) for a in (ws,xs,ys,ds,bs,ns,fs,ps,rs)]
                order = argsort(ws)
                ws,xs,ys,ds,bs,ns,fs,ps,rs = [take(a,order) for a in (ws,xs,ys,ds,bs,ns,fs,ps,rs)]
                es = divz((ds-fs)/ns)

                wxbasis = legendre((ws-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
                dcx0 = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ

                m1b = polynom(dcx0,len(data[r]["pm1"])-1)
                m1y1 = dot(data[r]["pm1"],m1b)
                wt = getsqrt(ds)*less(es**2,scut**2)*less(abs(ys-m1y1),h+1)

                W1Order = len(data[r]["pm1"]) - 1
                m1b = polynom(dcx0,W1Order)
                m1 = svdfit(transpose(m1b)*wt[::,NewAxis],ys*wt)
                print m1
                data[r]["pm1"] = m1
                dy1 = dot(data[r]["pm1"],m1b)
                yc = ys - dy1

                wt = getsqrt(ds)*less(es**2,scut**2)*less(abs(yc),min([h+1,3.0*data[r]["pm2"][0]]))
                W2Order = len(data[r]["pm2"]) - 1
                m2b = polynom(dcx0,W2Order)
                m2 = svdfit(transpose(m2b)*wt[::,NewAxis],power(yc,2)*wt)
                print m2
                data[r]["pm2"] = m2
                dy2 = getsqrt(dot(data[r]["pm2"],m2b))
                yn = divz(yc,dy2)
                gy = exp(-0.5*yn**2)
                mh = None

                try:
                   wt = not_equal(ds,0.0)*less(es**2,scut**2)*less(abs(yc),min([h+1,3.0*data[r]["pm2"][0]]))/ns
                   if scut > 100:
                      YHOrder = int(h)
                      WHOrder = 1
                   else:
                      YHOrder = int(h/2+1)
                      WHOrder = 0
                   print h,YHOrder
                   mhyb = hermite(yn,YHOrder,norm=1)
                   uds = ds
                   if scut < 100:
                     mhyb = mhyb[1:]
                     uds =  (ds-sqrt(pi)*gy*rs)
                   mhwb = polynom(dcx0,WHOrder)
                   mhb = [(gy*rs*wt*hyb1*hwb1) for hyb1 in mhyb for hwb1 in mhwb]
                   mh = svdfit(transpose(mhb),uds*wt)
                   print mh
                   data[r]["pmh"] = mh
                   mhb = [(gy*hyb1*hwb1) for hyb1 in mhyb for hwb1 in mhwb]
                   gh = dot(data[r]["pmh"],mhb)
                   if scut < 100: gh = gh + sqrt(pi)*gy
                except:
                   mh = None

                for ab in "ab"[:unsh+1]:
                   ws = data[r][ab]["ws"].flat
                   ys = data[r][ab]["ys"].flat
                   wxbasis = legendre((ws-LDPCOEFCi)/LDPCOEFSi,len(LDPCOEF)-1)
                   dcx0 = LDPZ*log(dot(LDPCOEF,wxbasis))+LDPZ
                   m1b = polynom(dcx0,W1Order)
                   dy1 = dot(data[r]["pm1"],m1b)
                   yc = ys - dy1
                   data[r][ab]["yc"] = reshape(yc, data[r][ab]["ys"].shape)
                   m2b = polynom(dcx0,W2Order)
                   dy2 = getsqrt(dot(data[r]["pm2"],m2b))
                   yn = divz(yc,dy2)
                   gy = exp(-0.5*yn**2)
                   ps = sqrt(pi)*gy
                   if mh:
                      mhyb = hermite(yn,YHOrder,norm=1)
                      if scut < 100:
                        mhyb = mhyb[1:]
                      mhwb = polynom(dcx0,WHOrder)
                      mhb = [(gy*hyb1*hwb1) for hyb1 in mhyb for hwb1 in mhwb]
                      ps = dot(data[r]["pmh"],mhb)
                      if scut < 100:
                         ps = ps + sqrt(pi)*gy
                   data[r][ab]["ps"] = reshape(ps, data[r][ab]["ps"].shape)
                   data[r][ab]["fs"] = data[r][ab]["rfs"] * data[r][ab]["ps"]

       if inter:
          opendev("/xs",nx=1,ny=2,size=9,land=0,asp=1.0)

          newbox([LDPDISP["lambda"][0],LDPDISP["lambda"][-1]],getmar((sbins2+ebins2),50,150),ch=2)
          xlabel("\gl (\A)",ch=2)
          ylabel("PV weighted Counts/s",ch=2)
          subtitle("%s (p=%d)" % (name,1+pit),ch=2)
          for r in ro:
             if norms[r]:
                wp,dwp,sp,dsp = compress(weights[r],[wbins,dbins,individ[r][4],individ[r][5]],1)
                drawpt(wp,norms[r]*sp,ps=1,ch=2,ci=4)
                drawpterr(wp,dwp/2.,norms[r]*sp,norms[r]*dsp,ch=2,ci=4)
          drawpt(wbins,sbins2,ps=1,ch=2,ci=1)
          drawpterr(wbins,dbins/2.,sbins2,ebins2,ch=2,ci=1)

          newbox([LDPDISP["lambda"][0],LDPDISP["lambda"][-1]],getmar((sbins2+ebins2)/dbins/2.,50,300),ch=2)
          xlabel("\gl (\A)",ch=2)
          ylabel("PV weighted Counts/A/s",ch=2)
          for r in ro:
             if norms[r]:
                wp,dwp,sp,dsp = compress(weights[r],[wbins,dbins,individ[r][4]/dbins,individ[r][5]/dbins],1)
                drawpt(wp,norms[r]*sp,ps=1,ch=2,ci=4)
                drawpterr(wp,dwp/2.,norms[r]*sp,norms[r]*dsp,ch=2,ci=4)
          drawpt(wbins,sbins2/dbins,ps=1,ch=2,ci=1)
          drawpterr(wbins,dbins/2.,sbins2/dbins,ebins2/dbins,ch=2,ci=1)

          if inter > 0:
             curs = ["O"]
             while curs[0] != "g":
                if curs[0] in "qQI": os._exit(0)
                else: curs = getcurs(0,0)
          closedev()

    results = {}
    results["w"] = wbins
    results["dw"] = dbins
    results["s"] = sbins2*tt
    results["ds"] = ebins2*tt
    results["b"] = bbins2*tt
    results["tt"] = tt
    results["A"] = A

    D = asarray([wbins,dbins,sbins2,ebins2,bbins2])

    naxes = array(D.shape)[::-1].tolist()
    F = FITS("%s/%s.fits" % (output,name),create=1)
    F.newhead("T",-32,len(naxes),naxes)
    F.modkey("object", name)
    F.modkey("exptime", tt)
    F.modkey("aperture", A)
    F.modkey("mnorm", mnorm)
    [F.modkey("norm%d" % (j+1), "(%s,%f)" % (r,norms[r])) for j,r in enumerate(ro)]
    F.putdata(D)
    F.close()

    return results

def PullOneGalaxy(object,robjects,objno,nbin=1,output="Spectra",inter=False,scut=5,w1=0,w2=0):
   data = ReadOneObject(object,robjects,objno)
   FitOneObject(data,nbin=nbin,output=output,name=object,inter=inter,scut=scut,w1=w1,w2=w2)

