
import math
import time
from BC03 import *
from PhotoZ import *
from basics import *
from FITS import *
from pgplot import *
from VTKHelperFunctions import *
from Multipack import *
from sets import Set
from NumRec import *
from LDP import *
#from LDPFitting import *
from LDPGridModels  import *
from TwistedClient import *
from scipy.special import *
from Mestimators import *
from nnls import *
import cPickle

gX, dummy, gW= gauher(180,eps=3e-20)

class Target:
  def __init__(self,z,lam,flam,rescale=1.0,qe=None):
    self.z = z
    self.lam = lam * (1+z)
    self.flam = rescale * flam / (1+z)
    self.lam = concatenate([[self.lam[0]/2.0],self.lam,[self.lam[-1]*2.0]])
    self.flam = concatenate([[0.0],self.flam,[0.0]])
#    hw = 10500.0
#    self.flam = self.flam*less(self.lam,hw)
    self.qe = qe
    self.fqe = 1
    self.qo = 1
    if self.qe:
       self.qo = between(self.lam,self.qe.qe_wav[0],self.qe.qe_wav[-1],inc=1)
       self.fqe = self.qo*self.qe.interp(self.lam)
    self.ftck = splrep(self.lam,self.fqe*self.flam,s=0,k=3,xb=self.lam[0],xe=self.lam[-1])
  def Instrument(self,sigma,lams,dlams):
    wave = lams
    dwave = dlams
    dtck = splrep(wave,dwave,s=0,k=3)
    bin0 = wave[0]-dwave[0]/2.
    bins = [bin0]
    nbin = 0.5
    while bins[-1] < wave[-1]+nbin*dwave[-1]/2:
       bw0 = 1e38
       bw = bins[-1]
       while abs(bw-bw0)/bw > 1e-6:
          bw0 = bw
          dwdxb = splev(bw,dtck)
          bw = bins[-1]+nbin*dwdxb/2.
       bins.append( bins[-1]+nbin*dwdxb)
    wbins = array(bins)
    dbins = splev(wbins,dtck)

    S = array([splint(w-dw/2.,w+dw/2.,self.ftck)/dw for w,dw in zip(wbins,dbins)])
    S = VTKGauss(S,sigma/nbin,numret=1)
    stck = splrep(wbins,S,s=0,k=3)
    R = array([splint(w-dw/2.,w+dw/2.,stck)/dw for w,dw in zip(wave,dwave)])

#    S = array([ghquad(w,sigma*dw/nbin,self.ftck,gX,gW) for w,dw in zip(wbins,dbins)])
#    stck = splrep(wbins,S,s=0,k=3)
#    R = splev(wave,stck)

    if self.qe:
       qo = between(wave,self.qe.qe_wav[0],self.qe.qe_wav[-1],inc=1)
       fqe = qo*self.qe.interp(wave)
       R = divz(R,fqe)

    return R

def BCModelSpecAndMags(Name,MName,lams,dlams,logage1,metal1,redshifts,zstart,row0,sigma,curves,elines,qfile,csf=True):
   print logage1,metal1
   if qfile: qe = QE(qfile)
   else: qe = None
   Fout = FITS(Name,readonly=0)
   MFout = FITS(MName,readonly=0)
   template = BCTemplate(None,None)
   Filters = [GetFilter(curve[0],ab=curve[1]) for curve in curves]
   for jz,z1 in enumerate(redshifts):
       zfac = (1+z1)
       if zstart > 0:
           tstart = log10(tlook(zstart)-tlook(z1)) + 9.0
           tau = 400.0
           tend = logage1
       else:
           tstart = -zstart
           tau = 400.0
           tend = 5.0
       print z1,tstart,tend
       if tstart > tend:
          if csf: template.tau(tstart,tend,tau,metal1,ninc=64,emin=tend)
          else: template.lagelz(logage1,metal1)
          target = Target(z1, template.lam, template.flam, rescale=1e11, qe=qe)
          flam0 = target.Instrument(sigma,lams,dlams)
          flam = [flam0]
          ftemplate = FauxTemplate(template.lam,template.flam*1e11)
          magsout = [array([filt.integrate(ftemplate,z1,redfilt=curves[jc][2],logmult=1) for jc,filt in enumerate(Filters)])]
          for eline in elines:
              ewh1 = between(lams,4500.0,11000.0)
              slams,sdlams = compress(ewh1,[lams,dlams],1)
              dtck  = splrep(slams,log10(sdlams),s=0,k=3)
              if 4500.0 < zfac*eline < 11000.0:
                 dline = exp10(splev(zfac*eline,dtck))
                 ey = lambda ls: (ls-zfac*eline)/(sigma*dline)
                 eprof0 = lambda l1: exp(-0.5*ey(l1)**2)/sqrt(2*pi*(dline*sigma)**2)
                 eprof = array([quad(eprof0,lam1-dlam1/2.,lam1+dlam1/2.)[0] for lam1,dlam1 in zip(lams,dlams)])
                 eprof = less(abs(ey(lams)),8.0)*eprof/dlams
#                 eprof = less(abs(ey(lams)),5.0)*exp(-0.5*ey(lams)**2)/sqrt(2*pi*(sigma*dline)**2)
                 eflams = eprof
              else:
                 eflams = 0.0*flam0
              flam.append(eflams)

              emagsout = []
              if 4500 < zfac*eline < 11000.0:
                 elams = arange(4000.0,11000.0,1.0,Float64)
                 ey = (elams-zfac*eline)
                 eprof = exp(-0.5*ey**2)/sqrt(2*pi)
                 eflams = eprof
                 etemplate = FauxTemplate(elams,eflams)
                 emagsout = [filt.integrate(etemplate,z1,redfilt=curves[jc][2],logmult=1) for jc,filt in enumerate(Filters)]
              else:
                 emagsout = len(Filters)*[99.999]
              magsout.append(emagsout)
          magsout  = asarray( magsout )
       else:
          flam = (len(elines)+1)*[0.0*lams]
          magsout = (len(elines)+1)*[array([99.999 for filt in curves])]
       for hdu in range(len(elines)+1):
#          print "writing to",Fout.name(),hdu,os.getcwd(),row0+jz
          Fout.goto(hdu+1)
          Fout.writerow(flam[hdu],row0+jz)
          MFout.goto(hdu+1)
          MFout.writerow(magsout[hdu],row0+jz)
#          Fout.flush()
#          MFout.flush()
   Fout.close()
   MFout.close()
#   print Fout.name(),"closed."
#   print MFout.name(),"closed."

def AGNModelSpecAndMags(Name,MName,lams,dlams,redshift,row,sigma,curves,templates=None):
   Fout = FITS(Name,readonly=0)
   MFout = FITS(MName,readonly=0)
   if not templates:
      templates = [SDSSAGNTemplate(default=True,no3727=True)]
   Filters = [GetFilter(curve[0],ab=curve[1]) for curve in curves]
   z1 = redshift
   jz = row
   print z1
   for jt,template in enumerate(templates):
     target = Target(z1, template.lam, template.flam)
     flam = target.Instrument(sigma,lams,dlams)
     ftemplate = FauxTemplate(lams,flam)
     magsout = array([filt.integrate(template,z1,redfilt=curves[jc][2],logmult=0) for jc,filt in enumerate(Filters)])
#     fmagsout = array([filt.integrate(ftemplate,z1,redfilt=curves[jc][2],logmult=0) for jc,filt in enumerate(Filters)])
#     dmags = (fmagsout-magsout)
#     dmags = compress(between(fmagsout,-90,90),dmags)
#     dmags = median(dmags)
#     magsout = magsout + dmags
     Fout.writerow(flam,len(templates)*jz+jt)
     MFout.writerow(magsout,len(templates)*jz+jt)
   Fout.close()
   MFout.close()

def ldp_import(name,comps=None):
   print "importing",name
   mod = __import__(name)
   if comps == "*":
     for k in dir(mod):
       globals()[k] = getattr(mod,k)
   elif comps:
     for k in comps.split(","):
       globals()[k] = getattr(mod,k)
   else:
     globals()[name] = mod

def NonNegFit(p,j,lbases,ly,lwt,elines=0,phot=0,verb=0):
    ep = exp10(p)
    if elines:
        r = (ly[0]-dot(ep[:1+len(elines)],lbases[0][::,j]))*lwt[0]
    else:
        r = (ly[0]-ep[0]*lbases[0][j])*lwt[0]
    if phot:
       if elines:
          pp = exp10(p[1+len(elines)] + (p[:1+len(elines)] - p[0]))
          r2 = (ly[1]-dot(pp,lbases[1][::,j]))*lwt[1]
       else:
          r2 = (ly[1]-ep[1]*lbases[1][j])*lwt[1]
       r = concatenate([r,r2])
    if verb: print exp10(p)
    return r

def GetScales(b,y):
     scales = divz(add.reduce(b*y,0),add.reduce(b*b,0))
     return scales

def unnls(redshift,j,lwav,ldwav,lbases,ly,lerr,lwt,full_output=0,verb=0,elines=0,phot=0,lsols=None):
    if elines:
       m = lbases[0].shape[1]
       k = lbases[0].shape[0]
       if j and not (j % (m/80)): sys.stdout.write("."); sys.stdout.flush()
#       if elines[0] > 3000:
#          oz = 1 + redshift
#          nspec = len(lwav)
#          for ns in range(nspec):
#            lwav1,ldwav1 = compress(lwt[ns],[lwav[ns],ldwav[ns]],1)
#            wav0,wav1 = min(lwav1+2.5*ldwav1),max(lwav1-2.5*ldwav1)
#            wav1 = 7650
#            atmos = logical_or(greater(lwav[ns]-2*ldwav[ns],7650),less(lwav[ns]+2.0*ldwav[ns],7650))
#            for je in range(len(elines)):
#               if type(elines[je]) in [IntType,FloatType]:
#                  if not (wav0 < oz*elines[je] < wav1):
#                     lbases[ns][1+je,j] = zeros(lwav[ns].shape,lbases[ns].typecode())
#                  uwav = greater(oz*elines[je],wav0)*less(oz*elines[je],wav1)
#                  lbases[ns][1+je,j] = lbases[ns][1+je,j]*uwav
       if 0 and not phot and len(lwt) == 1:
          for mi in range(1):
             if mi==0:
                 mw = 1
             else:
                 if lsols: wsols = lsols
                 else: wsols = sols
                 r = (dot(wsols,lbases[0][::,j])-ly[0])*lerr[0]
                 if lsols:
                    rfix = compress(r,r)
                    if rfix:
                      rfix = bwt(rfix)[1]
                      if rfix: r = r/rfix
                 mw = Lp(r,w=1)
             mw = mw*lwt[0]*lerr[0]
             sols = nnls(lbases[0][::,j]*mw[NewAxis,::],ly[0]*mw)[1]
          sbs = [1]
          sols = concatenate([sols,sbs])
          return sols
       if 1 or (phot or len(lwt) > 1):
          qsol = nnls(lbases[0][::,j]*(lwt[0]*lerr[0])[NewAxis,::],ly[0]*(lwt[0]*lerr[0]))[1]
          qsed = lambda ns: dot(qsol,lbases[ns][::,j])
          sbs = array([nnls(asarray([qsed(ns)])*(lwt[ns]*lerr[ns])[NewAxis,::],ly[ns]*(lwt[ns]*lerr[ns]))[1][0] for ns in range(1,len(lwt))])
          sbs = concatenate([[1.0],sbs])
          nspec = len(lwav)
          sbw = sbs[:nspec]
          msbw = mean(compress(sbw,sbw))
          if msbw: sbs = sbs/msbw
#          sbs[:nspec] = divz(sbs[:nspec],msbw)
#          if phot:
#             sbs[nspec] = divz(sbs[nspec],msbw)

          bases = concatenate([sbs[ns]*lbases[ns][::,j] for ns in range(len(lwt))],1)
          y = concatenate(ly,0)
          e = concatenate(lerr,0)
          w = concatenate(lwt,0)
#          sbs = leastsq(trysb,sbs,args=(y,bases,w,e,lsols,1,))[0]
# TO DO:
#   First, do the fit to the LDP SED alone
#   Second, find the scale factor(s) that match those fitted SEDs to the photometry
#   Third, use those scale factors and then fit the data simultaneously
          sols = trysb(sbs,y,bases,w,e,lsols,f=0)
#          qsed = lambda ns: sbs[ns]*dot(sols,lbases[ns][::,j])
#          print array([nnls(asarray([qsed(ns)])*(lwt[ns]*lerr[ns])[NewAxis,::],ly[ns]*(lwt[ns]*lerr[ns]))[1][0] for ns in range(len(lwt))])
          sols = concatenate([sols,sbs])
          return sols
    else:
       sols0 = nnls((lbases[0][j]*(lwt[0]*lerr[0]))[NewAxis,::],ly[0]*(lwt[0]*lerr[0]))[1]
       if len(ly) > 1:
          sols0 = concatenate([sols0,nnls((lbases[1][j]*(lwt[1]*lerr[1]))[NewAxis,::],ly[1]*(lwt[1]*lerr[1]))[1]])
    return sols0

def trysb(sb,y,bases,w,e,lsols,f=1):
   if type(sb) == FloatType: sb = [sb]
   for mi in range(1):
      if mi==0:
          mw = 1
      else:
          if lsols: wsols = lsols
          else: wsols = sols
          fr = dot(wsols,bases)
          r = (fr-y)*e
          if lsols:
             rfix = compress(r,r)
             if rfix:
               rfix = bwt(rfix)[1]
               if rfix: r = r/rfix
          mw = Lp(r,w=1)
      mw = mw*w*e
      sols = nnls(bases*mw[NewAxis,::],y*mw)[1]
   if f: return w*(dot(sols,bases)-y)*e
   else: return sols

def NonNeg(j,lbases,ly,lwt,full_output=0,verb=0,elines=0,phot=0):
    if not sum(lbases[0][0,j]):
       sols = asarray(len(lbases[0][::,j])*[0.0])
       return sols
    if elines:
       sols0 = asarray([GetScales(b*lwt[0],ly[0]*lwt[0]) for b in lbases[0][::,j]])
       sols0[1:] = sols0[1:]
       if phot:
          sols1 = asarray([GetScales(b*lwt[1],ly[1]*lwt[1]) for b in lbases[1][:1,j]])
          sols0 = concatenate([sols0,sols1])
    else:
       sols0 = asarray([GetScales(lbases[0][j]*lwt[0],ly[0]*lwt[0]),
                        GetScales(lbases[1][j]*lwt[1],ly[1]*lwt[1])])

    rsols = getlog10(sols0,rep=-999.)
    usols = 1.0*greater(rsols,-999)
    if verb: print sols0
    if sum(usols):
#       nsols = mean(compress(usols,rsols))
#       sols = where(usols,rsols,nsols)
       sols = rsols
       psols = leastsq(NonNegFit,sols,args=(j,lbases,ly,lwt,elines,phot,verb),full_output=full_output)
       if full_output: gcov = GetC(psols)
       if verb:
          if full_output: print psols
          else: print psols
       gsols = psols[0]
       if len(shape(gsols)) == 0: gsols = asarray([gsols])
#       gsols = where(not_equal(gsols,sols),gsols,-99.0)
       bad = 0
    else:
       gsols = -99.0*ones(sols0.shape,Float64)
       gcov = -99.0*ones((sols0.shape[0],sols0.shape[0]),Float64)
    if elines and phot:
       mp = gsols[1+len(elines)] + (gsols[:1+len(elines)] - gsols[0])
       gsols = concatenate([gsols[:1+len(elines)],mp])
    if full_output:
      return gsols,gcov
    else:
      return exp10(gsols)

def FitOneGalaxy(template,ffile,phot,inter=False,lam1=5000.0,lam2=9000.0,individ=0):
    start = time.time()
    etemplate = eval(template)
    print "Fitting",ffile
    F = FITS(ffile)
    objname = F["object"].strip()
    exptime = F["exptime"]
    aperture = F["aperture"]
    data = F.data()
    norms = []
    while F["norm%d" % (len(norms)+1)] != "N/A": norms.append(F["norm%d" % (len(norms)+1)][1:-1].split(","))
    F.close()
    print [os.path.basename(norm[0]) for norm in norms]
    specs = [norm[0].replace("od","ox") for norm in norms]
    Fs = [FITS(spec) for spec in specs]
    sdata = [[F.data()[::,j,::] for j in range(F["naxis2"]) if objname == F["obj%04d" % (j+1)].strip()] for F in Fs]
    stimes = [2*F["exptime"] for F in Fs]
    [F.close() for F in Fs]
    specn = [float(norm[1]) for norm in norms]

    if "obj" in objname: number = int(objname[3:])
    elif "ref" in objname: number = int(objname[3:])
    else: number = int(objname[1:])
    if inter: pgdev = "/xs"
    else:
       odir = os.path.dirname(ffile)
       ofile = os.path.basename(ffile).replace(".fits","_"+template+".ps/vcps")
       pgdev = "/".join([odir,template,ofile])
       print "Writing",pgdev
    if template == "Galaxy":
      nx,ny = 3,3
    if template == "AGN":
      nx,ny = 3,3
    for it in range(1):
       qe = getattr(etemplate,"qe")
       qw = getattr(etemplate,"qw")
       seds = getattr(etemplate,"seds")
       mags = getattr(etemplate,"mags")
       curves = getattr(etemplate,"curves")
       ab = getattr(etemplate,"ab")
       pshape = getattr(etemplate,"pshape")
       params = getattr(etemplate,"params")
       redshifts = getattr(etemplate,"redshifts")
       allz = getattr(etemplate,"allz")
       if hasattr(etemplate,"Avs"): Avs = getattr(etemplate,"Avs")
       else: Avs = []
       if hasattr(etemplate,"elines"): elines = getattr(etemplate,"elines")
       else: elines = []

       w,dw,s,ds,bg = data
       ds = getsqrt(2*exptime*abs(s+bg+bg) + 2*2.*aperture*(6)**2)/(2.*exptime)

       s = divz(s,dw)
       ds = divz(ds,dw)
       bg = divz(bg,dw)
       sq = divz(s,qe)
       dsq = divz(ds,qe)
       bgq = divz(bg,qe)

       atmos = logical_or(greater(w-1.5*dw,7650),less(w+1.5*dw,7650))
       swt = 1.0*greater(w,lam1)*less(w,lam2)
#       swt = swt*atmos
#       swt = swt*sqrt(dw)
       swt = swt*divz(qe,mean(compress(swt,qe)))
#       swt = swt*divz(getsqrt(qe),mean(compress(swt,getsqrt(qe))))
#       swt = swt*divz(getsqrt(dw*qe),mean(compress(swt,getsqrt(dw*qe))))
#       swt = swt*divz(dw*qe,mean(compress(swt,dw*qe)))
#       swt = swt*divz(dw*qe,mean(dw*qe))

       if len(nonzero(swt)) < 50:
          print len(nonzero(swt))
          return False

       if not it: opendev(pgdev,nx=nx,ny=ny,size=nx*3.5,land=0,asp=1.0*ny/nx)
       qtck = splrep(qw,qe,s=0,k=3)

       if individ:
          ydat = []
          yerr = []
          ywt = []
          ymod = []
          ywav = []
          ydwav = []

          wtck = splrep(w,arange(len(w)),s=0,k=3)
          for spec1,sdat,snorm,stime in zip(specs,sdata,specn,stimes):
             if 1:
                print spec1,stime,snorm
                ss1, se1, sb1, sw1, sd1 = sdat[0]
                sw1,sd1,ss1,se1 = compress(greater(sw1,lam1)*less(sw1,lam2),[sw1,sd1,ss1,se1],1)
                ss1 = where(between(ss1,1e-24,1e24),ss1,0.0)
                se1 = where(between(se1,1e-24,1e24),se1,0.0)
                sqe1 = splev(sw1,qtck)
                sq1 = divz(ss1,sd1*sqe1)/stime
                dsq1 = divz(se1,sd1*sqe1)/stime
#                print "se",sort(se1).tolist()
#                print "sd",sort(sd1).tolist()
#                print "sqe",sort(sqe1).tolist()
#                print "dsq",sort(dsq1).tolist()
                sa1 = logical_or(greater(sw1-sd1,7750),less(sw1+sd1,7650))
                swt1 = 1.0*sa1
#                print "sw",sort(swt1).tolist()
                ydat.append(sq1*1e20)
                yerr.append(divz(1,dsq1*1e20))
                ywt.append(swt1)
                xsed = splev(sw1,wtck)
                xsed0 = clip(xsed.astype(Int32),0,seds.shape[1])
                xsed1 = clip(xsed0+1,0,seds.shape[1])
                dxsed = xsed-xsed0
                seds0 = take(seds,xsed1,-1)
                seds1 = take(seds,xsed0,-1)
                sfrac = divz(dxsed,xsed1-xsed0)
                if elines: sfrac = sfrac[NewAxis,NewAxis,::]
                else: sfrac = sfrac[NewAxis,::]
                useds = (seds1-seds0)*sfrac + seds0
                ymod.append(useds)
                ywav.append(sw1)
                ydwav.append(sd1)
          print "Concatenating....",;sys.stdout.flush()
          print "done."

       else:

          ydat = [sq*1e20]
          yerr = [divz(1,dsq*1e20)]
          ywt = [swt]
          ymod = [seds]
          ywav = [w]
          ydwav = [dw]

       if phot:
          fs = phot["mag"].keys()
          cs = [phot["defs"][f] for f in fs]
          ci = [curves.index(c[0]) for c in cs]
          gs = [GetFilter(c[0],c[1],ab=c[2]) for jc,c in enumerate(cs)]
          wl  = array([g.wleff for g in gs])
          wb  = array([g.bandpass1 for g in gs])
          ab  = array([g.conv_AB for g in gs])
          fconv = (CLIGHT*1e8)/wl**2
          print fs
          print ab
          print ci
          uap = getattr(etemplate,"uap")
          if hasattr(etemplate,"uapir"): uap2 = getattr(etemplate,"uapir")
          else: uap2 = None
          magt = array([phot["mag"][f]["mag%d" % (4)] for f in fs])
          mago = array([phot["mag"][f]["mag%d" % (uap)] for f in fs])
          erro = array([phot["mag"][f]["err%d" % (uap)] for f in fs])
          if uap2:
             mago2 = array([phot["mag"][f]["mag%d" % ((f in uap2[1:-1] and uap2[-1]) or uap)] for f in fs])
             erro2 = array([phot["mag"][f]["err%d" % ((f in uap2[1:-1] and uap2[-1]) or uap)] for f in fs])
             fixap = array([(f in uap2[1:-1]) for f in fs])
             fixmag = phot["mag"][uap2[0]]["mag%d" % (uap)] - phot["mag"][uap2[0]]["mag%d" % (uap2[-1])]
             mago = where(fixap,mago2+fixmag,mago)
             erro = where(fixap,erro2,erro)
          erro = sqrt(erro**2 + 0.020**2)
          umag = between(mago,0,29) * less(erro,9)
          print magt
          print umag
          print mago
          print erro
          fluxo = fconv * exp10(-0.4*(mago+ab+48.60))
          sigo = (erro/1.0857)*fluxo
          fluxo = where(between(mago,-90,90),fluxo,0.0)
          sigo = where(between(mago,-90,90),sigo,0.0)
          mwt = umag
#          mwt = mwt * divz(wb,mean(wb))
          print fluxo
          print sigo

          if elines:
             print "Fit magnitudes including these extra lines",elines
             magm = take(mags,ci,2)
          else:
             magm = take(mags,ci,1)
          goodm = between(magm,-90,90)
          fluxes = goodm * fconv * exp10(-0.4*(magm+ab+48.60))

          ydat.append(fluxo*1e20)
          yerr.append(divz(1,sigo*1e20))
          ywt.append(mwt)
          ymod.append(fluxes)

#       exit()
       if elines:
          lsols = None
          for fiter in range(1):
              print "Fit LDP spectrum including these extra lines",elines
              gsols = asarray([unnls(allz[j],j,ywav,ydwav,ymod,ydat,yerr,ywt,elines=elines,phot=phot,verb=0,lsols=lsols) for j in range(ymod[0].shape[1])])
              nspec = len(ywav)
              srescales = []
              nol = 1+len(elines)
              for ns in range(nspec):
                 g = asarray([gsols[j,nol+ns]*dot(gsols[j,:nol],ymod[ns][::,j,::]) for j in range(ymod[ns].shape[1])])
                 subtract(g,ydat[ns][NewAxis,::],g)
                 multiply(g,yerr[ns][NewAxis,::],g)
                 g = Lp(g)
                 multiply(g,ywt[ns][NewAxis,::],g)
                 x2 = add.reduce(g,1)
                 ndeg = len(nonzero((ywt[ns]*yerr[ns]))) - len(pshape) - 1 - len(elines)
                 extras = len(gsols)
                 srescales.append( gsols[::,ns*nol])
              if len(ywav) > 1: srescale = median(compress(specn,srescales,0))
              else: srescale = srescales[0]
              if phot:
                 g = asarray([gsols[j,nol+nspec]*dot(gsols[j,:nol],ymod[nspec][::,j,::]) for j in range(ymod[0].shape[1])])
                 mrescale = gsols[::,nol+nspec]
                 ns = 0
                 subtract(g,ydat[nspec][NewAxis,::],g)
                 multiply(g,yerr[nspec][NewAxis,::],g)
                 g = Lp(g)
                 multiply(g,ywt[nspec][NewAxis,::],g)
                 x2 = x2 + add.reduce(g,1)
                 ndeg = ndeg - 1
              mx2 = argmin(x2)
              lsols = gsols[mx2,:nol]
       else:
          gsols = asarray([unnls(allz[j],j,ywav,ydwav,ymod,ydat,yerr,ywt,elines=0,phot=phot,verb=0) for j in range(ymod[0].shape[0])])
          g = gsols[::,0][::,NewAxis]*ymod[0]
          subtract(g,ydat[0][NewAxis,::],g)
          multiply(g,yerr[0][NewAxis,::],g)
          g = Lp(g)
          multiply(g,ywt[0][NewAxis,::],g)
          sx2 = add.reduce(g,1)
          x2 = sx2
          ndeg = len(nonzero((ywt[0]*yerr[0]))) - len(pshape) - 1
          srescale = gsols[::,0]/1e20
          if phot:
             g = gsols[::,1][::,NewAxis]*ymod[1]
             subtract(g,ydat[1][NewAxis,::],g)
             multiply(g,yerr[1][NewAxis,::],g)
             g = Lp(g)
             multiply(g,ywt[1][NewAxis,::],g)
             sx2 = add.reduce(g,1)
             x2 = x2 + sx2
             ndeg = ndeg + len(nonzero((ywt[1]*yerr[1]))) - 1
             if phot: mrescale = gsols[::,1]/1e20
#       x2 = where((allz>0.811)*(allz<0.86),x2,1e9)
#       x2 = where((allz>4.91),x2,1e9)
       x2 = 2*x2
       x2 = reshape(x2,pshape)
       if phot: mfacs = reshape(mrescale,pshape)
       else: mfacs = ones(pshape,Float32)
       if hasattr(etemplate,"bad"):
          bad = reshape(getattr(etemplate,"bad"),pshape)
          x2 = x2 + 1e24*bad
       minm = argmin(x2.flat)
       minz = allz[minm]
       minchi2 = x2.flat[minm]
       srescaled = reshape(srescale,pshape)
       if phot: mrescaled = reshape(mrescale,pshape)
#       print sort(x2.flat)[:100]
#       print sort(x2.flat)[-100:]
       dx2 = x2 - minchi2
       p = exp(-0.5*dx2)
       z0 = getattr(etemplate,"z0")
       z1 = getattr(etemplate,"z1")
       dz = getattr(etemplate,"dz")
       if template == "Galaxy":

          k20 = array([ 0.18009532,-3.19084644, 0.61086963, 0.54542968, 0.31197063,])
          k20p = k20[0]*cosh((allz-k20[1])/k20[2])*exp(-0.5*(allz-k20[3])**2/k20[4]**2)
          k20p = reshape(k20p,pshape)

          logage0 = getattr(etemplate,"logage0")
          logage1 = getattr(etemplate,"logage1")
          dlogage = getattr(etemplate,"dlogage")
          allt = reshape(getattr(etemplate,"allt"),pshape)
          metal0 = getattr(etemplate,"metal0")
          metal1 = getattr(etemplate,"metal1")
          dmetal = getattr(etemplate,"dmetal")
          allm = reshape(getattr(etemplate,"allm"),pshape)
          print allt.shape
          tp = k20p
          tp = tp * less(reshape(allz,pshape),1.2)
          tp = tp * exp10(allt-9.0)
          p = p * tp
          print p.shape
          zchi2 = minimum.reduce(minimum.reduce(x2,1),0)
          zprob = add.reduce(add.reduce(p,1),0)*dlogage*dmetal
          ztprob = add.reduce(p,0)*dmetal
          zmprob = add.reduce(p,1)*dlogage
          tmprob = add.reduce(p,2)*dz
          ztprob = divz(ztprob,sum(sum(ztprob*dz)*dlogage))*dz*dlogage
          zmprob = divz(zmprob,sum(sum(zmprob*dz)*dmetal))*dz*dmetal
          tmprob = divz(tmprob,sum(sum(tmprob*dmetal)*dlogage))*dmetal*dlogage
          zprob = divz(zprob,sum(zprob*dz))*dz
          lzprob = getlog10(zprob)
          tprob = add.reduce(add.reduce(p,2),0)*dz*dmetal
          mprob = add.reduce(add.reduce(p,2),1)*dz*dlogage
          sr = array([ravel(srescaled[::,::,l1])[argmax(ravel(p[::,::,l1]))] for l1 in range(len(redshifts))])
          if phot:
             mr = sr*array([ravel(mrescaled[::,::,l1])[argmax(ravel(p[::,::,l1]))] for l1 in range(len(redshifts))])
       if template == "AGN":
          zprob = add.reduce(p,1)
          zchi2 = minimum.reduce(x2,1)
          zprob = divz(zprob,sum(zprob*dz))*dz
          lzprob = getlog10(zprob)
          sr = srescaled.flat
          if phot:
             mr = mrescaled.flat*sr

       ldil = 4*pi*(CMPMPC*mod2mpc(cosmomod(redshifts)))**2
       lsr = getlog10(sr)
       if phot:
          lmr = getlog10(mr)

       maxp = argmax(p.flat)

       maxz = allz[maxp]
       print "z=",minz,maxz
       cz = add.accumulate(zprob)
       ic = clip(searchsorted(cz,[0.025,0.975]),0,len(redshifts)-1)
       print take(redshifts,ic)
       print take(cz,ic)
       print ConfidenceLimits(redshifts,zprob)
       print 
       print "Min chi^2=",minchi2,x2.flat[maxp]
       print "N degrees=",ndeg
       print "Min reduced chi^2=",divz(minchi2,ndeg),divz(x2.flat[maxp],ndeg)
       print gsols[maxp]
       gsols[::,:nol] = gsols[::,:nol]/1e20
       print gsols[maxp]

       xm,ym = [redshifts[0],redshifts[-1]],[max([0,min(zchi2)-ndeg]),median(compress(less(zchi2,1e24),zchi2))]
       newbox(xm,ym,ch=1.8)
       xlabel("z",ch=1.8)
       ylabel("\gx\u2\d",ch=1.8)
       drawln(redshifts,zchi2)
       drawln([minz,minz],ym,ls=2)

       xm,ym = [redshifts[0],redshifts[-1]],[max(lzprob)-5,max(lzprob)+0.3]
       newbox(xm,ym,ch=1.8)
       xlabel("z",ch=1.8)
       ylabel("P(z)dz",ch=1.8)
       drawln(redshifts,lzprob)
       drawln([minz,minz],ym,ls=2)

       if template == "Galaxy":
          xm,ym = [redshifts[0],redshifts[-1]],extrema(lsr)
          newbox(xm,ym,ch=1.8,rax=(phot==None))
          xlabel("z",ch=1.8)
          ylabel("S",ch=1.8)
          drawln(redshifts,lsr,ls=2,ci=1)
          drawln([minz,minz],ym,ls=4)
          drawln([maxz,maxz],ym,ls=2)
          if phot:
             xm,ym = [redshifts[0],redshifts[-1]],extrema(lsr)+median(lmr-lsr)
             newbox(xm,ym,ch=1.8,era=0,lax=0,rax=1)
             xlabel("z",ch=1.8)
             drawln(redshifts,lmr,ls=1,ci=4)
       else:
         cpgpage()

       plam1, plam2 = lam1 - 500, lam2 + 500

       us,uds = compress(swt,[s,ds],1)
       xm,ym = [plam1,plam2],[-0.3*getpct(us+2*uds,95),1.5*getpct(us+2*uds,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("DN/s/\A",ch=1.8)
       drawpt(w,s,ps=17,ci=14,ch=1)
       drawpterr(w,dw/2,s,ds,ci=14,ps=17,ch=1.8)
       wp,dwp,sp,dsp = compress(swt,[w,dw,s,ds],1)
       drawpt(wp,sp,ps=17,ch=1)
       drawpterr(wp,dwp/2,sp,dsp,ps=17,ch=1.8)
       if elines:
          ugsols = gsols[maxp,:nol]
          g = qe*dot(ugsols,seds[::,maxp,::])
          drawln(w,g,ci=2)
       else:
          g = qe*seds[maxp]
          drawln(w,g,ci=2)

       sq = divz(s,qe)
       dsq = divz(ds,qe)
       bgq = divz(bg,qe)
       uw,usq,udsq,ubgq = compress(swt,[w,sq,dsq,bgq],1)

       xm,ym = [plam1,plam2],[-0.3*getpct(usq+2*udsq,95),1.5*getpct(usq+2*udsq,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       for ns,sdat,snorm,stime in zip(range(len(stimes)),sdata,specn,stimes):
          if 1:
             ss1, se1, sb1, sw1, sd1 = sdat[0]
             sw1,sd1,ss1,se1 = compress(sw1,[sw1,sd1,ss1,se1],1)
             if len(sw1):
                ss1 = where(between(ss1,1e-24,1e24),ss1,0.0)
                se1 = where(between(se1,1e-24,1e24),se1,0.0)
                sqe1 = array([splev(sw11,qtck) for sw11 in sw1])
                if len(ywav) > 1:
                   ugsols = gsols[maxp,:nol]
                   usc = divz(srescale[maxp],srescales[ns][maxp])
                else:
                   ugsols = snorm * gsols[maxp,:nol]
                   usc = snorm
                g = qe*dot(ugsols,seds[::,maxp,::])
                if usc:
                   sq1 = usc*divz(ss1,sd1*sqe1)/stime
                   dsq1 = usc*divz(se1,sd1*sqe1)/stime
                   drawpt(sw1,sq1,ps=1,ch=1,ci=1)
                   drawpterr(sw1,sd1/2,sq1,dsq1,ps=17,ch=1.8,ci=1)
       if elines:
          ugsols = gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          drawln(w,g,ci=2)
       else:
          g = seds[maxp]
          drawln(w,g,ci=2)

       mfac = 1.0
       if phot:
          mfac = mrescale[maxp]
       uw,usq,udsq,ubgq = compress(swt*greater(w,5000),[w,sq,dsq,bgq],1)
       print mfac

       xm,ym = [plam1,plam2],[-0.3*mfac*getpct(ubgq+2*udsq,95),1.5*mfac*getpct(ubgq+2*udsq,95)]
       newbox(xm,ym,ch=1.8)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       if individ:
        for ns,sdat,snorm,stime in zip(range(len(stimes)),sdata,specn,stimes):
          if 1:
             ss1, se1, sb1, sw1, sd1 = sdat[0]
             sw1,sd1,ss1,se1 = compress(sw1,[sw1,sd1,ss1,se1],1)
             ss1 = where(between(ss1,1e-24,1e24),ss1,0.0)
             se1 = where(between(se1,1e-24,1e24),se1,0.0)
             sqe1 = splev(sw1,qtck)
             if len(ywav) > 1:
                ugsols = gsols[maxp,:nol]
                usc = divz(srescale[maxp],srescales[ns][maxp])
             else:
                ugsols = snorm * gsols[maxp,:nol]
                usc = snorm
             g = qe*dot(ugsols,seds[::,maxp,::])
             sq1 = mfac*usc*divz(ss1,sd1*sqe1)/stime
             dsq1 = mfac*usc*divz(se1,sd1*sqe1)/stime
             if usc:
                drawpt(sw1,sq1,ps=1,ch=1,ci=1)
                drawpterr(sw1,sd1/2,sq1,dsq1,ps=17,ch=1.8,ci=1)
       else:
          drawln(w,mfac*bgq,ci=8)
          drawpt(w,mfac*sq,ci=14,ps=17,ch=1)
          drawpterr(w,dw/2,mfac*sq,mfac*dsq,ci=14,ps=17,ch=1.8)
          wp,dwp,sqp,dsqp,bgqp = compress(swt,[w,dw,sq,dsq,bgq],1)
          drawpt(wp,mfac*sqp,ps=17,ch=1)
          drawpterr(wp,dwp/2,mfac*sqp,mfac*dsqp,ps=17,ch=1.8)
       if phot:
          drawpt(wl,fluxo,ps=17,ci=[3,7][inter])
          drawpterr(wl,wb/2.,fluxo,sigo,ps=17,ci=[3,7][inter])
       if elines:
          ugsols = mfac * gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          if not Avs: drawln(w,g,ci=2)
          if Avs:
             drawln(w,g,ci=4)
             ugsols = mfac * gsols[maxp,:nol]
             g = dot(ugsols,seds[:nol,maxp,::])
             drawln(w,g,ci=2)
       else:
          g = seds[maxp]
          drawln(w,g,ci=2)

       if phot:
          plam1 = min([plam1,min(wl-wb)])
          plam2 = max([plam2,max(wl+wb)])
       xm,ym = [plam1,plam2],[-0.3*mfac*getpct(usq+2*udsq,95),1.5*mfac*getpct(usq+2*udsq,95)]
       newbox(log10(xm),ym,ch=1.8,xlog=1)
       xlabel("\gl (\A)",ch=1.8)
       ylabel("F\d\gl\u (ergs s\u-1\d cm\u-2\d \A\u-1\d)",ch=1.8)
       if individ:
        for ns,sdat,snorm,stime in zip(range(len(stimes)),sdata,specn,stimes):
          if 1:
             ss1, se1, sb1, sw1, sd1 = sdat[0]
             sw1,sd1,ss1,se1 = compress(sw1,[sw1,sd1,ss1,se1],1)
             ss1 = where(between(ss1,1e-24,1e24),ss1,0.0)
             se1 = where(between(se1,1e-24,1e24),se1,0.0)
             sqe1 = splev(sw1,qtck)
             if len(ywav) > 1:
                ugsols = gsols[maxp,:nol]
                usc = divz(srescale[maxp],srescales[ns][maxp])
             else:
                ugsols = snorm * gsols[maxp,:nol]
                usc = snorm
             g = qe*dot(ugsols,seds[::,maxp,::])
             sq1 = mfac*usc*divz(ss1,sd1*sqe1)/stime
             dsq1 = mfac*usc*divz(se1,sd1*sqe1)/stime
             if usc:
                drawpt(log10(sw1),sq1,ps=1,ch=1,ci=1)
                drawpterr(log10(sw1),sd1/2/sw1/2.303,sq1,dsq1,ps=17,ch=1.8,ci=1)
       else:
          drawpt(log10(w),mfac*sq,ci=14,ps=17,ch=1)
          drawpterr(log10(w),dw/2/w/2.303,mfac*sq,mfac*dsq,ci=14,ps=17,ch=1.8)
          wp,dwp,sqp,dsqp,bgqp = compress(swt,[w,dw,sq,dsq,bgq],1)
          drawln(log10(wp),mfac*bgqp,ci=8)
          drawpt(log10(wp),mfac*sqp,ps=17,ch=1)
          drawpterr(log10(wp),dwp/2/wp/2.303,mfac*sqp,mfac*dsqp,ps=17,ch=1.8)
       if phot:
          drawpt(log10(wl),fluxo,ps=17,ci=[3,7][inter])
          drawpterr(log10(wl),wb/2./wl/2.303,fluxo,sigo,ps=17,ci=[3,7][inter])
          ugsols = gsols[maxp,:nol]
          g = mfac*dot(ugsols,fluxes[::,maxp,::])
          drawpt(log10(wl),g,ps=17,ci=6)
       if elines:
          ugsols = mfac * gsols[maxp,:nol]
          g = dot(ugsols,seds[::,maxp,::])
          if not Avs: drawln(log10(w),g,ci=2)
          if Avs:
             drawln(log10(w),g,ci=4)
             ugsols = mfac * gsols[maxp,:nol]
             g = dot(ugsols,seds[:nol,maxp,::])
             drawln(log10(w),g,ci=2)
       else:
          g = seds[maxp]*srescale[maxp]
          drawln(log10(w),g,ci=2)


       if template == "Galaxy":

          xm,ym = [z0,z1+dz],[logage0,logage1+dlogage]
          newbox(xm,ym,ch=1.8)
          xlabel("z",ch=1.8)
          ylabel("log t\dend\u",ch=1.8)
          xfm = array([z0-dz/2.,dz,0.0,logage0-dlogage/2.,0.0,dlogage])
          am = max(ztprob.flat)
          if inter: a1,a2 = -0.5*am,0.5*am
          else: a1,a2 = 0.5*am,-0.5*am
          cpggray(ztprob,ztprob.shape[1],ztprob.shape[0],1,ztprob.shape[1],1,ztprob.shape[0],a1,a2,xfm)

#          xm,ym = [z0,z1+dz],[metal0,metal1+dmetal]
#          newbox(xm,ym,ch=1.8)
#          xlabel("z",ch=1.8)
#          ylabel("[Fe/H]",ch=1.8)
#          xfm = array([z0-dz/2.,dz,0.0,metal0-dmetal/2.,0.0,dmetal])
#          am = max(zmprob.flat)
#          if inter: a1,a2 = -0.5*am,0.5*am
#          else: a1,a2 = 0.5*am,-0.5*am
#          cpggray(zmprob,zmprob.shape[1],zmprob.shape[0],1,zmprob.shape[1],1,zmprob.shape[0],a1,a2,xfm)

          xm,ym = [logage0,logage1+dlogage],[metal0,metal1+dmetal]
          newbox(xm,ym,ch=1.8)
          xlabel("log t\dend\u",ch=1.8)
          ylabel("[Fe/H]",ch=1.8)
          xfm = array([logage0-dlogage/2.,dlogage,0.0,metal0-dmetal/2.,0.0,dmetal])
          am = max(tmprob.flat)
          if inter: a1,a2 = -0.5*am,0.5*am
          else: a1,a2 = 0.5*am,-0.5*am
          cpggray(tmprob,tmprob.shape[1],tmprob.shape[0],1,tmprob.shape[1],1,tmprob.shape[0],a1,a2,xfm)

    closedev()

    odir = os.path.dirname(ffile)
    ofile = os.path.basename(ffile).replace(".fits","_"+template+".fits")
    ofile = "/".join([odir,template,ofile])
    print "Writing",ofile

    F = FITS(ofile,create=1,template=ffile)
    naxes = list(asarray(pshape)[::-1])
    F.resize(naxes)

    F.modkey("minchi2",minchi2)
    F.modkey("ndeg",ndeg)
    F.modkey("lam1",lam1)
    F.modkey("lam2",lam2)
    [F.modkey(par,params[par]) for par in params]
    F.putdata(x2)

    F.addhdu()
#    ugsols = divz(srescale[::,NewAxis]*gsols[::,:nol],srescales[0][::,NewAxis])
    ugsols = gsols[::,:nol]
    print ugsols[maxp]
    gsolsb = asarray([reshape(gs,pshape) for gs in transpose(ugsols)])
    gnaxes = asarray(gsolsb.shape)[::-1].tolist()
    F.newhead("T",-32,len(gnaxes),gnaxes)
    F.modkey("object","gsols")
    if phot: F.modkey("phot",1)
    else: F.modkey("phot",0)
    if hasattr(etemplate,"elines"):
       elines = getattr(etemplate,"elines")
       for jline,eline in enumerate(elines):
          if eline: F.modkey("eline%d" % (jline),eline)
    F.putdata(gsolsb)

    if phot:
       F.addhdu()
       F.newhead("T",-32,len(naxes),naxes)
       F.modkey("object","mrescale")
       F.putdata(mfacs)

    F.close()

    finish = time.time()
    print "Time = %.3fs" % (finish-start)

def SelectGalaxy(template,select,ofile,ffile,phot,inter=False,twistout=False):
    if type(template) == StringType: etemplate = eval(template)
    elif type(template) == ModuleType: etemplate,template = template,template.__name__
    if select:
       if type(select) == StringType: eselect = eval(select)
       elif type(select) == ModuleType: eselect,select = select,select.__name__
    print ffile,template,select
    info = ObjInfo(ofile)
    F = FITS(ffile)
    minchi2 = F["minchi2"]
    ndeg = F["ndeg"]
    x2 = F.data()
    F.goto(2)
    uphot = F["phot"]
    gsolsb = F.data()
    if phot and uphot:
      F.goto(3)
      mrescale = F.data()
    F.close()
    qe = getattr(etemplate,"qe")
    qw = getattr(etemplate,"qw")
    seds = getattr(etemplate,"seds")
    mags = getattr(etemplate,"mags")
    curves = getattr(etemplate,"curves")
    ab = getattr(etemplate,"ab")
    pshape = getattr(etemplate,"pshape")
    params = getattr(etemplate,"params")
    redshifts = getattr(etemplate,"redshifts")
    allz = getattr(etemplate,"allz")
    if hasattr(etemplate,"elines"): elines = getattr(etemplate,"elines")
    else: elines = []
    if select and hasattr(eselect,"prior"): priors = getattr(eselect,"prior")
    else: priors = None
    z0 = getattr(etemplate,"z0")
    z1 = getattr(etemplate,"z1")
    dz = getattr(etemplate,"dz")
    mx2 = min(x2.flat)
    p = exp(-0.5*(x2-mx2))
    srescale = gsolsb[0]
    print gsolsb.shape
    if phot and uphot:
       gsolsb  = concatenate([gsolsb,[mrescale]])
       print gsolsb.shape
    if elines:
       erescales = gsolsb[1:len(elines)+1]
    if template == "Galaxy":
       logages = getattr(etemplate,"logages")
       logage0 = getattr(etemplate,"logage0")
       logage1 = getattr(etemplate,"logage1")
       dlogage = getattr(etemplate,"dlogage")
       allt = reshape(getattr(etemplate,"allt"),pshape)
       metals = getattr(etemplate,"metals")
       metal0 = getattr(etemplate,"metal0")
       metal1 = getattr(etemplate,"metal1")
       dmetal = getattr(etemplate,"dmetal")
       allm = reshape(getattr(etemplate,"allm"),pshape)
       tp = ones(p.shape) * exp10(allt-9.0)
       p = p * tp
       if priors: p = priors * p
       pmax = argmax(p.flat)
       print "max:"
       zmax = allz.flat[pmax]
       print zmax
       pgs = array([gsolsb1.flat[pmax] for gsolsb1 in gsolsb])
#       pgs = gsolsb*p
#       pgs1 = add.reduce(add.reduce(add.reduce(pgs,3),2),1)*dz*dlogage*dmetal
#       pgs2 = add.reduce(add.reduce(add.reduce(p,2),1),0)*dz*dlogage*dmetal
#       pgs = divz(pgs1,pgs2)
       zchi2 = minimum.reduce(minimum.reduce(x2,1),0)
       zprob = add.reduce(add.reduce(p,1),0)*dlogage*dmetal
       ztprob = add.reduce(p,0)*dmetal
       zmprob = add.reduce(p,1)*dlogage
       tmprob = add.reduce(p,2)*dz
       ztprob = divz(ztprob,sum(sum(ztprob*dz)*dlogage))*dz*dlogage
       zmprob = divz(zmprob,sum(sum(zmprob*dz)*dmetal))*dz*dmetal
       tmprob = divz(tmprob,sum(sum(tmprob*dmetal)*dlogage))*dmetal*dlogage
       zprob = divz(zprob,sum(zprob*dz))*dz
       lzprob = getlog10(zprob)
       tprob = add.reduce(add.reduce(p,2),0)*dz*dmetal
       mprob = add.reduce(add.reduce(p,2),1)*dz*dlogage
    if template == "AGN":
       pmax = argmax(p.flat)
       zprob = add.reduce(p,1)
       zchi2 = minimum.reduce(x2,1)
       zprob = divz(zprob,sum(zprob*dz))*dz
       lzprob = getlog10(zprob)
       pgs = gsolsb*p
       pgs1 = add.reduce(add.reduce(pgs,2),1)*dz
       pgs2 = add.reduce(add.reduce(p,1),0)*dz
       pgs = divz(pgs1,pgs2)

    cls = {}
    cls["minchi2"] = minchi2
    cls["ndeg"] = ndeg
    cls["pgs"] = pgs
    cls["ags"] = transpose([PConfidenceLimits(gsolsb1.flat,p.flat) for gsolsb1 in gsolsb])
    cls["pmax"] = pmax
    cls["zmax"] = allz.flat[pmax]

    zconf = ConfidenceLimits(redshifts,zprob)
    print ffile
    cls["redshift"] = zconf
    print zconf
    if template == "Galaxy":
       tconf = ConfidenceLimits(logages,tprob)
       cls["logage"] = tconf
       print tconf
       mconf = ConfidenceLimits(metals,mprob)
       cls["metal"] = mconf
       print mconf

    if twistout:
       print "Returning pickled results"
       return cPickle.dumps(cls)
    else:
       return cls

def ConfidenceLimits(x,y,k=3):
    x,y = take([x,y],argsort(x),1)
    tck = splrep(x,y,s=0,k=3)
    g = lambda x1: splint(0,x1,tck)
    iy = array([g(x1) for x1 in x])
    if iy[-1]: iy = iy / iy[-1]
#    tck = splrep(x,iy,s=0,k=k)
    ps = [0.025,0.05,0.16,0.5,0.84,0.95,0.975]
    tx = concatenate([k*[x[0]],x,k*[x[-1]]])
    g = lambda r,p: splev(r,[tx,iy,k])-p
    cs = [fsolve(g,[x[clip(searchsorted(iy,p),0,len(x)-1)]],args=(p,)) for p in ps]
    m = argmax(y)
    p = x[m]
    return concatenate([[p],cs])

def PConfidenceLimits(x,y,k=3):
    x,y = compress(y,[x,y],1)
    x,y = take([x,y],argsort(x),1)
    iy = add.accumulate(y)
    if iy[-1]: iy = iy / iy[-1]
    sx,sy = compress(x,[x,y],1)
    if sx and sum(sy): s = 1e-4*exp10(sum(sy*log10(sort(sx)))/sum(sy))
    else: s = 1e-6
    x,iy = adjspac([x,iy],s=s,verb=1)
#    t,c,k = splrep(x,iy,s=0,k=3)
    ps = [0.025,0.05,0.16,0.5,0.84,0.95,0.975]
    tx = concatenate([k*[x[0]],x,k*[x[-1]]])
    g = lambda r,p: splev(r,[tx,iy,k])-p
    cs = [fsolve(g,[x[clip(searchsorted(iy,p),0,len(x)-1)]],args=(p,)) for p in ps]
    m = argmax(y)
    p = x[m]
    return concatenate([[p],cs])

def adjspac(A,s=1e-6,extras=None,verb=1):
   x = A[0]
   o = argsort(x)
   A = take(A,o,1)
   if extras: extras = [take(extra,o) for extra in extras]
   x = A[0]
   dx = diff(x)
   while sometrue(equal(dx,0)):
      if verb: print "adjusting spacings...", sum(equal(dx,0.0))
      p = concatenate([[0.0],equal(dx,0)])
      x = x + s * p * (random(p.shape)-0.5)
      A[0] = x
      o = argsort(x)
      A = take(A,o,1)
      if extras: extras = [take(extra,o) for extra in extras]
      x = A[0]
      dx = diff(x)
   if extras: return A,extras
   else: return A



def ObjInfo(ffile):
    F = FITS(ffile)
    objname = F["object"].strip()
    exptime = F["exptime"]
    aperture = F["aperture"]
    F.close()
    if "obj" in objname: n = int(objname[3:])
    elif "ref" in objname: n = int(objname[3:])
    else: n = int(objname[1:])
    info = {"exptime":exptime,"aperture":aperture,"n":n,"object":objname}
    return info

