
from basics import *
from sets import Set
from LDP import *
from LDPData import *
from FITS import *
import os


class LDPDataset:
   def __init__(self,root,smf,rawdir,mask,spec,lamp,ccds,saturate,standard=False,alternate=None,flatself=True,quiet=False):
      self.smf = smf
      self.quiet = quiet
      self.root = root
      self.kinds = "mask","spec","lamp"
      self.rawdir = rawdir
      self.mask = asarray(mask)
      self.spec = asarray(spec)
      self.lamp = asarray(lamp)
      self.ccds = asarray(ccds)
      self.saturate = saturate
      self.standard = standard
      self.flatself = flatself
      self.cwd = os.getcwd()
      self.GetPosition(alternate)
      self.Queue = []
      self.Outputs = []
   def rawf(self,f,c,gz=True):
      f = "ccd%04dc%d.fits" % (f,c)
      if gz: f = f + ".gz"
      return f
   def origf(self,f,c,loc=None,gz=True):
      if not loc:
         loc = self.rawdir
         b = "/".join([loc,self.rawf(f,c,gz=gz)])
      else:
         b = self.rawf(f,c,gz=gz)
      s = glob(b)
      if s: return s[0]
      else: return b
   def gflat(self,c):
      return "gflatc%d.fits" % (c)
   def dataf(self,kind,m,c,suf=""):
      return "%s%s%d%sc%d.fits" % (self.root,kind,m,suf,c)
   def rootf(self,kind,m,suf=""):
      return "%s%s%d%s" % (self.root,kind,m,suf)
   def GetPosition(self,alternate=None):
      if alternate:
         e = glob(self.origf(alternate,self.ccds[0]))[0]
      else:
         e = glob(self.origf(self.spec[0],self.ccds[0]))[0]
      if not self.quiet: print e
      F = FITS(e)
      self.ra = F["ra"]
      self.dec = F["dec"]
      self.rot = F["rotangle"]
      F.close()
   def Go(self,step=None,pout=False,event=None):
      if step and hasattr(self,step):
         getattr(self,step)()
      if pout:
         pid = os.getppid()
         mach = os.uname()[1].split(".")[0]
         fout = "%s_%d.out" % (mach,pid)
         fo = open(fout,"a")
         fo.write(72*"#"+"\n")
         fo.write(f+"\n")
         fo.close()
         pg = "PGPLOT_DEV=/NULL"
         func = lambda f: verbex(pg+" "+f+" >> "+fout)
      else:
         func = verbex
      map(func,self.Queue)
      self.Queue = []
      self.Outputs = []
   def Commands(self,base,args,kinds,new,ccds=None):
      if not ccds: ccds = self.ccds
      if not self.quiet: print base
      commands = []
      outputs = []
      for kind in kinds:
        fs = eval("self."+kind)
        for i,f in enumerate(fs):
           m = i+1
           for c in ccds:
             v = (base,) + args(kind,f,m,c)
             command = " ".join(v)
             commands.append(command)
             new1 = new(kind,f,m,c)
             outputs.append(new1)
      return commands,outputs
   def Overscans(self):
      base = "imacsbias"
      cwd = os.getcwd()
      if self.flatself:
         args = lambda kind,f,m,c: (self.origf(f,c),self.dataf(kind,m,c))
         new = lambda kind,f,m,c: self.dataf(kind,m,c)
      else:
         args = lambda kind,f,m,c: (self.origf(f,c),self.origf(f,c,loc=cwd,gz=False))
         new = lambda kind,f,m,c: self.origf(f,c,loc=cwd,gz=False)
      commands, outputs = self.Commands(base,args,self.kinds,new)
      self.Queue += commands
      self.Outputs += outputs
   def Flatten(self):
      base = "efits"
      cwd = os.getcwd()
      args = lambda kind,f,m,c: (self.origf(f,c,loc=cwd,gz=False),self.gflat(c),"\"VTKDivide(i1,i2)\"",self.dataf(kind,m,c))
      new = lambda kind,f,m,c: self.dataf(kind,m,c)
      commands, outputs = self.Commands(base,args,self.kinds,new)
      self.Queue += commands
      self.Outputs += outputs
   def SetupSolns(self):
      base = "imacs2soln -ra %s -dec %s -rot %f" % (self.ra,self.dec,self.rot)
      args = lambda kind,f,m,c: (self.dataf(kind,m,c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".soln")
      commands, outputs = self.Commands(base,args,("mask",),new)
      self.Queue += commands
      self.Outputs += outputs
   def FixSolns(self,keys="\(S*aF%aF#aF@aF!aFaF"):
      base = "LDPMeasure -keys \'%s\' -direct -smf %s" % (keys,self.smf)
      args = lambda kind,f,m,c: ("-root %s -chips %d" % (self.rootf(kind,m),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".cat")
      commands, outputs = self.Commands(base,args,("mask",),new)
      self.Queue += commands
      self.Outputs += outputs
   def FirstWaveSolns(self,keys="\(S\(aFaF*aFaF^aF1c$aF#aFcaF"):
      base = "LDPMeasure -keys \'%s\' -lamp -smf %s -l he.dat -clean" % (keys,self.smf)
      args = lambda kind,f,m,c: \
        ("-root %s -ref %s -chips %d" % (self.rootf(kind,m),self.rootf("mask",argmin(abs(self.mask-self.lamp[m-1]))+1),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".wsoln")
      commands, outputs = self.Commands(base,args,("lamp",),new)
      self.Queue += commands
      self.Outputs += outputs
   def FitGaussians(self,keys="G"):
      base = "LDPMeasure -keys \'%s\' -lamp -smf %s -l he_all.dat" % (keys,self.smf)
      args = lambda kind,f,m,c: \
        ("-root %s -ref %s -chips %d" % (self.rootf(kind,m),self.rootf("mask",argmin(abs(self.mask-self.lamp[m-1]))+1),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".gau")
      commands, outputs = self.Commands(base,args,("lamp",),new)
      self.Queue += commands
      self.Outputs += outputs
   def SecondWaveSolns(self,keys="$aF2#aF@aFaF!aFaF"):
      base = "LDPMeasure -keys \'%s\' -lamp -smf %s -l he_all.dat" % (keys,self.smf)
      args = lambda kind,f,m,c: \
        ("-root %s -ref %s -chips %d" % (self.rootf(kind,m),self.rootf("mask",argmin(abs(self.mask-self.lamp[m-1]))+1),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".wsoln")
      commands, outputs = self.Commands(base,args,("lamp",),new)
      self.Queue += commands
      self.Outputs += outputs
   def PlotWaveSolns(self,keys="@aF"):
      base = "LDPMeasure -keys \'%s\' -lamp -smf %s -l he_all.dat" % (keys,self.smf)
      args = lambda kind,f,m,c: \
        ("-root %s -dev %sc%d.ps/vcps -size 11 -ref %s -chips %d" % (self.rootf(kind,m),self.rootf(kind,m),c,self.rootf("mask",argmin(abs(self.mask-self.lamp[m-1]))+1),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c).replace(".fits",".ps")
      commands, outputs = self.Commands(base,args,("lamp",),new)
      self.Queue += commands
      self.Outputs += outputs
   def ShiftWaveSolns(self,keys="L"):
      base = "LDPMeasure -keys \'%s\' -spec -smf %s -lines -saturate %f" % (keys,self.smf,self.saturate)
      args = lambda kind,f,m,c: \
        ("-root %s -ref %s -clean -chips %d" % (self.rootf(kind,m),self.rootf("lamp",argmin(abs(self.lamp-self.spec[m-1]))+1),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c,"").replace(".fits",".wsoln")
      commands, outputs = self.Commands(base,args,("spec",),new)
      self.Queue += commands
      self.Outputs += outputs
   def SaturationMask(self,keys="W"):
      base = "LDPMeasure -keys \'%s\' -spec -smf %s -saturate %f" % (keys,self.smf,self.saturate)
      args = lambda kind,f,m,c: \
        ("-root %s -chips %d" % (self.rootf(kind,m),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c,"m")
      commands, outputs = self.Commands(base,args,("spec",),new)
      self.Queue += commands
      self.Outputs += outputs
   def SkySubtractions(self,keys="E"):
      base = "LDPMeasure -keys \'%s\' -spec -smf %s -saturate %f" % (keys,self.smf,self.saturate)
      if self.flatself: base = base + " -flatself"
      args = lambda kind,f,m,c: \
        ("-root %s -chips %d" % (self.rootf(kind,m),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c,"o")
      commands, outputs = self.Commands(base,args,("spec",),new)
      self.Queue += commands
      self.Outputs += outputs
   def ObjectExtractions(self,keys="O",hap=4.0):
      if self.standard:
         rkind = "lamp"
         suf = ""
         npass = 1
         down = 999.
         hap = 10.0
      else:
         rkind = "spec"
         suf = "o"
         npass = 3
         down = 3
         hap = 4.0
      base = "LDPMeasure -keys \'%s\' -spec -smf %s -hap %f -saturate %f -npass %d -down %f" % (keys,self.smf,hap,self.saturate,npass,down)
      args = lambda kind,f,m,c: \
        ("-clean -ref %s -root %s -chips %d" % (self.rootf(rkind,m),self.rootf(kind,m,suf),c),)
      new = lambda kind,f,m,c: self.dataf(kind,m,c,suf+"x")
      commands, outputs = self.Commands(base,args,("spec",),new)
      self.Queue += commands
      self.Outputs += outputs
   def RelativeSky(self,sig=3.6,chip=2):
      base = "LDPRelativeSky -sig %f -ref %d" % (sig,chip)
      args = lambda kind,f,m,c: (self.rootf(kind,m),"-d %s" % (self.rootf(kind,m,"oq.ps/vcps")))
      new = lambda kind,f,m,c: self.dataf(kind,m,c,"oq")
      commands, outputs = self.Commands(base,args,("spec",),new,[1])
      self.Queue += commands
      self.Outputs += outputs




