#!/usr/bin/env python

#Usage:
#losvd -gal galaxy.fits -temp templ.fits
#          -ix1 335 -ix2 670 -nc 4 -nh 5 -rn 10 -g 2

from os import path, times
from bisect import *
from Numeric import *
from umath import *
from pgplot import *
from FFT import *
import Multipack as M
from LinearAlgebra import *
from FITS import *
from basics import *
from sys import argv, exit, stdout

def hermite(l,y):
   if l == 0: h = ones(y.shape)
   elif l == 1: h = sqrt(2)*y
   elif l == 2: h = (2*power(y,2)-1)/sqrt(2)
   elif l == 3: h = (2*sqrt(2)*power(y,3)-3*sqrt(2)*y)/sqrt(6)
   elif l == 4: h = (4*power(y,4)-12*power(y,2)+3)/sqrt(24)
   return h

def phase(data,shift,sigma,sign=1,moments=0):
   n = len(data)
   n2n = pow(2,int(log2(n-1))+1)
   n2f = n2n/2+1
   w = 2*pi*arange(0,n2f,1,Float64)/float(n2n)
   ff1 = real_fft(data,n2n)
   ff2 = cos(shift*w) + sign*sin(shift*w)*1j
   if moments:
      mdata = []
      try:
         ff3e = -power(w*sigma,2)/2
      except:
         ff3e = zeros(ff2.shape)
      ff3 = exp( ff3e)
      for l in [0,3,4]:
         odata = ff1*ff2*ff3*pow(1j,l)*hermite(l,w*sigma)
         mdata.append( inverse_real_fft( odata)[:n])
      mdata = asarray(mdata)
      return mdata
   else:
      try:
        ff3e = -pow(w*sigma,2)/2
        ff3 = exp( ff3e)
      except: ff3 = ones(ff2.shape)
      odata = ff1*ff2*ff3
      return inverse_real_fft( odata)[:n]

def getharm(x,nharm):
  hbase=[]
  hbase.append(ones(x.shape,Float64))
  for i in range(1,nharm+1):
    amp = 1.0
    hbase.append(amp*cos(2*pi*i*x/len(x)))
    hbase.append(amp*sin(2*pi*i*x/len(x)))
  return hbase

def setup(sigmas,shifts,ix1,ix2,tspecs,usky,cbase,ibase,sbase,moments=0):
   nt=len(tspecs)
   b=[]
   for j in range(nt):
     if moments:
      ph = phase(tspecs[j],shifts[j],sigmas[j],moments=1)[::,ix1:ix2+1]
      for p in ph: b.append(p)
     else:
      ph = phase(tspecs[j],shifts[j],sigmas[j],moments=0)[ix1:ix2+1]
      b.append(ph)
   ib=len(ibase)
   cb=len(cbase)
   sb=len(sbase)
   basis = []
   for i in range(len(ibase)):
     for j in range(len(b)):
       basis.append( multiply(ibase[i],b[j]))
   for i in range(len(cbase)):
       basis.append( cbase[i])
   if sb:
    for i in range(len(sbase)):
         basis.append(sbase[n])
   return asarray(basis)

def mkmodel(waves,gspec,nois,xmask,tmask,pvel,tspecs,usky,sigmas,shifts,ix1,ix2,cbase,ibase,sbase,moments=0):
   for mom in range(moments+1):
      basis=setup(sigmas,shifts,ix1,ix2,tspecs,usky,cbase,ibase,sbase,moments=mom)
      reds = 1+shifts[0]/CLIGHTK
      smask=zeros(waves.shape,Float64)
      for mask in tmask: smask = smask + greater(waves,mask[0]*reds) * less(waves,mask[1]*reds)
      smask=equal(smask,0.0)
      nt = len(tspecs)
      ib = shape(ibase)[0]
      cb = shape(cbase)[0]
      sb = shape(sbase)[0]
      g = gspec[ix1:ix2+1]
      noi = nois[ix1:ix2+1]
      vari = power(noi,2)
      w = divz(1,getsqrt(vari),repl=0.0)
      w = w * xmask[ix1:ix2+1]
      w = w * smask[ix1:ix2+1]
      ub = transpose(basis*w[NewAxis,:])
      uy = g*w
      sol = svdfit(ub,uy)
      m = add.reduce(basis*sol[:,NewAxis])
      s = add.reduce(transpose(sol[-sb:]*transpose(basis[-sb:])))
      if moments and not mom:
          ibase = asarray([add.reduce(ibase*sol[:ib,NewAxis])])
   return add.reduce(transpose(sol*transpose(basis))), g, w, sol, s, basis

def getfit(params,fixsig,fixvel,ix1,ix2,waves,gspec,nois,xmask,tmask,tspecs,usky,cbase,ibase,sbase,ndeg,pvel,justchi=1,verbose=0):
   nt = len(tspecs)
   if fixsig and not fixvel:
      sigmas = fixsig
      shifts = params
   elif not fixsig and fixvel:
      sigmas = power(10,params)
      shifts = fixvel
   elif fixsig and fixvel:
      sigmas = fixsig
      shifts = fixvel
   else:
      sigmas = power(10,params[:nt])
      shifts = params[nt:]
   m,g,w,a,s,b = mkmodel(waves,gspec,nois,xmask,tmask,pvel,tspecs,usky,sigmas,shifts,ix1,ix2,cbase,ibase,sbase)
   resid = m - g
   chi2 = power(resid*w,2)
   if verbose == 1: print ".",
   if verbose == 2: print multiply(sigmas,pvel), multiply(shifts,pvel), add.reduce(chi2)/ndeg
   if justchi == 0: return chi2,a
   elif justchi == 1: return resid*w
   elif justchi == 2: return chi2, a, m, s, b
   elif justchi == 3: return sum(chi2)

def writefit(filefit,bdat,galaxy,fitspec):
   mfits = FITS(filefit,create=1,template=galaxy)
   mfits.modkey("naxis2",2)
   mfits.blank()
   mfits.writerow(fitspec,0)
   mfits.writerow(bdat,1)
   mfits.close()

