#!/usr/bin/env python

from pgplot import *
from VTKHelperFunctions import *
from FFT import *
from MLab import *
from Multipack import *
from LinearAlgebra import *
from basics import *
from FITS import *
from sys import argv,exit
from pgapack import *

def medsmooth(x,w=16):
   r = VTKMedianSmooth(x,w,numret=1)
#   r = VTKMedian(x,2*(w/2)+1,numret=1)
#   qdump("x.fits",x)
#   qdump("r.fits",r)
#   exit(1)
   return r
#        return VTKGrowN(VTKShrinkN(x,w,usemed=1),w,numret=1,interp=1,origsize=x.shape,xshift=-w/2)

def sifty(s,win,bg=1):
    if win:
        s = s-getpct(s,2)
        b = funcfilt(getpct,s,win,0,arg=(2)).astype(Float64)
        z = nonzero(greater(s,0))
        Ms = VTKMedian(s,win,1,1)
        if z:
           b[:z[0]+win] = Ms[z[0]+win]
           b[z[-1]-win:] = Ms[z[-1]-win]
        b = b - getpct(b,2)
        c = (s-b)
    else:
        c = zeros(s.shape,Float32)
        b = zeros(s.shape,Float32)
    b = sqrt(b**2 + rms(b)**2)/sqrt(2)
    t = divz(c,getsqrt(b))
    g = argsort(t)[::-1]
    return b,c,t,g

def uspv(l, tck, val=0.0): return [splev(l,tck,val==0.0)-val]

def peaky(tck, l, val=0.0):
    return fsolve(uspv, l, args=(tck,val), xtol=1E-4, factor=0.01)

def findy(x,s,t,g,fwhm=5.0,thresh=10.0):
        nuse = sum(greater(t,thresh))
        m = take(x,g[:nuse])
        tck = splrep(x,s,s=0)
        m = asarray(map(lambda l: peaky(tck,l),m))
        f = zeros(m.shape)
        for j in  range(len(m)):
            diff = less_equal(abs(m[j] - m[j+1:]),0.85*fwhm)
            f[j+1:] = f[j+1:] + diff
        f = equal(f,0)*greater(m,x[0])*less(m,x[-1])
        m = take(m,nonzero(f),0)
        m = sort(m)
        if len(m):
           n = splrep(x,t,s=0,k=3)
           gv = splev(m,n)
           if len(m) == 1: gv = [gv]
           gt = greater(gv,thresh)
           c = nonzero(gt)
           m = take(m,c)
           if len(m):
              a = splev(m-fwhm/3,tck,1); b = splev(m+fwhm/3,tck,1)
              if len(m) == 1:
                 a= [a]
                 b= [b]
              if shape(a) == shape(b) == (): a,b=[a],[b]
              c = nonzero(greater(a,b))
              m = take(m,c)
              if len(m):
                 q = splev(m,n)
                 if len(m) == 1: q = [q]
              else: return None,m,m
           else: return None,m,m
        else: return None,m,m
        return tck,m,q

def edgy(x,s,l,h=0.5,fwhm=5.0):
        a = int(max((x[0],l-5*fwhm)))
        b = int(min((x[-1],l+5*fwhm)))
        ux,us = compress(between(x,a,b,inc=1),[x,s],1)
        if len(ux): tck = splrep(ux,us,s=0,nest=len(x))
        else: return 0.0,0.0
        v = splev(l,tck)
        vtck = splrep(ux,us-h*v,s=0,nest=len(x))
        z = sproot(vtck,mest=len(ux))
        z = take(z,argsort(abs(z-l)))
        if len(z) > 1: return z[:2]
        else: return l,l

def centery(x,s,m,fwhm=5.0):
        t = transpose(map(lambda l: edgy(x,s,l,0.85,fwhm), m))
        p = add.reduce(t)/2.0
        f = transpose(map(lambda l: edgy(x,s,l,0.5,fwhm), m))
        h = abs(subtract.reduce(f))
        return p,h

def ccfindy(ct,fwhm=5.0,thresh=0.5):
        g=argsort(ct)[::-1]
        thresh = thresh * ct[g[0]]
        nuse = sum(greater(ct,thresh))
        m = g[:nuse]
        f = zeros(m.shape)
        for j in  range(len(m)):
            diff = less_equal(abs(m[j] - m[j+1:]),fwhm)
            f[j+1:] = f[j+1:] + diff
        f = equal(f,0)
        if sum(f):
          m = compress(f,m)
          q = take(ct,m)
          m,q = take([m,q],argsort(q)[::-1],1)
        else:
          m = zeros((0,),Float32)
          q = zeros((0,),Float32)
        return m,q


def proximity(m,fwhm=5.0):
    f = zeros(m.shape)
    for j in  range(len(m)):
        diff = less_equal(abs(m[j] - m[j+1:]),fwhm)
        f[j+1:] = f[j+1:] + diff
    f = equal(f,0)
    if sum(f):
      m = compress(f,m)
    else:
      m = zeros((0,),Float32)
    return m

def getccpeaks(s,c,t,g,fwhm,thresh):
   x=arange(len(s))
   print "Searching for peaks with %5.2f %5.2f" % (fwhm, thresh)
   tck,m,q = findy(x,s,t,g,fwhm,thresh)
   p,f = centery(x,c,m,fwhm)
   print "Found %d good peaks" % (len(p))
   ff,v = bwt(f)
   print "Widths = %5.2f %5.2f" % (ff,v)
   u = less_equal(f,ff+3*v)
   p,f,q = take([p,f,q],nonzero( u),1)
   print "Cut down to %d lines" % (len(p))
   return p,f,q

def getlines(x,s,c,t,g,fwhm,cut,thresh,verb=1):
   bc = (x[0]+x[-1])/2.0
   bw = (x[-1]-x[0])/2.0
   for i in range(1):
      if verb: print "Searching for lines with", fwhm, thresh
      tck,m,q = findy(x,s,t,g,fwhm,thresh)
      if not len(m):
         return zeros((0,),Float64), zeros((0,),Float64), zeros((0,),Float64), zeros((0,),Float64),
      wq = getsqrt(q)
      p,f = centery(x,c,m,fwhm)
      if shape(q) == (): q=[q]
      if verb: print 
#      if verb: print p,q,f
      if verb:
         print "Found %d good lines" % (len(p))
         for jl in range(len(p)): print "%8.2f (%6.2f), " % (p[jl],f[jl]),
         print
      if len(p) > 4:
        for i in range(1):
          xn = (p-bc)/bw
          bas = legendre(xn,1)
          sol = svdfit(transpose(bas),f)
          if verb: print sol
          ff = add.reduce(sol[::,NewAxis]*bas,0)
          v = bwt(ff-f)[1]
          if fwhm > 2: u = less_equal(f,ff+cut*v)*greater_equal(f,2.0)
          else: u = less_equal(f,ff+cut*v)*greater_equal(f,ff-cut*v)*greater_equal(f,0.5)
          if shape(q) == (): q=[q]
          if shape(wq) == (): wq=[wq]
          p,f,q,wq = take([p,f,q,wq],nonzero( u),1)
          if verb: print "Cut down to %d lines" % (len(p))
      else:
          p,f,q = take([p,f,q],nonzero( f),1)
      fwhm,sfwhm = getpct(f,75),bwt(f)[1]
      if verb: print "Using FWHM=",fwhm,sfwhm
   if verb:
      print
      lo=open("found.dat","w")
      for jl in range(len(p)): print >>lo, "%8.2f  %7.2f" % (p[jl],f[jl])
      lo.close()
      for jl in range(len(p)): print "%8.2f (%6.2f), " % (p[jl],f[jl]),
      print
      print
   return p,f,q,fwhm

def readll(lines,ltype="sky",fwhm=5.0,disp=0.0,dich=None):
        f=open(lines,"r")
        s=f.readlines()
        f.close()
        l = [l.split() for l in s if l[0] != "#"]
        lw=array(map(lambda l: float(l[0]), l))
        print
        if ltype != "amp":
           ln=map(lambda l1: join(l1[1:]), l)
        else:
           ln=map(lambda l1: l1[1], l)
           try: lh=100*array(map(lambda l1: float(l1[2]), l))
           except: lh=100*ones( (len(ln),),Float32)
           lu=greater(lh,0)
           lt={}

        if ltype == "sky":
           o1 =  greater(map(lambda n: find(n,'O I'), ln),0)
           o2 =  greater(map(lambda n: find(n,'O2')+1, ln),0)
           na1 =  greater(map(lambda n: (n=='Na I'), ln),0)
           q1s = greater(map(lambda n: find(n,'Q1'), ln),0)
           q2s = greater(map(lambda n: find(n,'Q2'), ln),0)
           r1s = greater(map(lambda n: find(n,'R1'), ln),0)
           r2s = greater(map(lambda n: find(n,'R2'), ln),0)
           p1s = greater(map(lambda n: find(n,'P1'), ln),0)
           p2s = greater(map(lambda n: find(n,'P2'), ln),0)
           hg =  greater(map(lambda n: (n[:2].upper()=='HG'), ln),0)
           kr =  greater(map(lambda n: (n[:2].upper()=='KR'), ln),0)
           ar =  greater(map(lambda n: (n[:1].upper()=='A'), ln),0)
           uselines = nonzero(o1+na1+p1s+q1s+r1s+o2+p2s+hg+kr)

           lt = {"O1":o1,"NA1":na1,"P1":p1s,"Q1":q1s,"R1":r1s,"O2":o2,"P2":p2s}

           lu = o1 + na1 + p1s + p2s + hg + kr + ar

           lh = zeros( lw.shape, Float32)
           if sum(hg) > 2: polluted=4
           else: polluted=1
           for i in nonzero(o1):
              if int(lw[i]) == 2*5577: lh[i] = 5000.0/polluted
              if int(lw[i]) == 5577: lh[i] = 15000.0/polluted
              if int(lw[i]) == 6300: lh[i] = 10000.0/polluted
              if int(lw[i]) == 6363: lh[i] =  5000.0/polluted
           for i in nonzero(na1):
               if disp < 0.2:
                  lh[i] =  5000.0*polluted
               else:
                  lh[i] =  2500.0*polluted
           for i in nonzero(p1s):
              t1 =  split(replace(replace( ln[i],"("," "),")"," "))[0]
              t2 = float( split(replace(replace( ln[i],"("," "),")"," "))[2] )
              lh[i] = 5*pow(lw[i]/2500,3) * 40 * exp(-pow(t2-3.5,2)/t2)
           for i in nonzero(p2s):
              t1 =  split(replace(replace( ln[i],"("," "),")"," "))[0]
              t2 = float( split(replace(replace( ln[i],"("," "),")"," "))[2] )
              lh[i] = 5*pow(lw[i]/2500,3) * 40 * exp(-pow(t2-3.5,2)/t2) / 3.0
           for i in nonzero(r1s):
              t2 = float( split(replace(replace( ln[i],"("," "),")"," "))[2] )
              lh[i] = 5*pow(lw[i]/2500,3) * 30 / t2
           for i in nonzero(q1s):
              t2 = float( split(replace(replace( ln[i],"("," "),")"," "))[2] )
              lh[i] = 5*pow(lw[i]/2500,3) * 60 / t2
           for i in nonzero(o2):
              lh[i] = 2200*exp(-0.5*pow(lw[i]-8630,2)/9**2) + \
                          2200*exp(-0.5*pow(lw[i]-8657,2)/10**2)

           for i in nonzero(hg):
              if int(lw[i]) == 3649: lh[i] =   20.0
              elif int(lw[i]) == 4046: lh[i] =  200.0
              elif int(lw[i]) == 4077: lh[i] =   10.0
              elif int(lw[i]) == 4358: lh[i] =  200.0
              elif int(lw[i]) == 5460: lh[i] =  200.0
              elif int(lw[i]) == 5769: lh[i] =   50.0
              elif int(lw[i]) == 5790: lh[i] =   50.0
              elif int(lw[i]) == 6123: lh[i] =    0.0
           for i in nonzero(kr):
              if int(lw[i]) == 4319: lh[i] =   10.0
              elif int(lw[i]) == 5460: lh[i] = 3000.0
           for i in nonzero(ar):
              if int(lw[i]) == 5460: lh[i] = 3000.0

           if dich: lh = lh*(1-(o1+na1+o2)*cos((lw/5600)*pi/4)**2)**2

        elif ltype == "arc":
           unk =  greater(map(lambda n: len(n)==0, ln),0)
           he =  greater(map(lambda n: (n[:2].upper()=='HE'), ln),0)
           hg =  greater(map(lambda n: (n[:2].upper()=='HG'), ln),0)
           kr =  greater(map(lambda n: (n[:2].upper()=='KR'), ln),0)
           ar =  greater(map(lambda n: (n[:1].upper()=='A'), ln),0)
           xe =  greater(map(lambda n: (n[:2].upper()=='XE'), ln),0)
           ne =  greater(map(lambda n: (n[:2].upper()=='NE'), ln),0)
           zn =  greater(map(lambda n: (n[:2].upper()=='ZN'), ln),0)
           cd =  greater(map(lambda n: (n[:2].upper()=='CD'), ln),0)
           thor =  greater(map(lambda n: (n[:2].upper()=='TH'), ln),0)
           lt = {"HG":hg,"KR":kr,"AR":ar,"XE":xe,"NE":ne,"ZN":zn,"CD":cd,"TH":thor}
           lu = hg+ar+kr+xe+ne+he+cd+zn+thor
           uselines = nonzero(lu)

           if sum(unk) == len(lw): lh = 10*ones(lw.shape,Float32); lu = logical_not(lu)
           else: lh = zeros(lw.shape,Float32)

           for i in nonzero(hg):
              if int(lw[i]) == 3649: lh[i] =    50.0
              elif int(lw[i]) == 4046: lh[i] =  200.0
              elif int(lw[i]) == 4077: lh[i] =    0.0
              elif int(lw[i]) == 4358: lh[i] = 1000.0
              elif int(lw[i]) == 5769: lh[i] =  400.0
              elif int(lw[i]) == 5790: lh[i] =  400.0
              elif int(lw[i]) == 6123: lh[i] =    0.0
              elif int(lw[i]) == 5460: lh[i] = 300.0
           for i in nonzero(kr):
              if int(lw[i]) == 4319: lh[i] =   10.0
              elif int(lw[i]) == 5460: lh[i] = 1000.0
              elif int(lw[i]) == 8263: lh[i] = 500.0
           for i in nonzero(ar):
              if int(lw[i]) == 4277: lh[i] =   10.0
              elif int(lw[i]) == 4300: lh[i] =   10.0
              elif int(lw[i]) == 4158: lh[i] =  100.0
              elif int(lw[i]) == 5460: lh[i] = 1000.0
              elif int(lw[i]) == 6752: lh[i] =  100.0
              elif int(lw[i]) == 6965: lh[i] =   50.0
              elif int(lw[i]) == 7383: lh[i] =  200.0
              elif int(lw[i]) == 7503: lh[i] =  100.0
              elif int(lw[i]) == 7635: lh[i] = 1000.0
              elif int(lw[i]) == 7723: lh[i] = 1000.0
              elif int(lw[i]) == 7891: lh[i] =  100.0
              elif int(lw[i]) == 7948: lh[i] =  100.0
              elif int(lw[i]) == 8115: lh[i] = 1000.0
              elif int(lw[i]) == 8521: lh[i] =  100.0
              elif int(lw[i]) == 8667: lh[i] =  100.0
              elif int(lw[i]) == 9122: lh[i] = 1500.0
              elif int(lw[i]) == 9224: lh[i] =  150.0
              elif int(lw[i]) == 9354: lh[i] =  50.0
              elif int(lw[i]) == 9657: lh[i] =  50.0
              elif int(lw[i]) <= 6000: lh[i] =  10.0
              elif int(lw[i]) >= 7500: lh[i] =  100.0
              else: lh[i] = 500.0
           for i in nonzero(thor):
              lh[i] =   50.0
           for i in nonzero(xe):
              lh[i] =   50.0
           for i in nonzero(zn):
              lh[i] =   1000.0
           for i in nonzero(cd):
              lh[i] =   1000.0
           for i in nonzero(ne):
              if int(lw[i]) == 5944: lh[i] =   600.0
              elif int(lw[i]) == 5975: lh[i] =   100.0
              elif int(lw[i]) == 6029: lh[i] =   100.0
              elif int(lw[i]) == 6143: lh[i] =   700.0
              elif int(lw[i]) == 6163: lh[i] =    300.0
              elif int(lw[i]) == 6219: lh[i] =    100.0
              elif int(lw[i]) == 6304: lh[i] =    200.0
              elif int(lw[i]) == 6334: lh[i] =    200.0
              elif int(lw[i]) == 6402: lh[i] =   200.0
              elif int(lw[i]) == 6506: lh[i] =   200.0
              elif int(lw[i]) == 6678: lh[i] =   200.0
              elif int(lw[i]) == 7032: lh[i] =  1000.0
              elif int(lw[i]) <= 6000: lh[i] =   50.0
              elif int(lw[i]) <= 7500: lh[i] =  200.0
              else: lh[i] =   100.0
           for i in nonzero(he):
              if int(lw[i]) == 3888: lh[i] =   300.0
              elif int(lw[i]) == 4471: lh[i] =   300.0
              elif int(lw[i]) == 4921: lh[i] =   100.0
              elif int(lw[i]) == 5015: lh[i] =   100.0
              else: lh[i] =   30.0

        return lw,ln,lh,lt,lu

def dispsol(sol,b,x,p,f,q,olw,ln,olh,lt,olu,wr,fwhm,amp,wz,lty,usep2=0,useqr=0,verb=0):
        lw = 1. * olw
        lu = 1. * olu
        lh = 1. * olh
        xc = (x[0]+x[-1])/2.0
        xw = (x[-1]-x[0])/2.0
        g = add.reduce(multiply(asarray(sol)[::,NewAxis],b))
        o = argsort(p)
        r = wr*fwhm

        if lt.has_key("NA1"):
           if fwhm*sol[1]/xw > 12.0:
               for i in nonzero(lt["NA1"]):
                  lw[i] = mean(take(olw,nonzero(lt["NA1"])))
                  lu[i] = 0
                  lh[i] = 0.0
           elif fwhm*sol[1]/xw > 7.5:
               for i in nonzero(lt["NA1"]): lw[i] = mean(take(olw,nonzero(lt["NA1"])))
           if not usep2:
               for i in nonzero(lt["P2"]): lu[i] = 0; lh[i] = 0.0
           if not useqr:
              if wr < 2:
                for i in nonzero(lt["Q1"]): lu[i] = 0; lh[i] = 0.0
                for i in nonzero(lt["R1"]): lu[i] = 0; lh[i] = 0.0

        fx = []; fy = []; fw = []; fg = []
        fn = []; fh = []; fq = []; ff = []
        if wz and len(lw) < 100 and len(p) > len(sol)+3 \
              or (lty in ["arc","amp"] and len(lw) < 100):
           dg = g[::,NewAxis] - g[NewAxis,::]
           dlw = lw[::,NewAxis] - lw[NewAxis,::]
           for i in o:
              dglw = abs(dg[i,::,NewAxis,NewAxis] - dlw[NewAxis,::,::])
              if lty in ["arc","amp"]: n4 = 2
              else: n4=2
              um = less(dglw,r*sol[1]/xw)*less(abs(g[i]-lw),n4*r*sol[1]/xw)[NewAxis,::,NewAxis]
              ium = add.reduce(um,2)
              num = add.reduce(ium)
              imax = argmax(num)
              ntol = max([sum(between(lw,g[0]-sol[1]/4.,g[-1]+sol[1]/4.,inc=1))/4.0,1])
              if num[imax] > ntol:
                 if verb>1: print sort(num)[-3:],ntol
                 fx = concatenate([fx,[p[i]]])
                 fy = concatenate([fy,[lw[imax]]])
                 if wr < 1: fw = concatenate([fw,[q[i]/f[i]]])
                 else: fw = concatenate([fw,[q[i]/f[i]]])
                 fh = concatenate([fh,[lh[imax]]])
                 fq = concatenate([fq,[q[i]]])
                 ff = concatenate([ff,[f[i]]])
                 fn = fn + [ln[imax]]
                 fg = concatenate([fg,[g[i]]])
           if verb>1:
              print fx
              print fy
              print fg
              print fg-fy
           if len(fy) < len(sol): wz = 1
        else:
#           if verb: print wr,sol
           for i in o:
              gl = g[i] - r*sol[1]/xw
              gh = g[i] + r*sol[1]/xw

              if wr < 2.5:
                  us = nonzero(greater(lw,gl)*less(lw,gh)*greater(lh,amp)*lu*greater(lh,0))
              else:
                  us = nonzero(greater(lw,gl)*less(lw,gh)*greater(lh,amp)*greater(lh,0))

              if len(us):
                 uh,uw,uu = take([lh,lw,lu], us, 1)
                 un = map(lambda us: ln[us], us)
                 uo = argsort(uh)[::-1]

                 uo = uo[:max([1,int(wr)])]

                 uh,uw,uu = take([uh,uw,uu], uo,1)
                 un = map(lambda us: un[us], uo)
#                 if verb: print g[i],gl,gh,uw

                 fx = concatenate([fx,len(uw)*[p[i]]])
                 fy = concatenate([fy,uw])
                 if wr < 1: fw = concatenate([fw,ones(uh.shape)*(q[i]/f[i])])
                 else: fw = concatenate([fw,uh*(q[i]/f[i])])
                 fh = concatenate([fh,uh])
                 fq = concatenate([fq,len(uw)*[q[i]]])
                 ff = concatenate([ff,len(uw)*[f[i]]])
                 fn = fn + un
              else:
                 fx = concatenate([fx,[p[i]]])
                 fy = concatenate([fy,[0.0]])
                 fw = concatenate([fw,[0.0]])
                 fh = concatenate([fh,[0.0]])
                 fq = concatenate([fq,[q[i]]])
                 ff = concatenate([ff,[f[i]]])
                 fn = fn + ["xxxxx"]

        if len(fx):
            fx,fy,fw=asarray([fx,fy,fw])
            fw0 = 1*fw
            fz = greater(fw,0)
            fb = legendre((fx-xc)/xw,len(sol)-1)
            iter = 0
            keep = ones(fy.shape)
            oldkeep = zeros(fy.shape)
            if 0 < wz < len(sol):
              fy0 = fy-add.reduce(sol[wz:,NewAxis]*fb[wz:])
              fb0 = fb[:wz]
              sol0 = svdfit(transpose(fb0*fw**2),fy0*fw**2)
              sol0 = planefit(transpose(fb0*fw**2),fy0*fw**2,sol0,minfunc=mad)[0]
              if type(sol0) == FloatType: sol0 = array([sol0])
              elif type(sol0) == ArrayType and not len(sol0.shape): sol0 = array([sol0])
              sol = concatenate([sol0,sol[wz:]])
              while keep != oldkeep and iter < 5:
                 iter += 1
                 oldkeep = 1*keep
                 fit = add.reduce(sol[::,NewAxis]*fb)
                 fres2 = abs(fit-fy)
                 rsol = svdfit(transpose(fb[:3]*fw**2),fres2*fw**2)
                 rco = 1.0
                 rfit = sqrt(add.reduce(rsol[::,NewAxis]*fb[:3])**2 + (sol[1]/xw/rco)**2)
                 keep = less(fres2,clip(rco*rfit,ff*sol[1]/xw,1e9))
                 fw = fw0 * keep
                 sol0 = svdfit(transpose(fb0*fw**2),fy0*fw**2)
                 sol = concatenate([sol0,sol[wz:]])
            else:
#              if verb: print sol,sum(keep)
#              sol = svdfit(transpose(fb*fw**2),fy*fw**2)
#              if verb: print sol,sum(keep)
              while keep != oldkeep and iter < (5 - 4*(lty in ["arc","amp"])):
                 iter += 1
                 oldkeep = 1*keep
                 fit = add.reduce(sol[::,NewAxis]*fb)
                 fres2 = abs(fit-fy)
                 rsol = svdfit(transpose(fb*fw**2),fres2*fw**2)
                 rco = 2.5
#                 if verb: print rsol,1.49*median(fres2)
                 rfit = sqrt(add.reduce(rsol[::,NewAxis]*fb)**2 + (ff/2*sol[1]/xw)**2)
                 keep = less(fres2,rco*rfit)
                 fw = fw0 * keep
                 sol = svdfit(transpose(fb*fw**2),fy*fw**2)
#                 if verb: print sol,sum(keep)
              ufb = compress(fw,fb,1); ufy = compress(fw,fy,0)
              sol = svdfit(transpose(ufb),ufy)
            fit = add.reduce(sol[::,NewAxis]*fb)
            bres = bwt(compress(greater(fw,0),fit-fy))[1]
            res = sqrt(mean(power(compress(greater(fw,0),fit-fy),2)))
            wres = sqrt(divz(sum(power(compress(greater(fw,0),(fit-fy)*fw),2)),sum(compress(greater(fw,0),fw)**2)))
            print
            print "Trimming iterations=",iter
            print "Solution ="; print sol
            print "Using %3d features." % (sum(greater(fy*fw,0)))
            if wz and sum(greater(fy*fw,0)) < wz:
                 raise "LackOfFeaturesError"
            print "Using minamp=%7.3f" % (amp)
            print "   Range (fwhm, pix and A) = %7.3f %7.3f %7.3f" % (wr, r, r*sol[1]/xw)
            print "UNWT BWT (pix and A) = %7.3f %7.3f" % (bres/(sol[1]/xw),bres)
            print "UNWT RMS (pix and A) = %7.3f %7.3f" % (res/(sol[1]/xw),res)
            print "  WT RMS (pix and A) = %7.3f %7.3f" % (wres/(sol[1]/xw),wres)
            hd="  N   Center   FWHM      S      Wpred     Wtrue   deltaW    H     Line"
            fmt1 = "%3d %8.2f %6.2f %8.1f %9.2f %9.2f %6.2f %8.2f  %s"
            fmt0 = "%3d %8.2f %6.2f %8.1f %9.2f %9s %6s %8s  %s"
            if verb:
               print hd; print (len(hd))*"-"
               for j in range(len(fx)):
                  if fw[j] > 0:
                    print fmt1 % (j+1,fx[j],ff[j],fq[j],fit[j],fy[j],fit[j]-fy[j],fh[j],fn[j])
                  else:
                    print fmt0 % (j+1,fx[j],ff[j],fq[j],fit[j],"","","","xxxxx")
            return bres,sol
        else:
           raise "LackOfFeaturesError"
#           print "Error.";exit()

def mkfake(w0,w1,nd,lw,ln,lh,lt,fw,amp=0):
        d = (w1-w0)/nd
        x = arange(nd)
        w = x*d + w0
        us = nonzero(greater(lw,w[0])*less(lw,w[-1])*greater(lh,amp))
        m = zeros(x.shape,Float64)
        if len(us):
           uh,uw = take([lh,lw],us,1)
           un = map(lambda us: ln[us], us)
           ux = asarray(map(lambda uw: (uw-w0)/d, uw))
           uo = nonzero(greater(ux,x[0]+5*fw)*less(ux,x[-1]-5*fw))
           uh,uw,ux = take([uh,uw,ux],uo,1)
           un = map(lambda uo: un[uo], uo)
           ux0 = (ux-3*fw).astype(Int32)
           ux1 = (ux+3*fw+1).astype(Int32)
           s = fw/2.35
           uk = map(lambda u,a,b: (-0.5*((x[a:b]-u)/s)**2), ux,ux0,ux1)
           ug = map(getexp,uk)
           for j in range(len(ug)): m[ux0[j]:ux1[j]] += uh[j]*ug[j]
        return m

def fftcor(c,t):
        n = pow(2,int(log2(len(t)-1))+2)
        cl=zeros((n),Float64); tl=zeros((n),Float64)
        nc = len(c)
        nt = len(t)
        cl[:len(c)] = c
        tl[:len(t)] = t
        if sum(c): cl = cl/mean(c) - 1.0
        if sum(t): tl = tl/mean(t) - 1.0
        fcl = real_fft(cl)
        ftl = real_fft(tl)
        ct = inverse_real_fft(fcl*conjugate(ftl))
        ct = take(ct,range(-n/2,n/2))
        return cl,tl,ct

def fftwav(disp,p,c,template,fwhm,w0=3000.0,w1=11000,dw=50,tf="",jl1=0):
        nw = max([int((w1-w0)/disp),len(c)])
        cx = arange(len(c))
        dc = sum(c**2)
        try: pcen = sum(c**2*cx)/dc
        except: pcen = len(c)/2
        w0b = w0
        w0e = w1 - len(c)*disp
        dwu = dw
        w0u = arange(w0b,w0e,dwu,Float32)
        b=zeros((int((w1-w0)/disp)),Float64)
        bn=[]
        if not len(w0u): w0u=[w0]
        for j in range(len(w0u)):
          a0 = int(j*dwu/disp)
          a1 = min([nw,a0+len(c)])
          tc = template[a0:a1]
          cl,tl,ct = fftcor(tc,c)
          if jl1: jc = len(ct)/2; jc1 = int(len(ct)/jl1); ct = ct[jc-jc1:jc+jc1]
          qct = gauss_conv1d(ct,fwhm)
          qx = (ct-qct)
          fac = 0.5
          act = nonzero(greater(qx,fac*max(qx)))
          if len(act):
             qx = take(qx,act)
             act = take(act,argsort(qx)[::-1])
#             act = proximity(act,4*fwhm)
             qx = take(ct,act)
             wc = w0u[j] + disp*(len(c)/2 + act -len(ct)/2)
             bx = ((wc-w0)/disp+0.5)
             bx,qx = compress(greater(bx,0)*less(bx,len(b)),[bx,qx],1)
             for jb in range(len(bx)): b[int(bx[jb])] += qx[jb]
          bn.append(b)
        if tf: qdump(tf,b,extras=[["crpix1",1],["crval1",w0],["cdelt1",disp],["dc-flag",0]])
        if tf: print "exiting";exit()
        bz = nonzero(b)
        if len(bz) and (pcen > fwhm and pcen < len(c)-fwhm):
            b = take(b,bz)
            o = argsort(b)[::-1]
            mb,bz = take([b,1.*bz],o,1)
            wc = bz*disp+w0
            th = max(mb)*0 + 1
            c = greater(mb,th)
            wc,mb = compress(c,[wc,mb],1)
            mb = mb / th
        else:
            wc=[]; mb=[]
        return wc,mb,dc

def fftsol(guess,p,q,c,lw,ln,lh,lt,fwhm,amp,w0,w1,order,ltype,um=0):

      wc, disp = guess
      print "Using dispersion guess of ",disp

      ns = len(c)
      x = arange(0,ns,1,Float32)
      xc = (x[0]+x[-1])/2.0
      xw = (x[-1]-x[0])/2.0

      tf = ""
      nw = max([int((w1-w0)/disp),len(c)])
      wf = arange(nw)*disp + w0

      if ltype == "sky":
        csm = c - VTKMedian(c,4*int(fwhm)+1,1,numret=1)
#        csm = mkfake(0,len(c),len(c),p,len(p)*["A"],q,0*p,fwhm/sqrt(2),amp=1)
      else:
        if ltype == "amp" and fwhm < 9:
           csm = mkfake(0,len(c),len(c),p,len(p)*["A"],q,0*p,fwhm/sqrt(2),amp=1)
        else:
           csm = c
      if fwhm > 9:
         fake = mkfake(w0,w0+nw*disp,nw,lw,ln,lh,lt,fwhm/2,1)
      else:
         fake = mkfake(w0,w0+nw*disp,nw,lw,ln,lh,lt,fwhm,1)
#      qdump("fake.fits",fake,extras=[["crpix1",1],["crval1",w0],["cdelt1",disp],["dc-flag",0]])
      faketck = splrep(wf,fake,s=0,k=1)

      wc,ac,dc = fftwav(disp,p,csm,fake,fwhm/2,w0,w1,dw=(w1-w0)/100,tf=tf,jl1=0)
      print wc.shape
      print wc[:5]
      wc = wc[0]

      print "Using central wavelength %7.2f" % (wc)
      print
      sol = asarray([wc,xw*disp])
      rs= bs = 99
      if um == 2: return bs,rs,sol

      if ltype == "sky":
        uns = [[3,4,5,6],[4,5,6,7],[5,6,7,8]][:2]
        unt = [8,8,8]
        ujl1 = [16,16,32]
        unf = [[2],[3],[4]]
      elif ltype in ["arc","amp"]:
        uns = [[3,4,5,6],[4,5,6,7],[5,6,7,8]][:2]
        unt = [8,8,8]
        ujl1 = [16,16,32]
        unf = [[2],[3],[4]]

      for mt in range(len(uns)):

         nt = unt[mt]
         jl1 = ujl1[mt]

         xcls = []; wcls = []
         acls = []; dcls = []
         for ns in uns[mt]:
            for ss in range(nt*ns-1):
               i0 = ss*len(x)/(nt*ns)
               i1 = i0 + len(x)/ns
               xcs = 0.5*(i0+i1)
               fin = sum(greater(p,i0)*less(p,i1))
               if fin:
                  xi = (arange(i0,i1,1)-xc)/xw
                  bi = legendre(xi, len(sol)-1)
                  ws = add.reduce(sol[::,NewAxis]*bi)
                  w0s, w1s = ws[0], ws[-1]; dw = (w1s-w0s)/10.0
                  wi = (w1s-w0s)/(i1-i0)
                  cs = csm[i0:i1]
                  ts = splev(ws, faketck)
                  wcs,acs,dcs = fftwav(wi,p,cs,ts,fwhm/sqrt(4),w0s,w1s,dw=dw,tf=tf,jl1=jl1)
                  if len(wcs):
                     wcs,acs = compress(greater(acs,max(acs)/10),[wcs,acs],1)
                     wcls += [list(wcs)]
                     acls += [list(acs)]
                     xcls += [len(wcs)*[xcs]]
                     dcls += [len(wcs)*[dcs]]

         print  "Old FFT Solution="
         print  sol

         xcl0 = concatenate( xcls)
         wcl0 = concatenate( wcls)
         acl0 = concatenate( acls)
         dcl0 = concatenate( dcls)

         xcl, wcl, acl, dcl = take( [xcl0, wcl0, acl0, dcl0], argsort(wcl0), 1)

         for nf in unf[mt]:
           first = 1
           keep = ones( xcl.shape)
           rs,bs,ms = 1e8,3e8,0.0
           oldsol = 1.0*sol
           newrs = 1e7

           print "Using nf=",nf,ltype
           nf = min([nf,order-1])
           while len(sol) < nf + 1: sol = concatenate([sol,[0]])
           while sum(equal(keep,0)) or first:
               rs=newrs

               if not first:
                  xcl,wcl,acl,dcl = compress(keep,[xcl,wcl,acl,dcl],1)
                  wfft = asarray(wcl)
                  xfft = (asarray(xcl)-xc)/xw
                  bfft = legendre(xfft,len(sol)-1)
                  tsol1 = add.reduce(sol[::,NewAxis]*bfft,0) - wfft
                  tsol2 = medsmooth(tsol1)
                  tsol3 = 1.49*medsmooth(abs(tsol1-tsol2))
                  tsol3 = clip(tsol3,fwhm*disp/4,tsol3)
                  tokay = less(abs(tsol1-tsol2)/tsol3,[5,10][first])

                  xcl,wcl,acl,dcl = compress(tokay,[xcl,wcl,acl,dcl],1)

               wfft = asarray(wcl)
               xfft = (asarray(xcl)-xc)/xw
               bfft = legendre(xfft,len(sol)-1)
               tsol1 = add.reduce(sol[::,NewAxis]*bfft,0) - wfft
               tsol2 = medsmooth(tsol1)
               tsol3 = 1.49*medsmooth(abs(tsol1-tsol2))
               tsol3 = clip(tsol3,fwhm*disp/4,tsol3)

#               temp = open("temp.dat","w")
#               for jx in range(len(xcl)):
#                 print >>temp, "%10.3f %10.3f %10.3f %10.3f %10.3f" % (xcl[jx], wfft[jx], tsol1[jx], tsol1[jx]-tsol2[jx], (tsol1[jx]-tsol2[jx])/tsol3[jx])
#               temp.close()
#               raw_input("<cr>")

               if ltype in ["sky","kamp"]:
                 sfft = planefit(transpose(bfft),wfft,sol,minfunc=mad)[0]
               elif ltype in ["arc","amp"]:
                 if first: sfft = svdfit(transpose(bfft),wfft)
                 else: sfft = planefit(transpose(bfft),wfft,sfft,minfunc=mad)[0]
               print "New Solution=",
               print sfft
               wo = add.reduce(sfft[::,NewAxis]*bfft)
               resid = wo-wfft
               ms,bs,newrs=mad(resid),bwt(resid)[1],rms(resid)
               print "MAD, BWT, RMS, LEN", ms,bs,newrs, len(resid)
               first = 0
               if len(resid) > len(sol):

                  tsol1 = resid
                  tsol2 = medsmooth(tsol1)
                  tsol3 = 1.49*medsmooth(abs(tsol1-tsol2))
                  tsol3 = clip(tsol3,fwhm*disp/4,tsol3)
                  keep = less(abs(tsol1-tsol2)/tsol3,5)
                  print median(tsol3),sum(keep)

         sfft = planefit(transpose(bfft),wfft,sol,minfunc=mad)[0]
#         sfft = svdfit(transpose(bfft),wfft)
         print "New FFT Solution="
         print sfft
         print
         sol = sfft

#         temp = open("temp.dat","w")
#         for jx in range(len(xcl)):
#           print >>temp, "%10.3f %10.3f %10.3f %10.3f %10.3f" % (xcl[jx], wfft[jx], tsol1[jx], tsol1[jx]-tsol2[jx], (tsol1[jx]-tsol2[jx])/tsol3[jx])
#         temp.close()
#         raw_input("<cr>")

      sfft = svdfit(transpose(bfft),wfft)

      return bs,rs,sol

def minfft(disp,p,q,c,lw,ln,lh,lt,fwhm,amp,w0,w1,order,ltype,wfft):

    if disp > 0.2: drat = 0.08
    else: drat = 0.04
    wguess = mean(lw)
    if ltype not in ["amp","arc"] or (disp > 0.0015):
       for nit in range(1):
          disps = disp*(1+arange(-drat/(1+nit),drat/(1+nit),0.008,Float32))
          nw = max([int((w1-w0)/disp),len(c)])
          print "Constructing templates with w0,w1,dispersions:",w0,w1
          print disps
          templates = asarray(map(lambda l1: mkfake(w0,w0+nw*l1,nw,lw,ln,lh,lt,2*fwhm,amp=1)/(l1*nw**2), disps))
#          qdump("temp0.fits",templates,extras=[["crpix1",1],["crval1",w0],["cdelt1",disp],["dc-flag",0]])

          nc = len(c)
          aw0 = 0
          bw1 = nw
          n = pow(2,int(log2(nw-1))+2)
          qc1 = int(10*fwhm)
          qc2 = int(200*fwhm)
          print "FFTing"
          cf = real_fft(c,n)
          qf = greater(arange(len(cf)),qc1)* less(arange(len(cf)),qc2)
          tf = map(lambda t: real_fft(t[aw0:bw1],n), templates)
          print "Cross correlating"
          fcc = asarray(map(lambda tf: inverse_real_fft(qf * cf * conjugate(tf)), tf))
          fcc = take(fcc,range(-n/2,n/2),1)
#          qdump("temp3.fits",fcc)

          acc = array(map(argmax, fcc))
          wcc = (w0+w1)/2.0 + (n/2.0-acc)*disps
          print wcc
          wguess = median(wcc)
          print wguess
          mcc = array(map(max, fcc))
          aa = argmax(mcc)

#          qdump("temp5.fits",mcc)

          print "Found  d=%9.3f" % (disps[aa])
          disp = disps[aa]

    args = (p,q,c,lw,ln,lh,lt,fwhm,amp,w0,w1,order,ltype,wfft)
    ms,rs,sol = apply(fftsol,([wguess,disp],)+args)

    return rs,sol

def genspac(slw,sr):
    dslw = slw[::,NewAxis] - slw[NewAxis,::]
    islw = indices(dslw.shape)[1]
    oslw = asarray([take(argsort(abs(dslw[j])),sr) for j in range(dslw.shape[0])])

    tslw = asarray([take(dslw[j],oslw[j]) for j in range(dslw.shape[0])])
    jslw = asarray([take(islw[j],oslw[j]) for j in range(dslw.shape[0])])

    mslw = asarray([argsort(tslw[j]) for j in range(dslw.shape[0])])
    nslw = asarray([take(tslw[j],mslw[j]) for j in range(dslw.shape[0])])
    kslw = asarray([take(jslw[j],mslw[j]) for j in range(dslw.shape[0])])

    rslw = (nslw[::,2:]-nslw[::,0][::,NewAxis]) / (nslw[::,1] - nslw[::,0])[::,NewAxis]

    return rslw,kslw

def spacmatch(disp,xc,xw,p,q,c,lw,ln,lh,lt,fwhm,amp,w0,w1,order,ltype,wfft):
    dw = disp*p
    wran = disp*xw
    wmin = w0 - wran/2
    wmax = w1 + wran/2
    lu = nonzero(between(lw,wmin,wmax)*greater(lh,0))
    lwu,lhu = take([lw,lh],lu,1)
    allmatches = []
    np = len(p)
    nw = len(lwu)
    maxis = (np < nw)
    nf = min([np,nw]) - 1
    ydor,xdor = indices( (np,nw))

    nl = min([10,nf])

    slw,slh = take([lwu,lhu],argsort(lhu)[::-1][:nl],1)

    glw = [0]
    for j in range(1,slw.shape[0]):
       if min(abs(slw[j]-take(slw,glw))) > 2*disp*fwhm: glw.append(j)
    slw,slh = take([slw,slh],glw,1)

    sp,sq = take([p,q],argsort(q)[::-1][:nl],1)

    nsp = concatenate([abs(sp[j]-sp[j+1:4]) for j in range(3)])
    nslw = concatenate([abs(slw[j]-slw[j+1:4]) for j in range(3)])
    print nsp
    print nslw
    print sort((nslw[::,NewAxis]/nsp[NewAxis,::]).flat)
    disps = (nslw[::,NewAxis]/nsp[NewAxis,::]).flat
#    qdump( "crap.fits", sort(disps))

#    slw,slh = take([slw,slh],argsort(slw),1)
#    sp,sq = take([sp,sq],argsort(sp),1)

    dsp =  sp[::,NewAxis]  - sp[NewAxis,::]
    isp = indices(dsp.shape)
    dslw = slw[::,NewAxis] - slw[NewAxis,::]
    islw = indices(dslw.shape)
    print
#    disps = array([disp])

    for j in range(disps.shape[0]):
       u = []
       rslw = dslw/disps[j]
       mt = []
       for k in range(sp.shape[0]):
         for l in range(slw.shape[0]):
             M = dsp[k][::,NewAxis]-rslw[l][NewAxis,::]
             K,L = indices(M.shape)
             I1 = argsort(abs(M),1)
             M1 = asarray([take(M[m],I1[m])[0] for m in range(I1.shape[0])])
             K1 = asarray([take(K[m],I1[m])[0] for m in range(I1.shape[0])])
             L1 = asarray([take(L[m],I1[m])[0] for m in range(I1.shape[0])])
             K2 = take(sp,K1)
             L2 = take(slw,L1)
             M2 = less(abs(M1),1.0)
             u = compress(M2,[K2,L2],1)
             if len(u) and sum(M2)>1:
#                print compress(M2,M1)
                mt.append(u)
#       print disps[j],mt

    exit()

#    sp,sq =   take([lwu,lhu],argsort(lhu)[::-1],1)

    nl = min([3,nf])

    sr = range(nl)
    srs = [sr]
#    for j in sr[1:]:
#       sr2 = range(nl+1)
#       sr2.remove(j)
#       srs += [sr2]
#    print srs
#    print

    for sr in srs:
       rslw,kslw = genspac(slw,sr)
       rsp,ksp = genspac(sp,sr)

       print sr 
       for j in range(rsp.shape[0])[:1]:
           m1 = []
           m2 = []
           s1 = []
           s2 = []
           for k in range(rslw.shape[0]):
             rrsp = rsp[j] / rslw[k]
             t1 = mean(rrsp); t2 = rms(rrsp,t1)
#             t1 = median(rrsp); t2 = 1.49*mad(rrsp,t1)
             print j,k,rsp[j],rslw[k],bwt(rsp[j]/rslw[k])
             print take(sp,ksp[j])
             print take(slw,kslw[k])
             print
             if abs(t1-1) < 0.05 and t2 < 0.05:
                 m1.append(ksp[j])
                 m2.append(kslw[k])
                 s1.append(t1-1)
                 s2.append(t2)
           print j,sp[j]
           print s1
           print s2
           print m1
           print m2
           print take(sp,m1)
           print take(slw,m2)
           print



    exit()
#    gss = []
#    hss = []
#    lss = []
#    pss = []
#
#    for k1 in range(nl-1):
#       for k2 in range(k1+1,nl):
#           print (k1,k2),;sys.stdout.flush()
#           sss = slw - min([slw[k1],slw[k2]])
#           dslw = abs(slw[k1] - slw[k2])
#           sss = sss/dslw
#           gs1 = []
#           hs1 = []
#           ls1 = []
#           ps1 = []
#
#           for j1 in range(nl-1):
#             for j2 in range(j1+1,nl):
#                 ss = sp - min([sp[j1],sp[j2]])
#                 dsp = abs(sp[j1] - sp[j2])
#                 ss = ss/dsp
#                 dss = ss[::,NewAxis] - sss[NewAxis,::]
#                 nd0 = dslw/dsp
#                 hsv = max(minimum.reduce(abs(dss),1))
#                 if hsv < 0.05 and abs(nd0/disp-1) < 0.2:
#                    hs1.append(hsv)
#                    gs1.append(nd0)
#                    ls1.append(sort([slw[k1],slw[k2]]))
#                    ps1.append(sort([sp[j1],sp[j2]]))
#           if len(hs1):
#              hs1,gs1 = asarray([hs1,gs1])
#              ns1 = argsort(hs1)[:5]
#              lw1 = take(ls1,ns1,0)
#              pw1 = take(ps1,ns1,0)
#              lss.append(lw1)
#              pss.append(pw1)
#
#    print
#    lss = ravel(transpose(concatenate(lss)))
#    pss = ravel(transpose(concatenate(pss)))
#    fea1 = []
#    fea2 = []
#    fea3 = []
#    for j in range(lss.shape[0]):
#        if pss[j] not in fea1:
#           fea1.append(pss[j])
#           fea2.append([lss[j]])
#           fea3.append([1])
#        elif pss[j] in fea1:
#           if lss[j] in fea2[fea1.index(pss[j])]:
#             fea3[fea1.index(pss[j])][fea2[fea1.index(pss[j])].index(lss[j])] += 1
#           else:
#             fea2[fea1.index(pss[j])].append(lss[j])
#             fea3[fea1.index(pss[j])].append(1)
#
#    fea0 = []
#    for j in range(len(fea1)):
#       fea0.append([fea1[j]]*len(fea2[j]))
#       fea2[j] = take(fea2[j],argsort(fea3[j])[::-1])
#       fea3[j] = take(fea3[j],argsort(fea3[j])[::-1])
#       print fea1[j]
#       print fea2[j]
#       print fea3[j]
#       print
#
#    fea0 = concatenate(fea0)
#    fea1 = asarray(fea1)
#    fea2 = concatenate(fea2)
#    fea3 = concatenate(fea3)**2
#    basis = legendre((fea0-xc)/xw,order-1)
#    sol = svdfit(transpose(basis*fea3),fea2*fea3)
#    basis = legendre((fea1-xc)/xw,order-1)
#    print fea1
#    print add.reduce(sol[::,NewAxis]*basis,1)
#    return -99,sol
#
    exit()

def wavecalib(s,order,sol,w0=3000.0,w1=11000.0,fwhm=5.0,thresh=5.0,disp=0.0,
       dich=None,cut=2.0,amp=0.0,lines="skylines.dat",ltype="sky",wzero=0,wfft=0,pgapop=0,pgamit=0,verb=0,
       tempfits="temp.fits",temp2fits="temp2.fits",full_output=0,splsol=1,usep2=0,useqr=0):

   ns = len(s)
   x = arange(0,ns,1,Float32)
   xc = (x[0]+x[-1])/2.0
   xw = (x[-1]-x[0])/2.0

   if wzero and temp2fits:
      sol = asarray(sol)
      basis = legendre((x-xc)/xw,len(sol)-1)
      wo = add.reduce(sol[::,NewAxis]*basis)
      tck = splrep(wo,s,s=0,k=1)
      dw = (wo[-1]-wo[0])/len(wo)
      w = dw*x + wo[0]
      co = splev(w,tck)
      qdump(temp2fits,co,extras=[["crpix1",1],["crval1",wo[0]],["cdelt1",dw],["dc-flag",0]])

   b,c,t,g=sifty(s,2*(int(10*fwhm)/2)+1)
   if tempfits: qdump(tempfits,c,extras=[["crpix1",1.0]])

   p,f,q,fwhm=getlines(x,s,c,t,g,fwhm,cut,thresh)
   if not len(f):
      res = 1e24
      co = zeros(c.shape,Float32)
      if full_output:
         return sol,res,p,f,q
      else:
         return sol,res,co

   if (wzero and len(p) < wzero):
      wzero = 1
      if not(len(p)): raise "LackOfFeaturesError"
   lw,ln,lh,lt,lu = readll(lines,ltype,fwhm,disp,dich)
   print "Read %d lines" % (len(lw))
   res = -99.9

   if wzero == 0:
      if not disp: print "Error: need a guess for the dispersion";exit()
      if pgapop and pgamit:
         usol = array([0.5*(w0+w1),xw*disp])
         if order  > 2:
           usol = concatenate([usol,(order-2)*[0.0]])
         ctol = 1.0
         sol = QuickMatchUp(w0,w1,p,(p-xc)/xw,f,q,lw,lh,usol,fwhm=fwhm,disp=disp,pgapop=pgapop,pgamit=pgamit)
      else:
         res,sol = minfft(disp,p,q,c,lw,ln,lh,lt,fwhm,amp,w0,w1,order,ltype,wfft)
      res = -99.9

   if 0 <= wfft < 1:
         if ltype == "sky": uwr = [2,1,0]; mamp = 1000
         elif ltype in ["arc","amp"]:
            uwr = [2,1,0]; mamp = 10
         if wzero == order: uwr=[0]
#         if pgapop and pgamit: uwr=[0]
         for jo in range(len(sol),order+1):
           basis = legendre((p-xc)/xw,len(sol)-1)
           for wr in power(2.0,uwr):
               mamp = max([mamp/2,5])
#               verb = 0
               res,sol = dispsol(sol,basis,x,p,f,q,lw,ln,lh,lt,lu,wr,fwhm,mamp,wzero,ltype,usep2=usep2,useqr=useqr,verb=verb)
           if ltype == "sky": uwr = [1,0,-1]; usep2 = 1
           elif ltype in ["arc","amp"]: uwr = [1,0]
           if order > len(sol): sol = concatenate([sol, [0.0]])
         if wzero: wrfinal = 1/2.35
         else: wrfinal = 2/3.
         res,sol = dispsol(sol,basis,x,p,f,q,lw,ln,lh,lt,lu,wrfinal,fwhm,mamp,wzero,ltype,usep2=usep2,verb=1)
         res,sol = dispsol(sol,basis,x,p,f,q,lw,ln,lh,lt,lu,wrfinal,fwhm,mamp,wzero,ltype,usep2=usep2,verb=1)
         print "Final Scatter = ",res
   sol = asarray(sol)
   if splsol:
      basis = legendre((x-xc)/xw,len(sol)-1)
      wo = add.reduce(sol[::,NewAxis]*basis)
      derw = (wo[2:]-wo[:-2])/2.0
      derw = concatenate([[derw[0]],derw,[derw[-1]]])
      gw = nonzero(greater(derw,0))+1
      g0 = gw[0]; g1 = gw[-1]
      for j in range(g0,-1,-1): wo[j] = wo[j+1] - derw[gw[g0]]
      for j in range(g1,len(wo)): wo[j] = wo[j-1] + derw[gw[g1]]
      try:
         tck = splrep(wo,s,s=0,k=1)
         dw = (wo[-1]-wo[0])/len(wo)
         w = dw*x + wo[0]
         co = splev(w,tck)
         if temp2fits: qdump(temp2fits,co,extras=[["crpix1",1],["crval1",wo[0]],["cdelt1",dw],["dc-flag",0]])
      except:
         print "problem splrep'ing",wo[0],wo[-1]
         qdump("dww.fits",derw)
         qdump("www.fits",wo)
         co = zeros(x.shape)

   if full_output:
      return sol,res,p,f,q
   else:
      return sol,res,co

class minfftClass:
    def __init__(self,disp,xc,xw,p,f,q,c,lw,ln,lh,lu,lt,fwhm,amp,w0,w1,order,ltype,wfft):
       self.disp=disp; self.p=p; self.f=f; self.q=q; self.c=c; self.lw=lw; self.ln=ln; self.lh=lh; self.lu=lu; self.lt=lt
       self.fwhm=fwhm; self.amp=amp; self.w0=w0; self.w1=w1; self.order=order; self.ltype=ltype; self.wfft=wfft
       self.xc=xc; self.xw=xw
       self.sqs = []
       self.fqs = []
    def genetic(self,pop,mit):
       wc = (self.w0+self.w1)/2.0
       self.LowerLimits = array([wc-0.25*(self.w1-self.w0),0.9*self.disp,0.5*self.fwhm])
       self.UpperLimits = array([wc+0.25*(self.w1-self.w0),1.1*self.disp,2.0*self.fwhm])
       print "LL=",self.LowerLimits
       print "UL=",self.UpperLimits
       FuncGenetic(self.tryfft,self.LowerLimits,self.UpperLimits,pop,mit)
       wc,disp,fwhm,sq = transpose(self.sqs)
       sol = self.fqs[argmax(sq)]
       return wc,disp,fwhm,sol,sq
    def tryfft(self,params):
       print params
       wc,disp,fwhm = params
       print
       print wc,disp,fwhm
       print
       w0 = wc - self.xw*disp
       w1 = wc + self.xw*disp
       res,sol = minfft(disp,self.p,self.q,self.c,self.lw,self.ln,self.lh,self.lt,fwhm,self.amp,w0,w1,self.order,self.ltype,self.wfft)
       bases = legendre((self.p-self.xc)/self.xw,len(sol)-1)
       w1 = dot(sol,bases)
       dw1 = abs(w1[::,NewAxis] - self.lw[NewAxis,::])
       c1 = argmin(dw1,1)
       o1 = take(self.lw,c1)
       u1 = 1.*less(abs(w1-o1),self.f*disp/4)*take(self.lu,c1)
       sq = sum(u1*self.q*take(self.lh,c1))
       self.sqs.append([wc,disp,fwhm,sq])
       self.fqs.append(sol)
       return sq

def FuncGenetic(Func,LowerLimits,UpperLimits,Pop,MaxIter):
   N = len(LowerLimits)
   GenCon = Context(PGA_DATATYPE_REAL, N, PGA_MAXIMIZE)
   GenCon.Set("RealInitRange",LowerLimits,UpperLimits)
   GenCon.Set("PopReplacementType",PGA_POPREPL_BEST)
   GenCon.Set("PopSize",Pop)
   GenCon.Set("NumReplaceValue",int(0.8*Pop))
   GenCon.Set("StoppingRuleType",PGA_STOP_MAXITER)
   GenCon.Set("MaxGAIterValue",MaxIter)
#   GenCon.Set("StoppingRuleType",PGA_STOP_TOOSIMILAR)
#   GenCon.Set("MaxSimilarityValue",95)
   GenCon.Set("MutationOrCrossoverFlag",PGA_TRUE)
   GenCon.Set("MutationType",PGA_MUTATION_GAUSSIAN)
   GenCon.Set("MutationRealValue",0.10)
   GenCon.Set("MutationBoundedFlag",PGA_FALSE)
   GenCon.Set("NoDuplicatesFlag",PGA_TRUE)
#   GenCon.Set("NoDuplicatesFlag",PGA_FALSE)
   GenCon.Set("CrossoverType",PGA_CROSSOVER_ONEPT)
   GenCon.SetUp()
   GenCon.Run(Func)

def QuickMatchUp(w0,w1,p,x,f,q,l,a,w,fwhm=4,disp=1,pgapop=800,pgamit=20):
    l,a = take([l,a],argsort(l),1)
    p,x,f,q = take([p,x,f,q],argsort(p),1)
    wo = len(w) - 1
    pw = dot(w,legendre(x,wo))
    G = GeneticSolution(w0,w1,p,x,pw,f,q,w,l,a,fwhm,disp,pgapop=pgapop,pgamit=pgamit)
    return G.best

class GeneticSolution:
   def __init__(self,w0,w1,p,x,pw,f,q,w,l,a,fwhm,disp,pgapop=350,pgamit=30):
       self.pgapop=pgapop
       self.pgamit=pgamit
       self.p = p
       self.x = x
       self.pw = pw
       self.f = f
       self.q = q
       self.l, self.a = compress(a,[l,a],1)
       self.fwhm = fwhm
       self.disp = disp
       LowerLimits = array([w0,0.95*w[1]] + (-0.15 * power(2,range(min([2,(len(w)-3)])))).tolist())
       UpperLimits = array([w1,1.05*w[1]] + (+0.15 * power(2,range(min([2,(len(w)-3)])))).tolist())
       self.wsols, self.nsols = [], []
       Genetic(self.Generate, LowerLimits, UpperLimits, self.pgapop, self.pgamit)
       self.best = self.wsols[argmax(self.nsols)]
       self.n,self.m = self.Generate(self.best,done=1)
       self.n,self.m = self.Generate(self.best,done=1)
       print self.best
   def Generate(self,w_in,done=0):
       w = 1*w_in
       if not done:
          for j in range(2,len(w)): w[j] = w[j]*abs(w[j-1])
       wo = len(w) - 1
       pw = dot(w,legendre(self.x,wo))
       dw = pw[::,NewAxis] - self.l[NewAxis,::]
       widths = self.disp*self.f/2.35
       dist = abs(dw)/widths[::,NewAxis]
       udw = 1.0*less(dist,1.0)
       if done:
          n,m = indices(udw.shape)
          n,m = compress(udw.flat,[n.flat,m.flat],1)
          return n,m
       else:
#          amp2 = sqrt(self.q[::,NewAxis]*self.a[NewAxis,::])
#          amp2 = sqrt(self.a[NewAxis,::])
#          amp2 = sqrt(self.q[::,NewAxis])
          ndw = add.reduce(udw,1)
          tdw = add.reduce(ndw)
          self.wsols.append(w)
          self.nsols.append(tdw)
          return tdw

def Genetic(Generate,LowerLimits,UpperLimits,pgapop,pgamit):
   B = Context(PGA_DATATYPE_REAL, len(LowerLimits), PGA_MAXIMIZE)
   B.Set("RealInitRange",LowerLimits,UpperLimits)
   B.Set("PopReplacementType",PGA_POPREPL_BEST)
   B.Set("PopSize",pgapop)
   B.Set("NumReplaceValue",pgapop-1)
   B.Set("StoppingRuleType",PGA_STOP_NOCHANGE)
   B.Set("MaxNoChangeValue",pgamit/2)
   B.Set("StoppingRuleType",PGA_STOP_MAXITER)
   B.Set("MaxGAIterValue",pgamit)
   B.Set("PrintFrequencyValue",10)
   B.Set("MutationOrCrossoverFlag",PGA_TRUE)
   B.Set("CrossoverType",PGA_CROSSOVER_ONEPT)
   B.Set("CrossoverProb",0.9)
#   B.Set("MutationProb",0.9)
   B.Set("MutationType",PGA_MUTATION_GAUSSIAN)
   B.Set("MutationRealValue",1.0)
   B.Set("MutationBoundedFlag",PGA_TRUE)
   B.Set("NoDuplicatesFlag",PGA_TRUE)
#   B.Set("NoDuplicatesFlag",PGA_FALSE)
   B.SetUp()
   B.Run(Generate)
   del B

