
from Multipack import *
from FFT import *
from VTKHelperFunctions import *
from FITS import *
from MLab import *
from basics import *
from masks import *
from Expector import *
from LineIdentify import *
from pgapack import *
from pgplot import *

def PlotEm(odev,image,b2,i1s,p1s,sxs,sys,fsol,l1s):
   opendev(odev,nx=2,ny=2,land=1,size=11)

   xa = arange( b2[0]-7*b2[1], b2[0]+7*b2[1]+b2[1]/5, b2[1]/5, Float32)
   ra = diff(xa)/2.0 + xa[:-1]
   na = diff(searchsorted(sort(fsol-l1s),xa))
   newbox([b2[0]-7*b2[1], b2[0]+7*b2[1]],[0,1.5*max(na)],ch=2)
   xlabel("\gD\gl (\A)",ch=2)
   ylabel("N",ch=2)
   subtitle(image,ch=2)
   drawln(ra,na,hist=1,ls=1)

   ymar = b2[0]-7*b2[1], b2[0]+7*b2[1]
   xmar = extrema(p1s)
   xmar = add(xmar,multiply(diff(xmar)[0],[-0.15,0.15]))
   newbox( xmar,ymar,ch=2)
   xlabel("x",ch=2)
   ylabel("\gD\gl (\A)",ch=2)
   drawpt(p1s,fsol-l1s,ps=17,ch=1.5)

   ii = extrema(i1s)
   bulk = [compress(equal(i1s,ii1),[sxs,sys,fsol-l1s],1)  for ii1 in range(ii[0],ii[1]+1,1)]

   ymar = b2[0]-7*b2[1], b2[0]+7*b2[1]
   xmar = extrema(sys)
   xmar = add(xmar,multiply(diff(xmar)[0],[-0.15,0.15]))
   newbox(xmar,ymar,ch=2)
   xlabel("y",ch=2)
   ylabel("\gD\gl (\A)",ch=2)
   drawpt(sys,fsol-l1s,ps=17,ch=1.5)
   for j in range(len(bulk)):
      drawpt1( median(bulk[j][1]), median(bulk[j][2]), ps=17,ch=3,ci=2)

   ymar = b2[0]-7*b2[1], b2[0]+7*b2[1]
   xmar = extrema(sxs)
   xmar = add(xmar,multiply(diff(xmar)[0],[-0.15,0.15]))
   newbox(xmar,ymar,ch=2)
   xlabel("X",ch=2)
   ylabel("\gD\gl (\A)",ch=2)
   drawpt(sxs,fsol-l1s,ps=17,ch=1.5)
   for j in range(len(bulk)):
      drawpt1( median(bulk[j][0]), median(bulk[j][2]), ps=17,ch=3,ci=2)

   closedev()

def GeneratePlots(image,b2,i1s,p1s,sxs,sys,fsol,l1s):
   odev = "/xs"
   PlotEm(odev,image,b2,i1s,p1s,sxs,sys,fsol,l1s)
   odev = image.replace(".fits","wdist.ps/vcps")
   PlotEm(odev,image,b2,i1s,p1s,sxs,sys,fsol,l1s)

def GetLines(i,s,fwhm=4,cut=3,thresh=5):
   print "Looking for lines in slit #%d." % (i+1)
   ns = len(s)
   x = arange(0,ns,1,Float32)
   xc = (x[0]+x[-1])/2.0
   xw = (x[-1]-x[0])/2.0
   b,c,t,g=sifty(s,2*(int(10*fwhm)/2)+1)
   p,f,q,fwhm=getlines(x,s,c,t,g,fwhm,cut,thresh)
   p,f,q = compress( greater(q,0), [p,f,q], 1)
   return i*ones(p.shape,Float32),p,f,q

def QuickMatchUp(p,x,f,q,l,a,w,xs,ys,fwhm=4,disp=1,ctol=0.05,pgapop=800,pgamit=20):
    l,a = take([l,a],argsort(l),1)
    p,x,f,q = take([p,x,f,q],argsort(p),1)
    wo = len(w) - 1
    pw = dot(w,legendre(x,wo))
    G = GeneticSolution(p,x,pw,f,q,w,l,a,fwhm,disp,ctol=ctol,pgapop=pgapop,pgamit=pgamit)
    return G.best

class GeneticSolution:
   def __init__(self,p,x,pw,f,q,w,l,a,fwhm,disp,pgapop=350,pgamit=30,ctol=0.05):
       self.pgapop=pgapop
       self.pgamit=pgamit
       self.p = p
       self.x = x
       self.pw = pw
       self.f = f
       self.q = q
       self.l = l
       self.a = a
       self.fwhm = fwhm
       self.disp = disp
       self.LowerLimits = array([w[0]-ctol*w[1],w[1]/1.10,-w[1]/20])
       self.UpperLimits = array([w[0]+ctol*w[1],w[1]*1.10, w[1]/20])
       self.Play()
       w = self.best
       print w
       self.n,self.m = self.Generate(self.best,done=1)
   def Play(self):
       self.wsols, self.nsols = [], []
       Genetic(self.Generate,self.LowerLimits, self.UpperLimits, self.pgapop, self.pgamit)
       self.best = self.wsols[argmax(self.nsols)]
   def Generate(self,w,done=0):
       wo = len(w) - 1
       pw = dot(w,legendre(self.x,wo))
       dw = pw[::,NewAxis] - self.l[NewAxis,::]
       udw = 1.0*less(abs(dw),self.f[::,NewAxis]*self.disp)
       if done:
          n,m = indices(udw.shape)
          n,m = compress(udw.flat,[n.flat,m.flat],1)
          return n,m
       else:
          ndw = add.reduce(udw*self.a[NewAxis,::],1)
          tdw = add.reduce(ndw*self.q)
          self.wsols.append(w)
          self.nsols.append(tdw)
          return tdw

def Genetic(Generate,LowerLimits,UpperLimits,pgapop,pgamit):
   B = Context(PGA_DATATYPE_REAL, len(LowerLimits), PGA_MAXIMIZE)
   B.Set("RealInitRange",LowerLimits,UpperLimits)
   B.Set("PopReplacementType",PGA_POPREPL_BEST)
   B.Set("PopSize",pgapop)
   B.Set("NumReplaceValue",int(0.90*pgapop))
   B.Set("StoppingRuleType",PGA_STOP_MAXITER)
   B.Set("MaxGAIterValue",pgamit)
#   B.Set("StoppingRuleType",PGA_STOP_TOOSIMILAR)
#   B.Set("MaxSimilarityValue",95)
   B.Set("MutationOrCrossoverFlag",PGA_TRUE)
   B.Set("MutationType",PGA_MUTATION_GAUSSIAN)
   B.Set("MutationRealValue",0.10)
   B.Set("MutationBoundedFlag",PGA_FALSE)
   B.Set("NoDuplicatesFlag",PGA_TRUE)
#   B.Set("NoDuplicatesFlag",PGA_FALSE)
   B.Set("CrossoverType",PGA_CROSSOVER_ONEPT)
   B.SetUp()
   B.Run(Generate)

def Tighten(slit,wsol,xa,x,p,f,q,linew, linen, linea, linet, lineu, fwhm, extend=1, word=5):
    print "Object #",slit+1
    for k in range(1):
       if extend: t = [2.0, 1.0, 0.5, 0.5, 0.5]
       else: t = [2.0, 2.0, 1.5, 0.75, 0.5,0.5]
       a = [0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01]
       for j in range(len(t)):
          if extend and k==0 and j in [1,2,3] and len(wsol) < word+1: wsol = concatenate([wsol,[0.0]])
          bases = legendre(x,len(wsol)-1)
          if k: wz = len(wsol)+1
          else: wz = k
          www = dispsol(wsol, bases, xa, p, f, q, linew, linen,\
             linea, linet, lineu, t[j], fwhm, a[j], wz, "sky", verb=1)
          wrms,wsol = www
          print "Scatter = ",wrms
          print
    return www

