#!/usr/bin/env python

from types import *
from math import atan2
from VTKHelperFunctions import *
from masks import *
from ephem import *
from basics import *

class multiastro:
    def __init__(self,i,n,wt=None,obj=None,suffix="cat"):
       self.chip = i
       self.image = n
       self.name = replace(n,".fits",".soln")
       self.catalog = replace(self.image,".fits",".%s" % (suffix))
       self.wt = wt
       self.obj = obj
       if self.wt:
          self.wtimage = replace(self.image,self.obj+".fits",self.wt+".fits")
    def __repr__(self):
       return self.name
    def center(self,r,d):
       self.ra = formatSexagesimal(r/15.0,4,3600000)
       self.dec = formatSexagesimal(d,4,3600000)
       self.rac = degrad(r)
       self.decc = degrad(d)
    def readorig(self,verb=0):
       if verb: print "Reading",self.name
       self.origsoln = readsoln(self.name)
    def read(self,verb=0):
       if verb: print "Reading",self.name
       f=open(self.name,"r")
       l = split(f.readline())
       self.nterms = int(l[0])
       self.xc = float(l[2])
       self.yc = float(l[3])
       l = split(f.readline())
       r = 15.0*scanSexagesimal(l[0])
       d =  scanSexagesimal(l[1])
       self.center(r,d)
       self.tran = []
       for j in range(self.nterms):
         self.tran.append(float(f.readline()))
       f.close()
       self.tran = array(self.tran)
       self.tranx = take(self.tran,range(0,self.nterms,2))
       self.trany = take(self.tran,range(1,self.nterms,2))
       self.scalex = sqrt(self.tranx[1]**2 + self.tranx[2]**2)*180/pi * 3600
       self.scaley = sqrt(self.trany[1]**2 + self.trany[2]**2)*180/pi * 3600
       self.scale = (self.scalex + self.scaley)/2.0
    def write(self,verb=0):
       if verb: print "Writing",self.name
       f=open(self.name,"w")
       f.write("%d 0 %9.3f %9.3f 0.0\n" % (self.nterms,self.xc,self.yc))
       f.write("%13s %13s\n" % (self.ra,self.dec))
       self.tran = concatenate([[self.tranx[j],self.trany[j]] for j in range(self.nterms/2)])
       [f.write("%17.9g\n" % (k)) for k in self.tran]
       f.close()
    def sex(self,sexdir="./sex/",thresh=5.0,saturate=30000,segm=0,fwhm=1.0):
       self.sexcom = ["sex",self.image]
       self.sexcom += ["-c "+sexdir+"default.sex"]
       self.sexcom += ["-PARAMETERS_NAME "+sexdir+"default.param"]
       if self.wt:
          self.sexcom += ["-WEIGHT_TYPE MAP_RMS"]
          self.sexcom += ["-WEIGHT_IMAGE "+self.wtimage]
       else:
          self.sexcom += ["-WEIGHT_TYPE BACKGROUND"]
       self.sexcom += ["-DETECT_MINAREA 9"]
       self.sexcom += ["-DETECT_THRESH "+str(thresh)]
       self.sexcom += ["-ANALYSIS_THRESH "+str(thresh)]
       self.sexcom += ["-PIXEL_SCALE "+str(self.scale)]
       self.sexcom += ["-SEEING_FWHM "+str(fwhm)]
       self.sexcom += ["-CATALOG_NAME",self.catalog]
#        self.sex += ["-FILTER_NAME "+sexdir+"default.conv"]
       self.sexcom += ["-FILTER_NAME "+sexdir+"gauss_3.0_5x5.conv"]
       self.sexcom += ["-STARNNW_NAME "+sexdir+"default.nnw"]
       self.sexcom += ["-SATUR_LEVEL "+str(saturate)]
       if segm:
          self.sexcom += ["-CHECKIMAGE_TYPE SEGMENTATION"]
          self.sexcom += ["-CHECKIMAGE_NAME "+ self.image.replace(".fits","seg.fits")]
       self.sexcom = join(self.sexcom)
       verbex(self.sexcom)
    def readcat(self,refsim=None,nuse=None,clean=0,all=0):
       self.nuse = nuse
       self.clean = clean
       self.all = all
       self.dat = readtable(self.catalog,icols=[0,7],start=9)
       self.imx = self.dat[1]; self.imy = self.dat[2]
       self.gra,self.gdec = xy2rd(self.name,self.imx,self.imy,text=0)
       self.nx, self.ny = normalcoord(self.gra,self.gdec,self.rac,self.decc,conv=3600*180/pi)
       self.dat[1] = self.nx
       self.dat[2] = self.ny
#       self.cra, self.cdec = xy2rd(self.name,array([0.0,2*self.xc]),array([0.0,2*self.yc]),text=0)
       self.cra, self.cdec = xy2rd(self.name,extrema(self.imx),extrema(self.imy),text=0)
       self.cx, self.cy = normalcoord(self.cra,self.cdec,self.rac,self.decc,conv=3600*180/pi)
       self.catdat = self.dat
#       if self.chip == 0: print "x",self.catmatch.x[:5]
       if refsim:
          self.refsim = refsim
          refdat = readsim(refsim)[0]
          refra = array([hrrad(scanSexagesimal(r)) for r in refdat[3]])
          refdec = array([degrad(scanSexagesimal(d)) for d in refdat[4]])
          refnx, refny = normalcoord(refra,refdec,self.rac,self.decc,conv=3600*180/pi)
          if self.nuse: self.rnuse = self.nuse
          else: self.rnuse = None
          if not all:
             self.xspan = diff(extrema(self.cx))[0]
             self.yspan = diff(extrema(self.cy))[0]
             self.uspan = max([self.xspan,self.yspan])/2/sqrt(2)
             self.xcen = mean(self.cx)
             self.ycen = mean(self.cy)
             print "pos/spans: %.2f %.2f" % (self.xcen, self.ycen),
             print "%.2f %.2f %.2f" % (self.xspan, self.yspan, self.uspan)
             keep = between(refnx,self.xcen-2*self.uspan,self.xcen+2*self.uspan) * \
                 between(refny,self.ycen-2*self.uspan,self.ycen+2*self.uspan)
          else:
             self.xborder = 0.0
             self.yborder = 0.0
             keep = ones(refny.shape)
          self.refid = [refdat[0][i] for i in nonzero(keep)]
          self.refmag  = array([refdat[2][i] for i in nonzero(keep)])
          self.refnx  = array([refnx[i] for i in nonzero(keep)])
          self.refny  = array([refny[i] for i in nonzero(keep)])
          self.refdat = [self.refid,self.refnx,self.refny,self.refmag]
          self.refmatch = matching(self.refdat,sky=1,cull=0,nuse=self.nuse)
       self.catmatch = matching(self.catdat,nuse=self.nuse,cull=clean,errlim=0.2)
    def touchup(self,verb=0,mtol=1.0,clean=0):
       if self.zero:
          if verb: print "Before"
          if verb: print self.tran
          self.tran,self.qual = self.catmatch.xfm(self,tran=None,verb=0,nord=1,mtol=mtol,arcsec=0)
          if verb: print "After"
          if verb: print self.tran
       else:
          self.catmatch = matching(self.catdat,sky=0,cull=clean,verb=0,nuse=None)
          self.refmatch = matching(self.refdat,sky=1,cull=0,verb=0,nuse=None)
          self.fixsol,self.qual = self.catmatch.xfm(self,verb=0,nord=0,mtol=mtol,arcsec=1)
          self.rotsol = rotsol(self.fixsol)
          self.fix()
    def matchup(self,verb=0,mtol=1,ttol=1,gangle=0.0):
       self.fixsol,self.matches = self.catmatch.match(1,self.refmatch,verb=verb,im=self.image,mtol=mtol,ttol=ttol,gangle=gangle)
       if self.zero:
          ix = take(self.imx,self.matches[0])
          iy = take(self.imy,self.matches[0])
          rx = (pi/180)*take(self.refnx,self.matches[1])/3600
          ry = (pi/180)*take(self.refny,self.matches[1])/3600
          b = abasis(1,ix-self.xc,iy-self.yc,rot=1)
          for j in range(1):
             f = add.reduce(self.tran[NewAxis,::]*b,1)
             fx = f[:len(ix)]; fy = f[len(ix):]
             b = abasis(1,ix-self.xc,iy-self.yc,rot=1)
             y = concatenate([rx,ry])
             self.tran = svdfit(b,y)
             if j: self.tran = planefit(b,y,self.tran,minfunc=mad)[0]
             f = add.reduce(self.tran[NewAxis,::]*b,1)
             fx = f[:len(ix)]; fy = f[len(ix):]
          if verb: print multiply(bwt(fx-rx),(3600*180/pi)),
          if verb: print multiply(bwt(fy-ry),(3600*180/pi)),len(ix)
       else:
          self.rotsol = rotsol(self.fixsol)
    def fix(self,verb=0):
        if verb: print "Old:",self.tran
        self.xsh = degrad(self.rotsol.xsh/3600)
        self.ysh = degrad(self.rotsol.ysh/3600)
        self.cos = self.rotsol.cos
        self.sin = self.rotsol.sin
        self.rescale = self.rotsol.rescale
        self.rac,self.decc = equicoord(-self.xsh,-self.ysh,self.rac,self.decc)
        self.center(raddeg(self.rac),raddeg(self.decc))
        self.tranx, self.trany = self.tranx*self.cos - self.trany*self.sin,\
                self.trany*self.cos + self.tranx*self.sin
        self.tranx, self.trany = self.tranx*self.rescale, self.trany*self.rescale
        if verb: print "New:",self.tran

class rotsol:
   def __init__(self,fixsol=None,fac=1.0,sfac=1.0):
      if fixsol:
         self.rot = pi/2 - apply(atan2,fixsol[2:])
         if fac: self.rot = fac*self.rot
         self.xsh,self.ysh = sfac*fixsol[:2]
         self.cos = cos(self.rot)
         self.sin = sin(self.rot)
         self.rescale = 1/sqrt(sum(fixsol[2:]**2))
         if fac < 0: self.rescale = 1/self.rescale 
         print "Shifts, dPA, Scale = (%.2f,%.2f) (arcsec), %.3f (deg), %.5f " % (self.xsh,self.ysh,raddeg(self.rot),self.rescale)

class mosaic:
   def __init__(self,q,nuse=1):
       self.q = q
       self.name = q[0].name
       self.nn = concatenate([c.chip*1000+c.catmatch.n for c in q])
       self.nx = concatenate([c.catmatch.x for c in q])
       self.ny = concatenate([c.catmatch.y for c in q])
       self.nm = concatenate([c.catmatch.m for c in q])
       self.ne = concatenate([c.catmatch.e for c in q])
       self.nf = zeros(self.ne.shape)
       if q[0].all:
          self.refnn = q[0].refmatch.n
          self.refnx = q[0].refmatch.x
          self.refny = q[0].refmatch.y
          self.refnm = q[0].refmatch.m
          self.refne = q[0].refmatch.e
       else:
          self.refnn = concatenate([c.chip*1000+c.refmatch.n for c in q])
          self.refnx = concatenate([c.refmatch.x for c in q])
          self.refny = concatenate([c.refmatch.y for c in q])
          self.refnm = concatenate([c.refmatch.m for c in q])
          self.refne = concatenate([c.refmatch.e for c in q])
       self.refnf = zeros(self.refne.shape)
       xspan = diff(extrema(self.refnx))[0]
       yspan = diff(extrema(self.refny))[0]
       opendev(getpgdev()[0],land=1,size=float(getpgdev()[1]),nx=1,ny=1)
       newbox([max(self.refnx)+xspan/4,min(self.refnx)-xspan/4],\
               [min(self.refny)-yspan/4,max(self.refny)+yspan/4],ch=2)
       subtitle(q[0].name,ch=2.0)
       drawpt(self.refnx,self.refny,ch=3,ps=17,ci=4)
       drawpt(self.nx,self.ny,ch=2,ps=17,ci=1)
       xlabel("X",ch=2.0)
       ylabel("Y",ch=2.0)
       closedev()
       if nuse: self.nuse = nuse
       else: self.nuse = len(self.nf)
       self.refdat = [ self.refnn,self.refnx,self.refny,self.refnm]
       self.catdat = [ self.nn,self.nx,self.ny,self.nm]
       print "Have %d sky objects and %d image objects." % (len(self.refnn),len(self.nn))
   def __repr__(self):
       return "mosaic (w/%s)" % (self.name)
   def touchup(self,verb=0,mtol=1.0,fixpa=0):
       self.catmatch = matching(self.catdat,sky=1,cull=0,verb=0,nuse=0)
       self.refmatch = matching(self.refdat,sky=1,cull=0,verb=0,nuse=0)
       self.fixsol,self.qual = self.catmatch.xfm(self,verb=verb,mtol=mtol,nord=0,arcsec=1,fixpa=fixpa,fixscale=0)
       print self.qual
       self.rotsol = rotsol(self.fixsol,fac=-1.0,sfac=-1.0)
   def matchup(self,verb=0,mtol=1.0,ttol=1.0,gangle=0.0,fixpa=0):
       self.catmatch = matching(self.catdat,sky=1,cull=0,verb=0,nuse=self.nuse)
       self.refmatch = matching(self.refdat,sky=1,cull=0,verb=0,nuse=self.nuse)
       print "Matching up using",self.nuse
       self.fixsol,self.matches = self.catmatch.match(1,self.refmatch,verb=verb,im="",mtol=mtol,ttol=ttol,gangle=gangle,fixpa=fixpa)
       if verb: print self.fixsol
       self.rotsol = rotsol(self.fixsol,fac=1.0,sfac=1.0)
   def adjust(self,angle,deltax,deltay,rescale=1):
       n = len(self.q); y = range(n)
       [self.q[i].read(verb=0) for i in y]
       print "Adjusting %s by angle,DX,DY,scale = %.3f, %.3f, %.3f %.5f" % (self,angle,deltax,deltay,rescale)
       mq = rotsol()
       mq.xsh = deltax
       mq.ysh = deltay
       mq.rot = degrad(angle)
       mq.cos = cos(mq.rot)
       mq.sin = sin(mq.rot)
       mq.rescale = rescale
       for i in y: self.q[i].rotsol = mq; self.q[i].fix()
       [self.q[i].write() for i in y]


class matching:
   def __init__(self,d,nuse=50,errlim=0.2,cull=1,sky=0,verb=0):
       if sky:
          n, x, y, m= d
          e = zeros(x.shape)
          fl = zeros(x.shape)
          u = arange(len(n))
          n = arange(len(n))
          un = n
          ux = x
          uy = y
          um = m
          ue = e
          uf = 1*e
       else:
          if len(d) == 4:
            n, x, y, m = d
            e = sqrt(m)/m/1.086
            m = -2.5*log10(m) + 25
            ma = m
            ea = e
            fl = 0
          else:
            n, x, y, m, e, ma, ea, fl = d[:8]
          n = arange(len(x))
          u = arange(len(n))

       if cull:
          mam0a,mam0b = bwt(ma-m)
          mam1 = mam0a - 3*mam0b
          mam2 = mam0a + 3*mam0b
          u = less(ea,errlim)
          u = u*between(ma-m,mam1,mam2,inc=1)
#          u = u*not_equal(fl,4)*less(fl,30)
          u = u*less(fl,30)
          u = nonzero(u)
          n = take(n,u)
          x,y,m,e,ma,ea,fl = take([x,y,m,e,ma,ea,fl],u,1)
          un = n; ux,uy,um,ue = x,y,m,e
#          un = take(n,u); ux,uy,um,ue = take([x,y,m,e],u,1)
          print "Culling",len(un),
          if nuse:
              a = argsort(m)[:nuse]
              n = take(n,a)
              x,y,m,e,ma,ea,fl = take([x,y,m,e,ma,ea,fl],a,1)
              u = arange(len(x))
              un = n; ux,uy,um,ue = x,y,m,e
              print "and using",len(un)
          else: print
       else:
          a = argsort(m)
          if nuse: a = a[:nuse]
          n = take(n,a)
          x,y,m,e, = take([x,y,m,e],a,1)
          un = n; ux,uy,um,ue = x,y,m,e
          u = arange(len(n))
       if nuse:             
           if verb: print "There are",len(n),"reference objects."

           id = []
           n1 = []; x1 = []; y1 = []; m1 = []; e1 = []; s1 = []; t1=[]
           dx1 = []; dy1 = []
           for j in u:
              n2 = []; x2 = []; y2 = []; m2 = []; e2 = []; s2 = []; t2=[]
              dx2 = []; dy2 = []
              for k in u:
                 dx = x[j] - x[k]
                 dy = y[j] - y[k]
                 ds = sqrt(dx**2+dy**2)
                 dt = raddeg(atan2(dy,dx)-pi/2) % (360.0)
                 if dt > 180: dt = dt - 360
                 n2.append( n[k]); x2.append( x[k]); y2.append( y[k])
                 m2.append( m[k]); e2.append( e[k])
                 s2.append( ds); t2.append( dt)
                 dx2.append(dx); dy2.append(dy)
              n2 = asarray(n2,Int32)
              x2,y2,m2,e2,s2,t2,dx2,dy2 = asarray([x2,y2,m2,e2,s2,t2,dx2,dy2])
              o2 = argsort(s2)
              n2 = take(n2,o2)
              x2,y2,m2,e2,s2,t2,dx2,dy2 = take([x2,y2,m2,e2,s2,t2,dx2,dy2],o2,1)
              id.append(n[j])
              n1.append(n2); x1.append(x2); y1.append(y2)
              m1.append(m2); e1.append(e2)
              s1.append(s2); t1.append(t2)
              dx1.append(dx2); dy1.append(dy2)
           id = asarray(id,Int32)
           n1 = asarray(n1,Int32)
           x1,y1,m1,e1,s1,t1 = asarray([x1,y1,m1,e1,s1,t1])
           self.id = id; self.n1 = n1; self.x1 = x1; self.y1 = y1
           self.m1 = m1; self.e1 = e1; self.s1 = s1; self.t1 = t1
           self.dx1 = dx1; self.dy1 = dy1
       self.n = un; self.x = ux; self.y = uy
       self.m = um; self.e = ue

   def match(self,Jim,other,verb=0,im=None,mtol=1.0,ttol=1.0,gangle=0.0,fixpa=0):
       sys.stdout.flush()
       print "Entering matching algorithm:",verb
       J1,J2=len(self.n1),len(other.n1)
       b = []; n = []; x = []; y = []
       for j1 in range(J1):
          if len(n) == J2: break
          print j1,
          if j1 and not j1%25: print
          sys.stdout.flush()
          possx = []
          possy = []
          possn = []
          possb = []
          possm = []
          for j2 in range(J2):
            if j2 not in b:
               ts = (self.s1[j1][::,NewAxis]-other.s1[j2][NewAxis,::])
               ss = less(abs(ts),mtol)
               ss[0,0] = 0
               if verb == 3:
                  print
                  print j1,j2
                  for temp in ts: print temp.astype(Int32)
                  for temp in ss: print temp.astype(Int32)
#               if sum(ss.flat) >= min([J1,J2])/4:
               if sum(ss.flat) > 0:
                   tt = self.t1[j1][::,NewAxis]-other.t1[j2][NewAxis,::]+gangle
                   tt1 = arctan2(self.dy1[j1], self.dx1[j1])
                   ctt1 = cos(tt1+gangle)
                   stt1 = sin(tt1+gangle)
                   tt2 = arctan2(other.dy1[j2], other.dx1[j2])
                   ctt2 = cos(tt2)
                   stt2 = sin(tt2)
                   st1 = less(abs(ctt1[::,NewAxis]-ctt2[NewAxis,::]),ttol*pi/180)
                   st2 = less(abs(stt1[::,NewAxis]-stt2[NewAxis,::]),ttol*pi/180)
#                   st = st1*st2
                   st = less(abs(tt),ttol)
                   st[0,0] = 0
                   mat = (ss*st)
                   tmat = sum(mat.flat)
                   if verb == 3:
                      for temp in tt: print temp.astype(Int32)
                      for temp in st: print temp.astype(Int32)
                      print
                      for temp in mat: print temp
                      print tmat
                   if tmat > 3:
                      if verb > 1:
                          print j1,j2,tmat
                          spaces = compress(mat.flat,ts.flat)
                          thetas = compress(mat.flat,tt.flat)
                          print spaces
                          print thetas
                          print
                      possx.append(self.x[j1])
                      possy.append(self.y[j1])
                      possn.append(self.n[j1])
                      possb.append(j2)
                      possm.append(tmat)
                      break
          if len(possm):
            if verb > 1:
               print j1, possb
            use = argmax(possm)
            x.append(possx[use])
            y.append(possy[use])
            b.append(possb[use])
            n.append(possn[use])
       x,y = asarray([x,y])
       print
       if verb: print n
       noth = take(other.n,b)
       xo = take(other.x,b)
       yo = take(other.y,b)
       if verb: print x-xo
       if verb: print y-yo
       ixo = (3*((x-xo)/3.0+0.5).astype(Int32)).tolist()
       iyo = (3*((y-yo)/3.0+0.5).astype(Int32)).tolist()
       nxo = array([ixo.count(ix) for ix in ixo])
       nyo = array([iyo.count(iy) for iy in iyo])
       no = nxo*nyo
       print ixo
       print nxo
       print iyo
       print nyo
       ao = argmax(no)
       bx = ixo[ao]; by = iyo[ao]
       sx = 10.0; sy = 10.0
       if verb: print bx,sx
       if verb: print by,sy
       usex = between(x-xo,bx-4*sx,bx+4*sy)
       usey = between(y-yo,by-4*sy,by+4*sy)
       if verb: print usex*usey
       xo,yo,x,y = compress(usex*usey,[xo,yo,x,y],1)
       n = compress(usex*usey,n,0)
       if verb: print n
       if verb: print xo-x
       if verb: print yo-y
       bx,sx = bwt(x-xo)
       by,sy = bwt(y-yo)
       if verb: print bx,sx
       if verb: print by,sy
       usex = between(x-xo,bx-8*sx,bx+8*sy)
       usey = between(y-yo,by-8*sy,by+8*sy)
       if verb: print usex*usey
       xo,yo,x,y = compress(usex*usey,[xo,yo,x,y],1)
       n = compress(usex*usey,n,0)
       if verb: print n
       if verb: print xo-x
       if verb: print yo-y
       print "Matched:",len(xo)
       bx = bwt(x-xo)[0]
       by = bwt(y-yo)[0]
#       rt = array([(atan2(y[j]-mean(y),x[j]-mean(x))-atan2(yo[j]-mean(yo),xo[j]-mean(xo))) for j in range(len(x))])
#       bt = [mean(arctan2(y-mean(y),x-mean(x))-arctan2(yo-mean(yo),xo-mean(xo))),
#               rms(arctan2(y-mean(y),x-mean(x))-arctan2(yo-mean(yo),xo-mean(xo)))]
       bt = bwt(arctan2(y-mean(y),x-mean(x))-arctan2(yo-mean(yo),xo-mean(xo)))
       print bt,tan(bt[0])
       basis = abasis(0,xo,yo,rot=1)
       dat = concatenate([x,y])
       sol = svdfit(basis,dat)
       sol = array([bx,by,sqrt(1.0-bt[0]**2),-bt[0]])
       if fixpa:
         sol = array([bx,by,1.0,0.0])
#         sol = planefit(basis,dat,sol,minfunc=mad,verb=1)[0]
       f = add.reduce(sol[NewAxis,::]*basis,1)
       fx = f[:len(x)]; fy = f[len(x):]
       print rms(fx-x), rms(fy-y)
       print bwt(fx-x), bwt(fy-y)
       return sol,[n,noth]

   def xfm(self,wirc,tran=[0,0,1,0],verb=0,mtol=3.0,nord=1,arcsec=0,fixpa=0,fixscale=1):
      other = wirc.refmatch
      if tran: tran = array(tran)
      else: tran = wirc.tran
      if arcsec:
         fac = 1.0; xc = 0.0; yc = 0.0
         aix = self.x; aiy = self.y
         arx = other.x; ary = other.y
         rot = 1
      else:
         fac = (3600*180/pi)
         xc = wirc.xc; yc = wirc.yc
         aix = wirc.imx - xc
         aiy = wirc.imy - yc
         arx = other.x; ary = other.y
         rot = 0
      for j in range(1):
          rx = []; ry = []; ix = []; iy = []; fx = []; fy = []
          b = abasis(nord,aix,aiy,rot=rot)
          f = add.reduce(tran[NewAxis,::]*b,1)*fac
          afx = f[:len(aix)]; afy = f[len(aix):]
          for k in range(len(aix)):
             dx = power(afx[k]-arx,2)
             dy = power(afy[k]-ary,2)
             ds = sqrt(dx+dy)
             l = argmin(ds)
             if ds[l] < mtol:
                ix += [aix[k]]; iy += [aiy[k]]
                fx += [afx[k]]; fy += [afy[k]]
                rx += [arx[l]]; ry += [ary[l]]
          ix,iy,rx,ry,fx,fy = asarray([ix,iy,rx,ry,fx,fy])
          mx,dx = bwt(fx-rx)
          my,dy = bwt(fy-ry)
          if verb: print mx,dx,mx,dy,len(fx)
          dx = max([dx,mtol/4.0])
          dy = max([dy,mtol/4.0])
          sel = between(fx-rx,mx-4*dx,mx+4*dx)*between(fy-ry,my-4*dy,my+4*dy)
          ix = compress(sel,ix,0)
          iy = compress(sel,iy,0)
          rx = compress(sel,rx,0)
          ry = compress(sel,ry,0)

          bx = bwt(rx-ix)[0]
          by = bwt(ry-iy)[0]
          if fixpa:
            bt = [0.0,1.0]
          else:
            rt = array([(atan2(ry[j]-mean(ry),rx[j]-mean(rx))-atan2(iy[j]-mean(iy),ix[j]-mean(ix))) for j in range(len(ix))])
            bt = bwt(rt)
          tran = array([bx,by,sqrt(1.0-bt[0]**2),-bt[0]])

          b = abasis(nord,ix,iy,rot=rot)
          if not fixscale:
             y = concatenate([rx,ry])
             tran = svdfit(b,y)
#             tran = planefit(b,y,tran,minfunc=mad)[0]
          tran = tran/fac
          f = add.reduce(tran[NewAxis,::]*b,1)*fac
      fx = f[:len(ix)]; fy = f[len(ix):]
      mx,dx = bwt(fx-rx)
      my,dy = bwt(fy-ry)
      if fixpa:
         tran = array([bx,by,1.0,0.0])
      print "Sigmas (x,y)=(%.4f,%.4f) [%d]" % (dx,dy,len(fx))
      return tran,[dx,dy,len(fx)]


