#!/usr/bin/env python

from bisect import *
import Numeric, Multipack
from scipy.special import erfc
from cmath import *
from umath import *
from basics import GRAV, CMPPC, CMPKPC, CMPMPC, MSUN, SUN_VMAG, SUN_BV
from basics import tlook, angsize, CLIGHTK, mpc2mod, average

M = Multipack
N = Numeric

Inf = M.Inf
upperlim = Inf

def erf(x):
   if x: return 1.0-erfc(x)
   else: return 0

def dens_iso(x,y0):
   if x>0: return 9*exp(x-y0)/(4*pi)
   else: return 0.0

def odes_iso( y, t, y0):
   val = N.zeros( (3,), N.Float32)
   val[0] = y[1]
   if t: val[1] = -2*y[1]/t - 4*pi*dens_iso(y[0],y0)
   else: val[1] = 0
   val[2] = 4*pi*t*t*9*exp(y[0]-y0)
   return val

def getiso(ra):
   w0 = 1E8
   an=M.odeint( odes_iso, [w0, 0.0, 0.0], ra, args=(w0,))[0]
   an0=N.transpose(an)[0]
   an1=N.transpose(an)[1]
   an2=N.transpose(an)[2]
   den = N.array(map(dens_iso, an0, w0*N.ones(an0.shape)))
   mr = N.array(an2)
   return den, mr

def mk_iso(sigma, rc_iso, ra):
   rhoc_iso = 9*pow(sigma/rc_iso,2)/(4*pi*GRAV)
   rhoc_iso = pow(sigma/rc_iso,2)
   den_iso, mr_iso = getiso(ra)
   gk = N.greater(den_iso, 0.0)
   ira = N.compress(gk, ra)
   iden = N.compress(gk, den_iso)
   imr = N.compress(gk, mr_iso)
   ira = ira * rc_iso
   iden = iden * rhoc_iso
   imr = imr * rhoc_iso * pow(rc_iso,3)
   return ira, iden, imr

def density131(x,y0):
   if x>0: return exp(x-y0)*erf(sqrt(x))-sqrt(4*x/pi)*(1+2*x/3)*exp(-y0)
   else: return 0.0

def odes_kingmod( y, t, y0):
   val = N.zeros( (3,), N.Float32)
   val[0] = y[1]
   if t: val[1] = -2*y[1]/t - 9*density131(y[0],y0)
   else: val[1] = 0
   val[2] = 4*pi*t*t*density131(y[0],y0)
   return val

def getking(w0, ra):
   an=M.odeint( odes_kingmod, [w0, 0.0, 0.0], ra, args=(w0,))[0]
   an0=N.transpose(an)[0]
   an1=N.transpose(an)[1]
   an2=N.transpose(an)[2]
   den = N.array(map(density131, an0, w0*N.ones(an0.shape)))
   mr = N.array(an2)
   return den, mr

def mk_king(w0, sigma, rc_king, ra):
   rhoc_king = 9*pow(sigma/rc_king,2)/(4*pi*GRAV)
#   rhoc_king = rhoc_king * 9 / (4*pi)
   den_king, mr_king = getking(w0, ra)
   gk = N.greater(den_king, 0.0)
   kra = N.compress(gk, ra)
   kden = N.compress(gk, den_king)
   kmr = N.compress(gk, mr_king)
   kra = kra * rc_king
   kden = kden * rhoc_king
   kmr = kmr * rhoc_king * pow(rc_king,3)
   return kra, kden, kmr

def oldnu(x,ra,dens):
   if x < ra[-1]:
      i = bisect(ra, x)
      r0 = ra[i-1]; r1 = ra[i]
      l0 = dens[i-1]; l1 = dens[i]
      if x and r0 and r1 and l0 and l1: onu = pow(10,log10(l0) + log10(x/r0)*log10(l1/l0)/log10(r1/r0))
      else: onu = l0 + (x-r0)*(l1-l0)/(r1-r0)
   else: onu = 0.0
   return onu

def oldMass(x,ra,ms):
   if x < ra[-1]:
      i = bisect(ra, x)
      r0 = ra[i-1]; r1 = ra[i]
      l0 = ms[i-1]; l1 = ms[i]
      if x and r0 and r1 and l0 and l1:
        try: omr = pow(10,log10(l0) + log10(x/r0)*log10(l1/l0)/log10(r1/r0))
        except: omr = l0 + (x-r0)*(l1-l0)/(r1-r0)
      else: omr = l0 + (x-r0)*(l1-l0)/(r1-r0)
   else: omr = ms[-1]
   return omr

def nu(x,dtck): return pow(10,M.splev(log10(x),dtck))

def Mass(x,mtck): return pow(10,M.splev(log10(x),mtck))

def sb_int(x,R,dtck):
   return 2*nu(x,dtck)*x/sqrt(x*x-R*R)

def intsb(r, dtck, epsabs=1.5E-4,epsrel=1.5E-4):
   return M.quad(sb_int, r, upperlim, args=(r,dtck), epsabs=epsabs, epsrel=epsrel, limit=1000)[0]

