
import types
import sys
import os
import MLab
import copy
from glob import glob
from masks import *
from basics import *
from pgplot import *
from FITS import *
from vtk import *
from VTKHelperFunctions import *
from mimetypes import guess_type


ROWPLOT = "ROWMODE"
COLPLOT = "COLMODE"

class LaunchPFITS:
   def __init__(self,imageinteractor,row=-1,col=-1,plotrange=None,linelist=None):
      self.numarray = imageinteractor.numarray
      self.naxis = len(imageinteractor.numarray.shape)
      self.pic = imageinteractor.pic
      self.header = imageinteractor.header
      self.crval1 = imageinteractor.crval1
      self.crval2 = imageinteractor.crval2
      self.cdelt1 = imageinteractor.cdelt1
      self.cdelt2 = imageinteractor.cdelt2
      self.crpix1 = imageinteractor.crpix1
      self.crpix2 = imageinteractor.crpix2
      self.dcflag = imageinteractor.dcflag
      self.redsh = imageinteractor.redsh
      if row >=0: self.mode = ROWPLOT; self.row = row
      if col >=0: self.mode = COLPLOT; self.col = col
      self.range = plotrange
      self.linelist = linelist
      self.setup()
      self.execute()
   def execute(self):
      inlist = self.inlist
      fits = [self.fits]
      s = len(sys.argv)
      showf = 0
      xi = self.xi
      yi = self.yi
      if self.range: xrs = ["-xr", join([str(self.range[0]), str(self.range[1])])]
      if self.linelist: lrs = ["-l",self.linelist]
      sys.argv = sys.argv + lrs + xrs
      if self.redsh: sys.argv += ["-z",str(self.redsh)]
      command = self.usepfits
      pchild = os.fork()
      if pchild == 0: execfile(command)
      del sys.argv[s:]
   def setup(self):
      trydirs=split(os.environ['PYTHONPATH'],":")
      pfitsdir=trydirs[nonzero(map(lambda d: "pfits" in os.listdir(d), trydirs))[0]]
      if self.mode == ROWPLOT:
           self.inlist = ["FORKED/ROW"]
           self.xi = 0
           self.yi = self.row
      if self.mode == COLPLOT:
           self.inlist = ["FORKED/COL"]
           self.xi = self.col
           self.yi = 0
      self.usepfits = pfitsdir+"/pfits"
      self.fits={}
      self.fits["input"] = self.pic
      self.fits["data"] = self.numarray
      self.fits["naxes"] = list(self.numarray.shape); self.fits["naxes"].reverse()
      self.fits["header"] = self.header
      self.fits["dcflag"] = self.dcflag
      self.fits["crval1"] = self.crval1
      self.fits["crval2"] = self.crval2
      self.fits["cdelt1"] = self.cdelt1
      self.fits["cdelt2"] = self.cdelt2
      self.fits["crpix1"] = self.crpix1
      self.fits["crpix2"] = self.crpix2


